/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifndef SPH_IFS_KEYWORDS_H_
#define SPH_IFS_KEYWORDS_H_
/*---------------------------------------------------------------------------*/
/*
 * The keywords and corresponding allowed values for IFS are
 * defined in this file.
 * Keywords and values common to all SPHERE instruments are
 * defined in the common API under src/Sphere/sph_common_keywords.h
 */
/*---------------------------------------------------------------------------*/

#define SPH_IFS_KEYWORD_CORONO_SETTING					       "ESO DRS IFS CORONO"

#define SPH_IFS_KEYWORD_LAMP1_ID                    "ESO INS2 LAMP1 ID"
#define SPH_IFS_KEYWORD_LAMP2_ID                    "ESO INS2 LAMP2 ID"
#define SPH_IFS_KEYWORD_LAMP3_ID                    "ESO INS2 LAMP3 ID"
#define SPH_IFS_KEYWORD_LAMP4_ID                    "ESO INS2 LAMP4 ID"
#define SPH_IFS_KEYWORD_LAMP5_ID                    "ESO INS2 LAMP5 ID"
#define SPH_IFS_KEYWORD_LAMP6_ID                    "ESO INS2 LAMP6 ID"
#define SPH_IFS_KEYWORD_LAMP1_SWITCH                "ESO INS2 LAMP1 ST"
#define SPH_IFS_KEYWORD_LAMP2_SWITCH                "ESO INS2 LAMP2 ST"
#define SPH_IFS_KEYWORD_LAMP3_SWITCH                "ESO INS2 LAMP3 ST"
#define SPH_IFS_KEYWORD_LAMP4_SWITCH                "ESO INS2 LAMP4 ST"
#define SPH_IFS_KEYWORD_LAMP5_SWITCH                "ESO INS2 LAMP5 ST"
#define SPH_IFS_KEYWORD_LAMP6_SWITCH                "ESO INS2 LAMP6 ST"

//descr Number of spectra found
//comment:  Number of spectra found
#define SPH_IFS_KEYWORD_SPECPOS_QC_NREGS                      "ESO QC NUMBER SPECTRA"
//descr Threshold used for spectra detection
//comment: Threshold used for spectra detection
#define SPH_IFS_KEYWORD_SPECPOS_QC_THRESHOLD                  "ESO QC THRESHOLD USED"
//descr Number of pixels illuminated outside spectra
//comment: Number of pixels illuminated outside spectra
#define SPH_IFS_KEYWORD_SPECPOS_QC_PIXS_OUT                   "ESO DRS NUM ILLUM PIXELS OUT"
//descr Number of pixels illuminated inside spectra
//comment: Number of pixels illuminated inside spectra
#define SPH_IFS_KEYWORD_SPECPOS_QC_PIXS_IN                    "ESO DRS NUM ILLUM PIXELS IN"
//descr Number of dark pixels inside spectra
//comment: Number of dark pixels inside spectra
#define SPH_IFS_KEYWORD_SPECPOS_QC_DARK_PIXS_IN               "ESO DRS NUM DARK PIXELS IN"
//descr Scale of specpos model as measured
//comment: Scale of specpos model as measured
#define SPH_IFS_KEYWORD_SPECPOS_QC_SCALE                      "ESO QC SCALE MEASURED"
//descr X offset of specpos model as measured
//comment: X offset of specpos model as measured
#define SPH_IFS_KEYWORD_SPECPOS_QC_OFFSETX                    "ESO DRS OFFSETX MEASURED"
//descr Y offset of specpos model as measured
//comment: Y offset of specpos model as measured
#define SPH_IFS_KEYWORD_SPECPOS_QC_OFFSETY                    "ESO DRS OFFSETY MEASURED"

//descr Smoothing length applied to BG
//comment:Smooting length applied to BG
#define SPH_IFS_KEYWORD_CAL_BG_QC_SMOOTHING_LENGTH            "ESO DRS BG SMOOTHING LENGTH"
//descr Background mean value
//comment: beckground mean value
#define SPH_IFS_KEYWORD_CAL_BG_QC_MEAN                        "ESO QC BACKGROUND MEAN"
//descr Bckground RMS
//comment: Background RMS
#define SPH_IFS_KEYWORD_CAL_BG_QC_RMS                         "ESO QC BACKGROUND RMS"
//descr Background median value
//comment: Background median value
#define SPH_IFS_KEYWORD_CAL_BG_QC_MEDIAN                      "ESO QC BACKGROUND MEDIAN"

//descr Bench temperature
//comment: Bench temperature
#define SPH_IFS_KEYWORD_BENCH_TEMP                            "ESO INS2 TEMP4 VAL"

//descr Mean value of preamp correlation
//comment: Mean value of preamp correlation
#define SPH_IFS_KEYWORD_PREAMPCORR_MEAN                        "ESO QC PREAMP CORR MEAN"
//descr Median value of preamp correlation
//comment: Median value of preamp correlation
#define SPH_IFS_KEYWORD_PREAMPCORR_MEDIAN                      "ESO QC PREAMP CORR MEDIAN"
//descr RMS of preamp correlation
//comment: RMS of preamp correlation
#define SPH_IFS_KEYWORD_PREAMPCORR_RMS                         "ESO QC PREAMP CORR RMS"

//descr Number of points observed for distortion map
//comment: Number of points observed for distortion map
#define SPH_IFS_KEYWORD_DISTMAP_NPOINTS_OBS             "ESO QC DISTMAP NPOINTS OBS"

//descr Number of points used for distortion map
//comment: Number of points used for distortion map
#define SPH_IFS_KEYWORD_DISTMAP_NPOINTS_IN              "ESO DRS DISTMAP NPOINTS IN"
//descr X position of optical axis for distortion map
//comment: X position of optical axis for distortion map
#define SPH_IFS_KEYWORD_DISTMAP_OPTICAL_AXIS_X          "ESO DRS DISTMAP OPT AXIS X"
//descr Y position of optical axis for distortion map
//comment: Y position of optical axis for distortion map
#define SPH_IFS_KEYWORD_DISTMAP_OPTICAL_AXIS_Y          "ESO DRS DISTMAP OPT AXIS Y"

#define SPH_IFS_KEYWORD_ROT_ANGLE                       "ESO DRS IFS ROTANGLE"

#define SPH_IFS_KEYWORD_VALUE_CORONO_SETTING_OUT	           "OUT"
#define SPH_IFS_KEYWORD_VALUE_CORONO_SETTING_IN		           "IN"

#define SPH_IFS_KEYWORD_VALUE_DPR_TYPE_DARK                   "DARK"
#define SPH_IFS_KEYWORD_VALUE_DPR_TECH_DARK                   "IMAGE"
#define SPH_IFS_KEYWORD_VALUE_DPR_CATG_DARK                   "CALIB"

#define SPH_IFS_KEYWORD_VALUE_DPR_TYPE_DFF                    "FLAT,LAMP"
#define SPH_IFS_KEYWORD_VALUE_DPR_TECH_DFF                    "IMAGE"
#define SPH_IFS_KEYWORD_VALUE_DPR_CATG_DFF                    "CALIB"

#define SPH_IFS_KEYWORD_VALUE_DPR_TYPE_IFSFLAT                "FLAT,LAMP"
#define SPH_IFS_KEYWORD_VALUE_DPR_TECH_IFSFLAT                "IFU"
#define SPH_IFS_KEYWORD_VALUE_DPR_CATG_IFSFLAT                "CALIB"

#define SPH_IFS_KEYWORD_VALUE_DPR_TYPE_WAVECALIB              "LAMP,WAVE"
#define SPH_IFS_KEYWORD_VALUE_DPR_TECH_WAVECALIB              "IFU"
#define SPH_IFS_KEYWORD_VALUE_DPR_CATG_WAVECALIB              "CALIB"

#define SPH_IFS_KEYWORD_VALUE_DPR_TYPE_RON              "DARK"
#define SPH_IFS_KEYWORD_VALUE_DPR_TECH_RON              "IMAGE"
#define SPH_IFS_KEYWORD_VALUE_DPR_CATG_RON              "CALIB"


#define SPH_IFS_KEYWORD_VALUE_PRO_CATG_PIXEL_LINEARITY        "PIXEL_LINEARITY_MAP"
#define SPH_IFS_KEYWORD_VALUE_PRO_CATG_PIXEL_DESCRIPTION_TAB  "PIXEL_DESCRIPTION_TABLE"

#define SPH_IFS_KEYWORD_VALUE_PRO_CATG_MASTER_DARK            "IFS_MASTER_DARK"
#define SPH_IFS_KEYWORD_VALUE_PRO_CATG_RON            		   "IFS_RON"
#define SPH_IFS_KEYWORD_VALUE_PRO_CATG_MASTER_DFF             "IFS_MASTER_DFF"
#define SPH_IFS_KEYWORD_VALUE_PRO_CATG_MASTER_FLAT            "IFS_MASTER_FLAT"
#define SPH_IFS_KEYWORD_VALUE_PRO_CATG_STATIC_BADPIXELMAP     "IFS_STATIC_BADPIXELMAP"
#define SPH_IFS_KEYWORD_VALUE_PRO_CATG_LINEAR_BADPIXELMAP     "IFS_LINEAR_BADPIXELMAP"
#define SPH_IFS_KEYWORD_VALUE_PRO_CATG_PDT_MASK               "PIXEL_DESCRIPTION_TABLE"
#define SPH_IFS_KEYWORD_VALUE_PRO_CATG_FFLAMP_SPECTRUM        "FFLAMP_SPECTRUM"
#define SPH_IFS_KEYWORD_VALUE_PRO_CATG_SCIENCE_DR        		"SCIENCE_DR"




/* sph_ifs_master_dark */

#endif /* SPH_IFS_KEYWORDS_H_ */
