/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <cpl.h>
#include "sph_error.h"
#include "sph_ifs_proc_apply_master_dark.h"
#include <math.h>
#include "sph_common_keywords.h"

static const double SPH_IFS_PROC_APPLY_MASTER_DARK_TOL = 0.001;
static const double SPH_IFS_PROC_APPLY_MASTER_DARK_MIN_SCALED_DIT = 2.0;

/*----------------------------------------------------------------------------*/
/**
 * @defgroup A SPHERE API Module
 * @par Synopsis:
 * @code
 * typedef _module_ {
 * } module
 * @endcode
 * @par Desciption:
 *
 * This module provides functionality for apertures, extending the functionality
 * as it exists for cpl_apertures.
 */
/*----------------------------------------------------------------------------*/
/**@{*/
sph_error_code
sph_ifs_proc_apply_master_dark( sph_master_frame* mframe, sph_master_frame* mdark )
{
    int                 keysok = 0;
    double              dit_dark = 0.0;
    double              dit_frame = 0.0;
    sph_master_frame*   tmpdark = NULL;
    cpl_ensure_code(mframe, CPL_ERROR_NULL_INPUT);
    cpl_ensure_code(mdark, CPL_ERROR_NULL_INPUT);
    if ( mdark->properties && mframe->properties ) {
        if ( cpl_propertylist_has( mdark->properties, SPH_COMMON_KEYWORD_DIT )
                && cpl_propertylist_has( mframe->properties, SPH_COMMON_KEYWORD_DIT ) ) {
            keysok = 1;
            dit_dark =  cpl_propertylist_get_double( mdark->properties, SPH_COMMON_KEYWORD_DIT );
            dit_frame =  cpl_propertylist_get_double( mframe->properties, SPH_COMMON_KEYWORD_DIT );
        }
    }
    if ( keysok == 1 &&
            ( fabs(dit_dark - dit_frame) >
                    SPH_IFS_PROC_APPLY_MASTER_DARK_TOL) )
    {
        if ( dit_dark <= 0.0 || dit_frame <= 0.0 ) {
            SPH_ERROR_RAISE_ERR(CPL_ERROR_ILLEGAL_INPUT,
                    "Bad input DITs (%f, %f)!", dit_dark, dit_frame);
            return CPL_ERROR_ILLEGAL_INPUT;
        }
//        if ( dit_dark <= SPH_IFS_PROC_APPLY_MASTER_DARK_MIN_SCALED_DIT &&
//                dit_frame > SPH_IFS_PROC_APPLY_MASTER_DARK_MIN_SCALED_DIT ) {
//            SPH_ERROR_RAISE_WARNING(CPL_ERROR_ILLEGAL_INPUT,
//                    "The DITS dont match for dark subtraction: cant have a "
//                    "dark with a short DIT, but a raw frame with a long DIT."
//                    "Please select a different dark."
//                    "Am subtracting dark as is.");
//            sph_master_frame_subtract_master_frame(mframe,mdark);
//            return CPL_ERROR_NONE;
//        }
//        if ( dit_frame <= SPH_IFS_PROC_APPLY_MASTER_DARK_MIN_SCALED_DIT &&
//                dit_dark > SPH_IFS_PROC_APPLY_MASTER_DARK_MIN_SCALED_DIT ) {
//            SPH_ERROR_RAISE_WARNING(CPL_ERROR_ILLEGAL_INPUT,
//                    "The DITS dont match for dark subtraction: cant have a "
//                    "dark with a long DIT, but a raw frame with a short DIT."
//                    "Am subtracting dark as is.");
//            sph_master_frame_subtract_master_frame(mframe,mdark);
//            return CPL_ERROR_NONE;
//        }
        SPH_ERROR_RAISE_INFO(CPL_ERROR_CONTINUE,"Am scaling dark by ratio of DITs: %5.3f", dit_frame / dit_dark);
        tmpdark = sph_master_frame_duplicate(mdark);
        sph_master_frame_divide_double(tmpdark,dit_dark / dit_frame );
        sph_master_frame_subtract_master_frame(mframe,tmpdark);
        sph_master_frame_delete(tmpdark); tmpdark = NULL;
    }
    else {
        if ( keysok != 1 ) {
            SPH_ERROR_RAISE_WARNING(CPL_ERROR_ILLEGAL_INPUT,"Could not obtain DIT "
                    "from keywords %s from either master dark and/or raw frame. "
                    "So can not perform any dark rescaling before subtracting. "
                    "Am subtracting dark as it is.", SPH_COMMON_KEYWORD_DIT);
        }
        sph_master_frame_subtract_master_frame(mframe,mdark);
    }
    SPH_ERROR_CHECK_STATE_RETURN_ERRCODE;
}

/**@}*/
