/***********************************************************************************************/
/******** DANGER !! DANGER !! DANGER !! DO NOT EDIT FILE -- IT WAS GENERATED AUTOMATICALLY !!! */
/***********************************************************************************************/

/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>
#include <math.h>
#include "sph_common_keywords.h"
#include "sph_zpl_keywords.h"
#include "sph_zpl_tags.h"
#include "sph_zpl_aoc_efficiency.h"
#include "sph_master_frame.h"
#include "sph_error.h"
#include "sph_cube.h"
#include "sph_utils.h"

/*-----------------------------------------------------------------------------
  Error Codes
 -----------------------------------------------------------------------------*/

sph_error_code SPH_ZPL_AOC_EFFICIENCY_GENERAL              = SPH_ZPL_AOC_EFFICIENCY_ERR_START + 0;
sph_error_code SPH_ZPL_AOC_EFFICIENCY_NO_VALID_DIT_LIST    = SPH_ZPL_AOC_EFFICIENCY_ERR_START + 1;

sph_error_code SPH_ZPL_AOC_EFFICIENCY_PARAMETER_MISSING    = SPH_ZPL_AOC_EFFICIENCY_ERR_START + 3;
sph_error_code SPH_ZPL_AOC_EFFICIENCY_FRAMES_MISSING       = SPH_ZPL_AOC_EFFICIENCY_ERR_START + 4;

/*-----------------------------------------------------------------------------
 Private function prototypes
 -----------------------------------------------------------------------------*/

static int sph_zpl_aoc_efficiency_create_plugin ( cpl_plugin * );
static int sph_zpl_aoc_efficiency_exec_plugin ( cpl_plugin * );
static int sph_zpl_aoc_efficiency_destroy_plugin ( cpl_plugin * );

/*-----------------------------------------------------------------------------
 Static variables
 -----------------------------------------------------------------------------*/

static const char* sph_zpl_aoc_efficiency_help = 
    "RECIPE INPUT FRAMES: \n"
    "-------------------- \n"
    "Tag                           Min,Max   Description\n"
    "ZPL_AOC_EFF_PREPROC_RAW         1, -1   The pre-processed raw aoc efficiency frames.\n"
    "                                        Format: ZPL EXP\n"
    "ZPL_MASTER_BIAS                 0,  1   The master bias frame. Should have readout mode as the pre-processed aoc efficiency frames.\n"
    "                                        Format: QUAD IMAGE\n"
    "ZPL_MASTER_DARK                 0,  1   The master dark frame. Should have readout mode as the pre-processed aoc efficiency frames.\n"
    "                                        Format: QUAD IMAGE\n"
    "ZPL_INT_FLAT_FIELD_MASTER       0,  1   The master intensity flat field frame. Should have readout mode as the pre-processed science frames.\n"
    "                                        Format: MASTER FRAME\n"
    "ZPL_INT_FLAT_FIELD              0,  1   The master intensity flat field frame. Should have readout mode as the pre-processed aoc efficiency frames.\n"
    "                                        Format: MASTER FRAME\n"
    "ZPL_POL_FLAT_FIELD              0,  1   The master polarization flat field frame. Should have readout mode as the pre-processed aoc efficiency frames.\n"
    "                                        Format: DOUBLE IMAGE\n"
    "ZPL_MODEM_EFF                   0,  1   The master modem efficiency frame. Should have readout mode as the pre-processed aoc efficiency frames.\n"
    "                                        Format: MASTER FRAME\n"
    "\n"
    "RAW FRAME KEYWORDS: \n"
    "-------------------- \n"
    "Keyword                       Type   Optional  Description\n"
    "ESO DRS PC PROD TYPE          string No        >>SPH PC PREPROC ZPL EXP<< \n"
    "ESO DRS ZPL HWP2 SETTING      double No                       \n"
    "                                               SPH_ZPL_KEYWORD_STOCK_PARAMETER_SIGN		string			0       0     0 \n"
    "                                               >>PLUS<<		>>MINUS<< \n"
    "                                               SPH_ZPL_KEYWORD_DEROTATOR_ANGLE 			float           0		 0	 360.0 \n"
    "                                               The rotation angle of frame in degrees. \n"
    "\n"
    "DESCRIPTION:\n"
    "------------\n"
    "The recipe creates master AOC efficiency calibration frame,  using the input exposures \n"
    "which should be taken as described in the ZPL calibration plan. \n"
    "The input raw frames used by this recipe are the pre-processed raw aoc efficiency frames of two groups \n"
    "for several de-rotator angle position [0, 2PI], distinguished from each other by the opposite sign of the stock parameter [PLUS, MINUS]  \n"
    "These raw frames  should carry the ZPL_AOC_EFF_PREPROC_RAW tag (normally, it must be raw frames with one particular DIT and filter).\n"
    "The input calibration frames (if any) are the master bias frame with BIAS_CALIB tag, master dark frame with the DARK_CALIB tag, \n"
    "the master intensity flat field frame with the IFF_CALIB tag, the polarization flat field with the PFF_CALIB tag, and\n"
    "modulation / demodulation efficiency with the MODEM_CALIB tag.\n"
    "After reading the pre-processed aoc polarization efficiency raw frames, they are grouped by two with regard to the sign of the stock parameter [PLUS, MINUS]\n"
    "for each de-rotator position. \n"
    "The frames from each group are combined  using the specified \n"
    "collapse algorithm (usually the clean_mean algorithm, defined as a default one) for each zpl exposure sub-frames.\n"
    "The combined frames are of the QUAD IMAGE (16 extensions) format specified as follows:\n"
    "- zpl exp phase zero odd sub-frame combined image, badpixel-map, ncomb-map and rms-map;\n"
    "- zpl exp phase zero even  sub-frame combined image, badpixel-map, ncomb-map and rms-map;\n"
    "- zpl exp phase PI odd sub-frame master combined image, badpixel-map, ncomb-map and rms-map;\n"
    "- zpl exp phase PI even sub-frame  master combined image, badpixel-map, ncomb-map and rms-map;\n"
    "The master bias, dark and intensity flat field are applied to the two [PLUS, MINUS] combined \n"
    "master frames and then the stock parameters (I,P) are calculated for both group of frames \n"
    "for each de-rotator position in the form of DOUBLE IMAGE:\n"
    "- master intensity stock parameter image, badpixel-map, ncomb-map and rms-map;\n"
    "- master polarization stock parameter image, badpixel-map, ncomb-map and rms-map.\n"
    "Then, the polarization flat field and modem efficiency are applied to the [PLUS, MINUS] stock parameters double image frames.\n"
    "and the two opposite polarization frames are combined for each de-rotator position by subtracting \n"
    "the MINUS polarization image frame from the PLUS one.\n"
    "The output master aoc polarization efficiency frame for one given de-rotator position is of the MASTER FRAME format specified as follows:\n"
    "aoc efficiency image, badpixel-map, rms-map and  weight-map. \n"
    "Finally, recipe performs a fit for the mean values (SPH_COMMON_KEYWORD_QC_MEANMASTERFRAME) with it standard \n"
    "deviation (SPH_COMMON_KEYWORD_QC_RMSMASTERFRAME)  of the obtained master frames\n"
    "with respect to its de-rotator positions. The fitting coefficients are written out in the header of the product\n"
    "fits file. \n"
    "\n"
    "This master AOC efficiency product is used in the science polarization recipes and in all on sky \n"
    "polarization calibration recipes.\n"
    "\n"
    "RECIPE PRODUCTS: \n"
    "-------------------- \n"
    "Tag                      Format         Description\n"
    "ZPL_AOC_EFF              FITS[Im(4)]    The resulting Q->Q, U->U or V->V master aoc polarization efficiency frame is of the MASTER FRAME format.\n"
    "                                        This frame contains 4 image extensions:\n"
    "                                        - master aoc efficiency image, badpixel-map, ncomb-map and rms-map.\n"
    "ZPL_AOC_EFF_PLUS         FITS[Im(8)]    The resulting +Q/+U aoc polarization efficiency frame is of the DOUBLE IMAGE format.\n"
    "                                        This frame contains 4 image extensions:\n"
    "                                        - aoc efficiency qplus intensity image, badpixel-map, ncomb-map and rms-map.\n"
    "                                        - aoc efficiency qplus polarization image, badpixel-map, ncomb-map and rms-map.\n"
    "ZPL_AOC_EFF_MINUS        FITS[Im(8)]    The resulting -Q/-U aoc polarization efficiency frame is of the DOUBLE IMAGE format.\n"
    "                                        This frame contains 4 image extensions:\n"
    "                                        - aoc efficiency qminus intensity image, badpixel-map, ncomb-map and rms-map.\n"
    "                                        - aoc efficiency qminus polarization image, badpixel-map, ncomb-map and rms-map.\n"
    "\n"
    "PRODUCT FRAME KEYWORDS: \n"
    "-------------------- \n"
    "Keyword                       Frame          Type    Description\n"
    "\n"
;

/*-----------------------------------------------------------------------------
 Function code
 -----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/**
 * @defgroup sph_zpl_aoc_efficiency Create Master Dark Recipe
 *
 * This module provides the recipe plugin and relating functions for the
 * creation of the master dark .
 *
 * @par Synopsis:
 * @code
 *   #include "sph_zpl_aoc_efficiency.h"
 * @endcode
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*****************************************************************************
 *
 *
 *      First come the static and global functions necessary for the
 *      normal working of the recipe as plugin....
 *
 *
 * ***************************************************************************/
/*----------------------------------------------------------------------------*/
/**
 @brief    Build the list of available plugins, for this module.
 @param    list    the plugin list
 @return   0 if everything is ok, 1 otherwise
 @note     Only this function is exported

 Create the recipe instance and make it available to the application
 using the interface.
 */
/*----------------------------------------------------------------------------*/
int cpl_plugin_get_info(cpl_pluginlist * list)
{
    cpl_recipe * recipe = cpl_calloc( 1, sizeof *recipe);
    cpl_plugin * plugin = &recipe->interface;

    if (cpl_plugin_init(plugin,
                        CPL_PLUGIN_API,
                        SPH_ZPL_AOC_EFFICIENCY_VERSION,
                        CPL_PLUGIN_TYPE_RECIPE,
                        "sph_zpl_aoc_efficiency",
                        "Measure AOC efficiency, polarimetric modes (not currently applicable).",
                        sph_zpl_aoc_efficiency_help, "Alexey Pavlov",
                        "pavlov@mpia-hd.mpg.de", cpl_get_license("SPHERE DRH","2012"),
                        sph_zpl_aoc_efficiency_create_plugin,
                        sph_zpl_aoc_efficiency_exec_plugin,
                        sph_zpl_aoc_efficiency_destroy_plugin) )
    {
        cpl_msg_error( cpl_func, "Plugin initialization failed");
        (void)cpl_error_set_where( cpl_func );
        return 1;
    }

    if (cpl_pluginlist_append(list, plugin) )
    {
        cpl_msg_error( cpl_func, "Error adding plugin to list");
        (void)cpl_error_set_where( cpl_func );
        return 1;
    }
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Setup the recipe options
 @param    plugin  the plugin
 @return   0 if everything is ok

 Defining the command-line/configuration parameters for the recipe.
 */
/*----------------------------------------------------------------------------*/
static int sph_zpl_aoc_efficiency_create_plugin(cpl_plugin * plugin)
{
    cpl_recipe * recipe             = NULL;

    if (cpl_error_get_code() != CPL_ERROR_NONE)
    {
        cpl_msg_error( cpl_func, "%s():%d: An error is already set: %s",
                       cpl_func, __LINE__, cpl_error_get_where());
        return (int)cpl_error_get_code();
    }

    if (plugin == NULL)
    {
        cpl_msg_error(cpl_func, "Null plugin");
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
    }

    if ( plugin != NULL ) {
        recipe = (cpl_recipe *)plugin;
        recipe->parameters = sph_zpl_aoc_efficiency_create_paramlist();
        if ( recipe->parameters == NULL ) {
            return SPH_ZPL_AOC_EFFICIENCY_GENERAL;
        }
    }
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Testing for setup of the recipe options
 @param    plugin  the plugin
 @return   0 if everything is ok

 Wrapper to make the static plugin creation function accessible from outside
 for testing purposes.
 */
/*----------------------------------------------------------------------------*/
int sph_zpl_aoc_efficiency_create_test(cpl_plugin * plugin)
{
    return (int)sph_zpl_aoc_efficiency_create_plugin(plugin);
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Execute the plugin instance given by the interface
 @param    plugin  the plugin
 @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int sph_zpl_aoc_efficiency_exec_plugin(cpl_plugin * plugin)
{

    cpl_recipe*             recipe                  = NULL;
    int                     recipe_status           = 0;
    sph_zpl_aoc_efficiency*    sph_recipe      = NULL;
    cpl_errorstate          initial_errorstate      = cpl_errorstate_get();

    if (cpl_error_get_code() != CPL_ERROR_NONE)
    {
        cpl_msg_error( cpl_func, "%s():%d: A cpl error is already set: %s",
        cpl_func, __LINE__, cpl_error_get_where());
        return (int)cpl_error_get_code();
    }

    if (plugin == NULL)
    {
        cpl_msg_error( cpl_func, "Null plugin");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
        return (int)cpl_error_get_code();
    }

    recipe = (cpl_recipe *)plugin;

    if (recipe->parameters == NULL)
    {
        cpl_msg_error( cpl_func, "Recipe invoked with NULL parameter list");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    if (recipe->frames == NULL)
    {
        cpl_msg_error( cpl_func, "Recipe invoked with NULL frame set");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    sph_recipe = sph_zpl_aoc_efficiency_new( recipe->frames, recipe->parameters );
    if ( sph_recipe == NULL ) {
        cpl_msg_error( cpl_func, "SPHERE recipe initialisation call failed. "
                "Please check the SPHERE error messages.");
        sph_end_erex(); //close the sphere system
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    recipe_status = sph_zpl_aoc_efficiency_run( sph_recipe );

    sph_zpl_aoc_efficiency_delete( sph_recipe );

    if ( !cpl_errorstate_is_equal(initial_errorstate) )
    {
        cpl_errorstate_dump(initial_errorstate, CPL_FALSE, NULL);
    }
    sph_end_erex(); //close the sphere system
    return recipe_status;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Destroy what has been created by the 'create' function
 @param    plugin  the plugin
 @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int sph_zpl_aoc_efficiency_destroy_plugin(cpl_plugin * plugin)
{
    cpl_recipe * recipe;

    if (plugin == NULL)
    {
        cpl_msg_error( cpl_func, "Null plugin");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
    }

    recipe = (cpl_recipe *)plugin;

    cpl_parameterlist_delete(recipe->parameters);

    return 0;
}

/*****************************************************************************
 *
 *
 *      Here come the "Memnber" Functions of the sph_zpl_aoc_efficiency struct
 *
 *
 * ***************************************************************************/

/*----------------------------------------------------------------------------*/
/**
 @brief    Create and return the standard parameterlist for the recipe

 @return   the standard parameterlis or NULL if an error occcured.

 This function creates the standard parameterlist for the recipe
 and returns it.

 The return value is NULL when an error occured.
 */
/*----------------------------------------------------------------------------*/
cpl_parameterlist*  sph_zpl_aoc_efficiency_create_paramlist(void)
{
    cpl_parameterlist*          result              = NULL;
    cpl_parameter*              p                   = NULL;
    result = cpl_parameterlist_new();

    if (result == NULL)
    {
        cpl_msg_error( cpl_func, "Null parameterlist");
        return NULL;
    }
    else {
        /* Code to set up parameters GENERATED DO NOT EDIT */

        p = cpl_parameter_new_value("zpl.aoc_efficiency.outfilename",
                                   CPL_TYPE_STRING,
                                   "The output filename for the product. Please also see the esorex documentation "
                                   "for naming of output products. "
                                   ,"zpl.aoc_efficiency",
                                   "zpl_aoc_efficiency.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.aoc_efficiency_plus.outfilename",
                                   CPL_TYPE_STRING,
                                   "The output filename for the plus product. Please also see the esorex documentation "
                                   "for naming of output products. "
                                   ,"zpl.aoc_efficiency_plus",
                                   "zpl_aoc_efficiency_plus.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.aoc_efficiency_minus.outfilename",
                                   CPL_TYPE_STRING,
                                   "The output filename for the minus product. Please also see the esorex documentation "
                                   "for naming of output products. "
                                   ,"zpl.aoc_efficiency_minus",
                                   "zpl_aoc_efficiency_minus.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.aoc_efficiency.subtract_overscan",
                                   CPL_TYPE_BOOL,
                                   "Flag to set if the overscan mean values must be subtracted from pre-processed data (TRUE) "
                                   "Note that this parameter is applied if pre-processed data containt overscan table "
                                   ,"zpl.aoc_efficiency",
                                   1 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_enum("zpl.aoc_efficiency.coll_alg",
                                   CPL_TYPE_INT,
                                   "Set the collapse algorithm. The available algorithms: 0 = Mean, 1 = Median, 2 = Clean Mean. Default is 2 = Clean Mean "
                                   ,"zpl.aoc_efficiency",
                                   2 ,
                                   3,0,1,2);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("zpl.aoc_efficiency.coll_alg.clean_mean.reject_high",
                                   CPL_TYPE_INT,
                                   "The number of pixels to reject when combining frames at the high end. Number of input frames must be > reject_high +reject_low "
                                   ,"zpl.aoc_efficiency",
                                   0 ,
                                   0,20);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("zpl.aoc_efficiency.coll_alg.clean_mean.reject_low",
                                   CPL_TYPE_INT,
                                   "The number of pixels to reject when combining frames at the low end. Number of input frames must be > reject_high +reject_low "
                                   ,"zpl.aoc_efficiency",
                                   0 ,
                                   0,20);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        /* End of code to set up parameters BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    }


    return result;


}

/*----------------------------------------------------------------------------*/
/**
 @brief    Check the recipe frames, and assign them to the sph_zpl_aoc_efficiency
           structure.
 @param    self     the sph_zpl_aoc_efficiency recipe structure
 @return   error code of the check.

 This checks the frames that were given in the sph_zpl_aoc_efficiency_new function
 as parameters for the correctness and completeness. If the frames needed are
 found they are also assigned to the corresponding structure members.

 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code  sph_zpl_aoc_efficiency_check_frames( sph_zpl_aoc_efficiency* self )
{
    sph_error_code           rerr            = CPL_ERROR_NONE;
    int                      ii	             = 0;
    cpl_frame*               aframe          = NULL;
    cpl_propertylist*        pl              = NULL;
    int	                     haskey          = 0;
    int                      okval           = 0;
    /* Code to check frames GENERATED DO NOT EDIT */

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_AOC_EFF_PREPROC_RAW );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_RAW );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->rawframes = sph_utils_extract_frames( self->inframes, SPH_ZPL_TAG_AOC_EFF_PREPROC_RAW );
    if ( ! self->rawframes )
    {
        sph_error_raise( SPH_ZPL_AOC_EFFICIENCY_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_ERROR,
                         "Could not extract rawframes frames."
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_AOC_EFF_PREPROC_RAW);
        rerr = SPH_ZPL_AOC_EFFICIENCY_FRAMES_MISSING;
        return rerr;
    }

    else {

         for ( ii = 0; ii < cpl_frameset_get_size( self->rawframes ); ++ii ) {
             rerr = cpl_frame_set_group( cpl_frameset_get_frame( self->rawframes, ii ), CPL_FRAME_GROUP_RAW );
             pl = cpl_propertylist_load(cpl_frame_get_filename(cpl_frameset_get_frame( self->rawframes, ii )),0);
             if (pl == NULL ) {
                 sph_error_raise(cpl_error_get_code(),
                    __FILE__, __func__, __LINE__ ,SPH_ERROR_ERROR,
                 "Could not read keywords from file %s. "
                 "Either the file is not readable or corrupted. ",
                  cpl_frame_get_filename(cpl_frameset_get_frame( self->rawframes, ii )));
             }
             else {
                 haskey = 0;okval = 0;
                 if ( cpl_propertylist_has(pl,"ESO DRS PC PROD TYPE") )  {
                     haskey = 1;
                     if ( cpl_propertylist_get_type(pl,"ESO DRS PC PROD TYPE") == CPL_TYPE_STRING )  {
                         okval = 1; //Type is string so always ok
                     } 
                     if ( haskey == 0 || okval == 0 ) {
                         sph_error_raise(CPL_ERROR_ILLEGAL_INPUT,
                            __FILE__, __func__, __LINE__ ,SPH_ERROR_ERROR,
                         "Could not read a keyword ESO DRS PC PROD TYPE from file %s. "
                         "that has __type string "
                         "and values between 0.0 and 0.0 "
                         "The recipe requires this. ",
                         cpl_frame_get_filename(cpl_frameset_get_frame( self->rawframes, ii )));
                     }
                 }
                 haskey = 0;okval = 0;
                 if ( cpl_propertylist_has(pl,"ESO DRS ZPL HWP2 SETTING") )  {
                     haskey = 1;
                     if ( cpl_propertylist_get_type(pl,"ESO DRS ZPL HWP2 SETTING") == CPL_TYPE_DOUBLE )  {
                         if ( cpl_propertylist_get_double(pl,"ESO DRS ZPL HWP2 SETTING") >= 0.0 && 
                             cpl_propertylist_get_double(pl,"ESO DRS ZPL HWP2 SETTING") <= 67.5) okval=1;
                     } 
                     if ( haskey == 0 || okval == 0 ) {
                         sph_error_raise(CPL_ERROR_ILLEGAL_INPUT,
                            __FILE__, __func__, __LINE__ ,SPH_ERROR_ERROR,
                         "Could not read a keyword ESO DRS ZPL HWP2 SETTING from file %s. "
                         "that has __type double "
                         "and values between 0.0 and 67.5 "
                         "The recipe requires this. ",
                         cpl_frame_get_filename(cpl_frameset_get_frame( self->rawframes, ii )));
                     }
                 }
                 cpl_propertylist_delete(pl); pl=NULL;
             }
         }
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_BIAS_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->master_bias_frame = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_BIAS_CALIB );
    if ( ! self->master_bias_frame )
    {
        sph_error_raise( SPH_ZPL_AOC_EFFICIENCY_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract master_bias_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_BIAS_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->master_bias_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_DARK_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->master_dark_frame = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_DARK_CALIB );
    if ( ! self->master_dark_frame )
    {
        sph_error_raise( SPH_ZPL_AOC_EFFICIENCY_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract master_dark_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_DARK_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->master_dark_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_IFFM_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->intensity_flat_frame_master = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_IFFM_CALIB );
    if ( ! self->intensity_flat_frame_master )
    {
        sph_error_raise( SPH_ZPL_AOC_EFFICIENCY_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract intensity_flat_frame_master frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_IFFM_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->intensity_flat_frame_master, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_IFF_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->intensity_flat_frame = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_IFF_CALIB );
    if ( ! self->intensity_flat_frame )
    {
        sph_error_raise( SPH_ZPL_AOC_EFFICIENCY_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract intensity_flat_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_IFF_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->intensity_flat_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_PFF_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->polarization_flat_frame = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_PFF_CALIB );
    if ( ! self->polarization_flat_frame )
    {
        sph_error_raise( SPH_ZPL_AOC_EFFICIENCY_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract polarization_flat_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_PFF_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->polarization_flat_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_MODEM_EFF_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->modem_efficiency_frame = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_MODEM_EFF_CALIB );
    if ( ! self->modem_efficiency_frame )
    {
        sph_error_raise( SPH_ZPL_AOC_EFFICIENCY_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract modem_efficiency_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_MODEM_EFF_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->modem_efficiency_frame, CPL_FRAME_GROUP_CALIB );
    }

    /* End of code to check frames BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    self->current_raw_frameset = sph_utils_extract_frames_group( self->inframes, CPL_FRAME_GROUP_RAW );
    haskey = 0; /* This is just to remove compiler warnings */
    okval = 0;/* This is just to remove compiler warnings */
    ii = 0; pl = NULL; /* This is just to remove compiler warnings */

    return rerr;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Check the recipe parameters, and assign them to the sph_zpl_aoc_efficiency
           structure.
 @param    self     the sph_zpl_aoc_efficiency recipe structure
 @return   error code of the check.

 This checks the parameters that were given in the sph_zpl_aoc_efficiency_new function
 as parameters for the correctness and completeness. If the parameters needed are
 found they are also assigned to the corresponding structure members.

 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code sph_zpl_aoc_efficiency_check_params( sph_zpl_aoc_efficiency* self ) {
    sph_error_code           	   rerr               = CPL_ERROR_NONE;
    const cpl_parameter*           param              = NULL;

    if ( self->framecomb_parameterlist != NULL ) {
    	cpl_parameterlist_delete( self->framecomb_parameterlist );
    	self->framecomb_parameterlist = NULL;
    }
    self->framecomb_parameterlist = cpl_parameterlist_new();
    /* Code to check parameters GENERATED DO NOT EDIT */

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.aoc_efficiency.outfilename");
    self->aoc_efficiency_outfilename = cpl_parameter_get_string( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_ZPL_AOC_EFFICIENCY_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: zpl.aoc_efficiency.outfilename"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.aoc_efficiency_plus.outfilename");
    self->aoc_efficiency_plus_outfilename = cpl_parameter_get_string( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_ZPL_AOC_EFFICIENCY_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: zpl.aoc_efficiency_plus.outfilename"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.aoc_efficiency_minus.outfilename");
    self->aoc_efficiency_minus_outfilename = cpl_parameter_get_string( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_ZPL_AOC_EFFICIENCY_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: zpl.aoc_efficiency_minus.outfilename"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.aoc_efficiency.subtract_overscan");
    self->subtract_overscan = cpl_parameter_get_bool( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_ZPL_AOC_EFFICIENCY_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: zpl.aoc_efficiency.subtract_overscan"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.aoc_efficiency.coll_alg");
    self->coll_alg = cpl_parameter_get_int( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_ZPL_AOC_EFFICIENCY_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: zpl.aoc_efficiency.coll_alg"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.aoc_efficiency.coll_alg.clean_mean.reject_high");
    self->clean_mean_reject_high = cpl_parameter_get_int( ( const cpl_parameter*  )param);
    cpl_parameterlist_append( self->framecomb_parameterlist,
                              cpl_parameter_new_range( "clean_mean.reject_high",
                                                       CPL_TYPE_INT,
                                                       NULL,
                                                       NULL,
                                                       cpl_parameter_get_int( param ) ,
                                                       0,20
                                                      )
                            );

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.aoc_efficiency.coll_alg.clean_mean.reject_low");
    self->clean_mean_reject_low = cpl_parameter_get_int( ( const cpl_parameter*  )param);
    cpl_parameterlist_append( self->framecomb_parameterlist,
                              cpl_parameter_new_range( "clean_mean.reject_low",
                                                       CPL_TYPE_INT,
                                                       NULL,
                                                       NULL,
                                                       cpl_parameter_get_int( param ) ,
                                                       0,20
                                                      )
                            );

    /* End of code to check parameters BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/

    return rerr;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Create a new sph_zpl_aoc_efficiency structure.

 @param    frameset     the input recipe frames
 @param    parlist      the input recipe parameters

 @return   pointer to newly created structure or NULL in case of error

 This creates a new sph_zpl_aoc_efficiency_structure and fills the inparams and
 inframes fields with the pointers pf frameset and parlist.
 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_zpl_aoc_efficiency* sph_zpl_aoc_efficiency_new( cpl_frameset * frameset,
                                              cpl_parameterlist * parlist )
{
    sph_zpl_aoc_efficiency*    result      = NULL;
    int                     rerr        = CPL_ERROR_NONE;

    sph_init_erex();
    if ( frameset == NULL || parlist == NULL ) {
        sph_error_raise( CPL_ERROR_NULL_INPUT, __FILE__,
                         __func__, __LINE__,
                         SPH_ERROR_ERROR,
                         "Null input pointer.");
        return NULL;
    }

    result = cpl_calloc( 1, sizeof(sph_zpl_aoc_efficiency) );

    if ( result == NULL ) {
        sph_error_raise( SPH_ZPL_AOC_EFFICIENCY_GENERAL, __FILE__, __func__,
                         __LINE__, SPH_ERROR_ERROR,
                         "Could not allocate the structure.");
        return result;
    }

    result->inframes = frameset;
    result->inparams = parlist;

    rerr = sph_zpl_aoc_efficiency_check_frames( result );
    if ( rerr == CPL_ERROR_NONE) rerr = sph_zpl_aoc_efficiency_check_params( result );

    if ( rerr != CPL_ERROR_NONE ) {
    	if ( result ) {
    		sph_zpl_aoc_efficiency_delete( result );
    	}
    	result = NULL;
    }

    return result;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Delete the sph_zpl_aoc_efficiency structure.

 @param    self         the structure tp delete

 @return   error code of the operation

 This creates a new sph_zpl_aoc_efficiency_structure and fills the inparams and
 inframes fields with the pointers pf frameset and parlist.
 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code sph_zpl_aoc_efficiency_delete( sph_zpl_aoc_efficiency* self )
{
    sph_error_code          rerr            = CPL_ERROR_NONE;

    if ( self == NULL ) {
        sph_error_raise( CPL_ERROR_NULL_INPUT, __FILE__,
                         __func__, __LINE__,
                         SPH_ERROR_ERROR,
                         "Null input pointer.");
        return rerr;
    }

    if ( self->framecomb_parameterlist != NULL ) {
    	cpl_parameterlist_delete( self->framecomb_parameterlist );
	    self->framecomb_parameterlist = NULL;
    }

    /* Code to delete recipe pointers GENERATED DO NOT EDIT */

    if ( self->rawframes != NULL ) {
        cpl_frameset_delete( self->rawframes );
        self->rawframes = NULL;
    }
    if ( self->master_bias_frame != NULL ) {
        cpl_frame_delete( self->master_bias_frame );
        self->master_bias_frame = NULL;
    }
    if ( self->master_dark_frame != NULL ) {
        cpl_frame_delete( self->master_dark_frame );
        self->master_dark_frame = NULL;
    }
    if ( self->intensity_flat_frame_master != NULL ) {
        cpl_frame_delete( self->intensity_flat_frame_master );
        self->intensity_flat_frame_master = NULL;
    }
    if ( self->intensity_flat_frame != NULL ) {
        cpl_frame_delete( self->intensity_flat_frame );
        self->intensity_flat_frame = NULL;
    }
    if ( self->polarization_flat_frame != NULL ) {
        cpl_frame_delete( self->polarization_flat_frame );
        self->polarization_flat_frame = NULL;
    }
    if ( self->modem_efficiency_frame != NULL ) {
        cpl_frame_delete( self->modem_efficiency_frame );
        self->modem_efficiency_frame = NULL;
    }
 
    sph_master_frame_delete(self->aoc_efficiency);
    sph_double_image_delete(self->aoc_efficiency_di_plus);
    sph_double_image_delete(self->aoc_efficiency_di_minus);
    /* End of code to delete pointers BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    if ( self->current_raw_frameset ) {
    	cpl_frameset_delete(self->current_raw_frameset);
	self->current_raw_frameset = NULL;
    }
    sph_polygon_free_all();
    cpl_free(    self);
    return rerr;

}


/**@}*/
