/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <cpl.h>
#include "sph_error.h"
#include "sph_distortion_vector.h"

/*----------------------------------------------------------------------------*/
/**
 * @defgroup A SPHERE API Module
 * @par Synopsis:
 * @code
 * typedef _module_ {
 * } module
 * @endcode
 * @par Desciption:
 *
 * This module provides ...
 */
/*----------------------------------------------------------------------------*/
/**@{*/

sph_distortion_vector*
sph_distortion_vector_new(double x, double y, double dx, double dy) {
    sph_distortion_vector* result = cpl_calloc( sizeof(sph_distortion_vector), 1);
    if ( !result ) return NULL;
    result->dx = dx;
    result->dy = dy;
    result->x = x;
    result->y = y;
    return result;
}

sph_error_code
sph_distortion_vector_append_new(sph_distortion_vector* vec,double x, double y, double dx, double dy) {
    sph_distortion_vector* result = NULL;
    if ( !vec ) return CPL_ERROR_NULL_INPUT;

    result = sph_distortion_vector_new(x, y, dx, dy);

    if ( !result ) return SPH_ERROR_MEMORY_FAIL;
    vec->next = result;
    result->prev = vec;
    return CPL_ERROR_NONE;
}

sph_error_code
sph_distortion_vector_append(sph_distortion_vector* vec,sph_distortion_vector* app) {
    if ( !vec ) return CPL_ERROR_NULL_INPUT;
    vec->next = app;
    app->prev = vec;
    return CPL_ERROR_NONE;
}

sph_distortion_vector*
sph_distortion_vector_next(sph_distortion_vector* vec) {
    if ( !vec ) return NULL;
    return vec->next;
}

sph_distortion_vector*
sph_distortion_vector_prev(sph_distortion_vector* vec) {
    if ( !vec ) return NULL;
    return vec->prev;
}
void
sph_distortion_vector_delete(sph_distortion_vector* vec) {
    if ( vec ) {
        if ( vec->prev && vec->next ) ((sph_distortion_vector*)(vec->prev))->next = vec->next;
        if ( vec->prev && vec->next == NULL ) ((sph_distortion_vector*)(vec->prev))->next = NULL;
        if ( vec->next && vec->prev ) ((sph_distortion_vector*)(vec->next))->prev = vec->prev;
        if ( vec->next && vec->prev == NULL ) ((sph_distortion_vector*)(vec->next))->prev = NULL;
        cpl_free(vec);
    }
}
void
sph_distortion_vector_delete_all(sph_distortion_vector* vec) {
    sph_distortion_vector* vec0 = vec;
    // Find last element first
    while (vec0->prev != NULL) vec0 = vec0->prev;
    // Find last element first
    while (vec->next != NULL) vec = vec->next;
    // Now, need to go back one
    vec = vec->prev;
    while ( vec ) {
        sph_distortion_vector_delete(vec->next);
        vec = vec->prev;
    }
    //The first element is remaining delete it now.
    sph_distortion_vector_delete(vec0);
}

/**@}*/
