/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifndef SPH_TEST_H_
#define SPH_TEST_H_
#include "sph_ifs_lenslet_model.h"
#include "sph_pixel_description_table.h"
#include "sph_dictionary.h"
#include "sph_filemanager.h"
enum _sph_test_fits_compare_ {
    SPH_TEST_PURE = 0,
    SPH_TEST_SUBTRACT_MEDIAN = 1,
    SPH_TEST_SUBTRACT_MEAN = 2,
    SPH_TEST_DIVIDE_MEAN = 3
};

typedef struct _sph_test_fits_stats_ {
    double          rms;
    double          mean;
    double          median;
    double          max;
    double          min;
    int             nbadpix;
} sph_test_fits_stats;

const char* frame_group_to_string(cpl_frame_group group);

int sph_test_create_dit_list(void);

int sph_test_verify_fits(char* czFilename, char* compFile,
                         int compmode,
                         sph_test_fits_stats* pStats,
                         sph_test_fits_stats* pStatsComp,
                         cpl_image** difim );

int sph_test_verify_fits_constant( char* czFilename, double correctval);
cpl_error_code
sph_test_retag_frames_by_group(
        cpl_frameset* inoutframes,
        cpl_frame_group group, const char* newtag);

int sph_test_create_results_file( const char* czFilename );

int sph_test_get_stats_cpl( cpl_image* im, sph_test_fits_stats* pStats);
cpl_vector*	sph_test_cpl_vector_linear( int np, double min, double max );
cpl_vector*	sph_test_cpl_vector_gauss( int np, double min, double max, double mean, double sigma, double peak );
cpl_error_code sph_test_create_ifs_master_dark_frame( cpl_frameset* frames, int ndits );
cpl_error_code sph_test_create_ifs_raw_dark_frameset( cpl_frameset* frames, int nframes );
cpl_error_code sph_test_create_ifs_raw_dark_frameset_big( cpl_frameset* frames, int nframes );
cpl_error_code sph_test_create_ifs_raw_dff_frameset_broad_band( cpl_frameset* frames, int nframes, int lampOnId );
cpl_error_code sph_test_create_ifs_raw_wave_calib_frameset( cpl_frameset* frames, int nframes );
sph_pixel_description_table*
sph_test_create_ifs_master_pdt_frame( cpl_frameset* frames );
cpl_parameterlist*  sph_test_create_paramlist_ifs_spectra_positions(void);
cpl_parameterlist*  sph_test_create_paramlist_ifs_master_dark(void);
cpl_error_code sph_test_create_zpl_master_modem_eff_frame( cpl_frameset* frames, int numberDITs );
cpl_error_code sph_test_create_ird_master_dark_frame( cpl_frameset* frames, int ndits );
cpl_error_code sph_test_create_ird_raw_dark_frameset( cpl_frameset* frames, int nframes );
cpl_error_code sph_test_create_ird_raw_iff_frameset_ci( cpl_frameset* frames, int nframes );
cpl_parameterlist*  sph_test_create_paramlist_ird_master_dark(void);
cpl_error_code sph_test_create_zpl_raw_dark_frameset( cpl_frameset* frames, int nframes );
cpl_error_code sph_test_create_zpl_raw_dark_frameset_et( cpl_frameset* frames, int nframes );
cpl_parameterlist*  sph_test_create_paramlist_zpl_master_dark(void);
cpl_error_code sph_test_create_ifs_master_ifu_frame( cpl_frameset* frames );
cpl_error_code sph_test_create_ifs_sample_ifu_frame( cpl_frameset* frames );
//cpl_error_code  sph_test_create_zpl_raw_intensity_frameset( cpl_frameset *frames, int nframes );
cpl_error_code sph_test_create_ifs_master_detector_flat_frame( cpl_frameset* frames, int numberDITs );
int sph_test_check_result(void);
sph_ifs_lenslet_model* sph_test_create_small_lenslet_model(void);
cpl_error_code sph_test_create_zpl_raw_iff_frameset( cpl_frameset* frames, int nframes );
cpl_error_code sph_test_create_zpl_raw_iff_const_frameset(  cpl_frameset* frames, int nframes  );
cpl_error_code sph_test_create_zpl_raw_pff_const_frameset( cpl_frameset* frames, int nframes );
cpl_error_code sph_test_create_zpl_raw_meff_const_frameset( cpl_frameset* frames, int nframes );
cpl_error_code sph_test_create_zpl_master_iff_const_frame( cpl_frameset* frames, int nfilterIDs );
cpl_error_code sph_test_create_zpl_master_pff_const_frame( cpl_frameset *frames, int nmodeIDs );
cpl_error_code sph_test_create_zpl_master_int_flat_frame( cpl_frameset* frames, int numberDITs );
cpl_error_code sph_test_create_zpl_master_pol_flat_frame( cpl_frameset* frames, int numberDITs );
//cpl_error_code sph_test_create_zpl_preproc_frameset ( cpl_frameset *frames, int nframes, int nx, int ny );
void
sph_test_testfiles_add_file( const char* filename );
cpl_frame*	sph_test_frame_image( cpl_image* inimage, cpl_type_bpp bpp, const char* tag, cpl_frame_group group );
void sph_test_cleanup_testfiles();
cpl_frame*	sph_test_cubify_image( cpl_image* inimage,
		cpl_type_bpp bpp,
		const char* tag,
		cpl_frame_group group );
sph_pixel_description_table* sph_test_create_ifs_master_specpos_frame( cpl_frameset* frames );
void
sph_test_frame_add_key_int( cpl_frame* frame, unsigned int ext, const char* name, int value ) ;
void
sph_test_frame_add_key_string( cpl_frame* frame, unsigned int ext, const char* name, const char* value );
void
sph_test_frame_add_key_double( cpl_frame* frame, unsigned int ext, const char* name, double value );

const char*
sph_test_get_key_string( const char* filename, unsigned int ext, const char* name );
int
sph_test_get_key_int( const char* filename, unsigned int ext, const char* name );
double
sph_test_get_key_double( const char* filename, unsigned int ext, const char* name );
cpl_error_code sph_test_create_zpl_master_dark_frame( cpl_frameset* frames, int numberDITs );
void
sph_test_frame_add_datestamp( cpl_frame* aframe, unsigned int ext );
#endif /*SPH_TEST_H_*/
