/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <Basic.h>
#include <CUnit.h>
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_ngc_ir_simulator.h"
#include "sph_test_pupilimage_creator.h"
#include "sph_master_frame.h"
#include "sph_cube.h"
#include "sph_utils.h"
#include "sph_ifs_tags.h"
#include "sph_test_ifs_instrument_model.h"
#include "sph_ifs_science_dr.h"
#include "sph_common_keywords.h"
#include "sph_test_image_tools.h"
#include "sph_fits.h"
#include "sph_frame_validator.h"
#include "sph_ifs_science.c"
#include "sph_time.h"
#include "sph_create_flat.c"
#include "sph_create_super_flat.c"
#include "sph_ifs_science_dr_run.c"
#include "sph_ifs_instrument_flat_run.c"
#include "sph_ifs_proc_apply_master_dark.c"
#include "sph_ifs_test_fixtures_science_raw.h"
#include "sph_ifs_test_fixtures_calib_products.h"

/*----------------------------------------------------------------------------*/
/**
 * @defgroup A CUnit Test Suite -- representing a collection of testcases
 * @par Synopsis:
 * @code
 * @endcode
 * @par Desciption:
 *
 * This module provides a collection of tests for one specific, distinct
 * module or set-up. The testing code is implemented using the CUnit
 * framework.
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*----------------------------------------------------------------------------*/
/*-                            INTERNAL HELPER FUNCTIONS                          */
/*----------------------------------------------------------------------------*/


static
sph_ifs_science_dr* cutest_setup_recipe() {
    sph_test_ngc_ir_simulator*        ngc     = NULL;
    sph_ifs_lenslet_model*          lensmodel = NULL;
    sph_test_ifs_instrument_model*    insmodel = NULL;
    cpl_frame*                      mframe = NULL;
    cpl_frameset*                   rawframes = NULL;
    static sph_ifs_science_dr*             srecipe = NULL;

    if ( srecipe == NULL ) {
      sph_time_start_timer(1);
      ngc = sph_test_ngc_ir_simulator_new(1024,1024);
      CU_ASSERT_PTR_NOT_NULL_FATAL( ngc );
      lensmodel = sph_ifs_lenslet_model_new();
      lensmodel->lenslets_per_side = 30;
      lensmodel->detsize_pixels = ngc->det_size_x;
      insmodel = sph_test_ifs_instrument_model_new_from_lenslet_model( lensmodel
               );
      SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL,"Time %f", sph_time_stop_timer(1));
      CU_ASSERT_PTR_NOT_NULL_FATAL( insmodel );
      rawframes = sph_ifs_test_fixtures_science_raw_constant_flux( ngc, insmodel );
      SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL,"Time %f", sph_time_stop_timer(1));
      CU_ASSERT_PTR_NOT_NULL_FATAL( rawframes );
      //mframe = cutest_create_master_dark( ngc );
      //SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL,"Time %f", sph_time_stop_timer(1));
      //CU_ASSERT_PTR_NOT_NULL_FATAL( mframe );
      //cpl_frameset_insert( rawframes, mframe );
      //mframe = cutest_dff_short( ngc );
      //SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL,"Time %f", sph_time_stop_timer(1));
      //CU_ASSERT_PTR_NOT_NULL_FATAL( mframe );
      //cpl_frameset_insert( rawframes, mframe );
      //mframe = cutest_dff_long( ngc, 1.0, 2.0, 1.0, 1 );
      //SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL,"Time %f", sph_time_stop_timer(1));
      //CU_ASSERT_PTR_NOT_NULL_FATAL( mframe );
      //cpl_frameset_insert( rawframes, mframe );
      //mframe = cutest_dff_long( ngc, 1.5, 4.0, 1.2, 2 );
      //SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL,"Time %f", sph_time_stop_timer(1));
      //CU_ASSERT_PTR_NOT_NULL_FATAL( mframe );
      //cpl_frameset_insert( rawframes, mframe );
      //mframe = cutest_dff_long( ngc, 1.7, 6.0, 1.5, 3 );
      //SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL,"Time %f", sph_time_stop_timer(1));
      //CU_ASSERT_PTR_NOT_NULL_FATAL( mframe );
      //cpl_frameset_insert( rawframes, mframe );
      //mframe = cutest_dff_long( ngc, 1.9, 7.0, 1.7, 4 );
      //SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL,"Time %f", sph_time_stop_timer(1));
      //CU_ASSERT_PTR_NOT_NULL_FATAL( mframe );
      //cpl_frameset_insert( rawframes, mframe );
      //mframe = cutest_create_pdt( insmodel );
      //SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL,"Time %f", sph_time_stop_timer(1));
      //CU_ASSERT_PTR_NOT_NULL_FATAL( mframe );
      //cpl_frameset_insert( rawframes, mframe );
      mframe = sph_ifs_test_fixtures_calib_products_wave_calib( insmodel );
      SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL,"Time %f", sph_time_stop_timer(1));
      CU_ASSERT_PTR_NOT_NULL_FATAL( mframe );
      cpl_frameset_insert( rawframes, mframe );
      //mframe = cutest_ifu_flat( ngc, insmodel );
      //SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL,"Time %f", sph_time_stop_timer(1));
      //CU_ASSERT_PTR_NOT_NULL_FATAL( mframe );
      //cpl_frameset_insert( rawframes, mframe );
      //SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL,"Time %f", sph_time_stop_timer(1));
      srecipe = sph_ifs_science_dr_new( rawframes, sph_ifs_science_dr_create_paramlist() );
      SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL,"Time %f", sph_time_stop_timer(1));
      //srecipe->pupil_stab = 1;
      //srecipe->use_adi = 1;
    }
        CU_ASSERT_PTR_NOT_NULL_FATAL( srecipe );
    sph_test_ngc_ir_simulator_delete( ngc );
    ngc = NULL;
    sph_test_ifs_instrument_model_delete( insmodel );
    insmodel = NULL;
    return srecipe;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to initiailise the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_init_testsuite() {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__,"moeller@mpia-hd.mpg.de",CPL_MSG_DEBUG);
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to clean the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_clean_testsuite() {
    sph_error_dump( SPH_ERROR_ERROR );
    return sph_end_test();
}


/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_science_dr_run() {
    sph_ifs_science_dr*             recipe = NULL;
    sph_error_code                  rerr = CPL_ERROR_NONE;
    sph_ldt*  ldt = NULL;
    cpl_frameset*                   prodframes = NULL;
    int                             ll          = 0;
    cpl_vector*                     vals = NULL;
    double                                                                                                                         rms = 0;
    cpl_vector*                     lensl = NULL;
    FILE*                           fp = NULL;
    int                             pp = 0;
    recipe = cutest_setup_recipe();
    CU_ASSERT_PTR_NOT_NULL_FATAL( recipe );
    rerr = sph_ifs_science_dr_run( recipe );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );
    prodframes = sph_utils_extract_frames(recipe->inframes,SPH_IFS_TAG_SCIENCE_DR_CALIB);
    CU_ASSERT_EQUAL( cpl_frameset_get_size(prodframes), 1);
    ldt = sph_ldt_load(
            cpl_frame_get_filename(
                    cpl_frameset_get_first(prodframes)
                    )
             );

    CU_ASSERT_PTR_NOT_NULL( ldt );
    if ( ldt ) {
        fp = fopen("ldt_table.txt","w");
        vals = cpl_vector_new(ldt->nlens);
        for (ll = 0; ll < ldt->nlens; ++ll) {
            cpl_vector_set( vals, ll,
                    sph_lenslet_descriptor_get_median(ldt->arr[ll], &rms) );
            fprintf(fp,"%3d  %3d,%3d     %4.2f\n",
                    ll,ldt->arr[ll]->u,ldt->arr[ll]->v,
                    cpl_vector_get(vals,ll) );
            if ( ldt->arr[ll]->u < 2 && ldt->arr[ll]->u > - 2 &&
                    ldt->arr[ll]->v < 2 && ldt->arr[ll]->v > - 2) {
                for (pp = 1; pp < cpl_vector_get_size( ldt->reflambdas ); ++pp) {
                    if ( fabs(ldt->arr[ll]->values[pp] - cpl_vector_get(vals,ll)) > 0.1 ) {
                        printf("%d,%d - %d: Value was: %4.2f vs expected %4.2f\n",
                                ldt->arr[ll]->u,ldt->arr[ll]->v,
                                pp,ldt->arr[ll]->values[pp],cpl_vector_get(vals,ll));
                        //CU_FAIL_FATAL("Bad value");
                    }
                }
            }
        }
        fprintf(fp,"\n\n------------------------------------\n\n");
        fprintf(fp,"Mean median: %f\n",cpl_vector_get_mean(vals));
        fclose(fp);
        CU_ASSERT_DOUBLE_EQUAL( cpl_vector_get_mean(vals), 1000.0, 0.00000001 );
        cpl_vector_delete(vals); vals = NULL;
        sph_ldt_delete( ldt ); ldt = NULL;
    }

    cpl_parameterlist_delete( recipe->inparams);
    cpl_frameset_delete( recipe->inframes );
    cpl_frameset_delete(prodframes);
    sph_ifs_science_dr_delete( recipe );
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test MAIN function
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    int         result      = 0;
    CU_pSuite   pSuite      = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("Testing sph_ifs_science_dr_run module", cutest_init_testsuite,
                          cutest_clean_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "cutest_ifs_science_dr_run",
                             cutest_ifs_science_dr_run ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
