/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <Basic.h>
#include <CUnit.h>
#include "sph_error.h"
#include "sph_test.h"
#include "sph_ifs_keywords.h"
#include "sph_ifs_simple_adi.h"
#include "sph_test_ngc_ir_simulator.h"
#include "sph_test_pupilimage_creator.h"
#include "sph_test_image_tools.h"
#include "sph_transform.h"
#include "sph_utils.h"
static
void cutest_ifs_simple_adi_test_it(void);
static
int cutest_ifs_simple_adi_init_testsuite();
static
int cutest_ifs_simple_adi_clean_testsuite();
static
cpl_frame*
__cutest_ifs_simple_adi_fixture_planet_cube( double angle );

/*----------------------------------------------------------------------------*/
/**
 * @defgroup A CUnit Test Suite -- representing a collection of testcases
 * @par Synopsis:
 * @code
 * @endcode
 * @par Desciption:
 *
 * This module provides a collection of tests for one specific, distinct
 * module or set-up. The testing code is implemented using the CUnit
 * framework.
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test MAIN function
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    int         result      = 0;
    CU_pSuite   pSuite      = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("A cutest_ifs_simple_adi testsuite",
            cutest_ifs_simple_adi_init_testsuite,
                          cutest_ifs_simple_adi_clean_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "Testing simple ADI",
                             cutest_ifs_simple_adi_test_it ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}


/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_simple_adi_test_it(void) {
    sph_ifs_simple_adi*         ifs_adi     = NULL;
    int                         nf           = 10;
    double                      angle        = 0.0;
    double                      dangle       = 10.0;
    int                         ff          = 0;
    const char*                 fname       = "ifs_adi.fits";
    ifs_adi = cpl_calloc(1,sizeof(sph_ifs_simple_adi));
    CU_ASSERT_PTR_NOT_NULL_FATAL(ifs_adi);
    ifs_adi->transform_method = SPH_TRANSFORM_METHOD_CPL_WARP;
    ifs_adi->current_raw_frameset = cpl_frameset_new();
    ifs_adi->outfilename = fname;
    for (ff = 0; ff < nf; ++ff) {
        cpl_frameset_insert(
                ifs_adi->current_raw_frameset,
                __cutest_ifs_simple_adi_fixture_planet_cube(angle) );
        angle+=dangle;
    }

    CU_ASSERT_EQUAL( sph_ifs_simple_adi_run(ifs_adi), CPL_ERROR_NONE);

    cpl_frameset_delete(ifs_adi->current_raw_frameset);
    cpl_free(ifs_adi);
    return;
}

/*----------------------------------------------------------------------------*/
/*-                            INTERNAL HELPER FUNCTIONS                          */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to initiailise the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_ifs_simple_adi_init_testsuite() {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__,"moeller@mpia-hd.mpg.de",CPL_MSG_DEBUG);
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to clean the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_ifs_simple_adi_clean_testsuite() {
    sph_error_dump( SPH_ERROR_ERROR );
    return sph_end_test();
}

static
cpl_frame*
__cutest_ifs_simple_adi_fixture_planet_cube( double angle ) {
    cpl_frame*          input_cube_frame      = NULL;
    int                 np                      = 10;
    int                 nx                      = 128;
    int                 ny                      = 128;
    int                 pp                      = 0;
    cpl_image*          im                      = NULL;
    sph_cube*           incube                   = NULL;
    sph_point*          p                        = NULL;
    cpl_propertylist*   pl                      = NULL;

    input_cube_frame = sph_filemanager_create_temp_frame("testcube",
            "SOMETAG",
            CPL_FRAME_GROUP_CALIB);
    pl = cpl_propertylist_new();
    cpl_propertylist_update_double(pl,SPH_IFS_KEYWORD_ROT_ANGLE,angle);

    incube = sph_cube_new(cpl_frame_get_filename(input_cube_frame));

    angle = angle * CPL_MATH_PI / 180.0;

    for (pp = 0; pp < np; ++pp)
    {
        im = sph_test_image_tools_create_flat_image_double(nx,ny,0.0);
        sph_test_image_tools_add_gauss(im,nx/2.0, ny/2.0,10.0,1000.0);
        p = sph_point_new(20.0,0.0);
        sph_point_rotate_around(p,
                0.0,0.0,
                cos(angle),
                sin(angle),
                0.0, 0.0);
        sph_test_image_tools_add_gauss(
                im,nx/2.0 + p->x, ny/2.0 + p->y,
                1.0,10.0);
        sph_cube_append_image(
                cpl_frame_get_filename(
                        input_cube_frame),
                im,pl,0);
        cpl_image_delete(im); im = NULL;
        sph_point_delete(p); p = NULL;
    }

    sph_cube_finalise_file(cpl_frame_get_filename(input_cube_frame));
    im = sph_test_image_tools_create_flat_image_double(nx,ny,0.0);
    cpl_image_delete(im); im = NULL;
    sph_cube_delete(incube); incube = NULL;
    cpl_propertylist_delete(pl); pl = NULL;
    return input_cube_frame;
}
/**@}*/
