/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <Basic.h>
#include <CUnit.h>
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_ngc_ir_simulator.h"
#include "sph_test_pupilimage_creator.h"
#include "sph_test_image_tools.h"
#include "sph_common_keywords.h"
#include "sph_utils.h"
#include "sph_ifs_proc_apply_master_dark.h"

/*----------------------------------------------------------------------------*/
/**
 * @defgroup A CUnit Test Suite -- representing a collection of testcases
 * @par Synopsis:
 * @code
 * @endcode
 * @par Desciption:
 *
 * This module provides a collection of tests for one specific, distinct
 * module or set-up. The testing code is implemented using the CUnit
 * framework.
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*----------------------------------------------------------------------------*/
/*-                            INTERNAL HELPER FUNCTIONS                          */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to initiailise the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_init_testsuite() {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__,"moeller@mpia-hd.mpg.de",CPL_MSG_DEBUG);
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to clean the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_clean_testsuite() {
    sph_error_dump( SPH_ERROR_ERROR );
    sph_end_test();
    return 0;
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_ifs_proc_apply_master_dark_no_dit_info(void) {
    sph_master_frame*       mdark  = NULL;
    sph_master_frame*       rawframe = NULL;
    int                     nx      = 256;
    int                     ny      = 256;
    int                     bpix    = 0;

    mdark = sph_master_frame_new(nx,ny);
    sph_test_image_tools_add_gradient_x(mdark->image,-100.0,100.0);
    cpl_image_set(mdark->badpixelmap,100,100,1);
    cpl_image_set(mdark->image,100,100,100000000);

    rawframe = sph_master_frame_new(nx,ny);
    sph_test_image_tools_add_gradient_x(rawframe->image,-100.0,100.0);
    cpl_image_set(rawframe->image,100,100,100000000);

    CU_ASSERT_EQUAL(sph_ifs_proc_apply_master_dark(rawframe,mdark), CPL_ERROR_NONE);
    CU_ASSERT_EQUAL( cpl_image_get(rawframe->badpixelmap,100,100,&bpix), 1);
    CU_ASSERT_EQUAL( sph_master_frame_get_nbads(rawframe), 1);
    CU_ASSERT_DOUBLE_EQUAL( cpl_image_get_absflux(rawframe->image), 0.0, 0.0001);

    sph_master_frame_delete(rawframe); rawframe = NULL;
    sph_master_frame_delete(mdark); mdark = NULL;

    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_ifs_proc_apply_master_dark_dit_short_match(void) {
    sph_master_frame*       mdark  = NULL;
    sph_master_frame*       rawframe = NULL;
    int                     nx      = 256;
    int                     ny      = 256;
    int                     bpix    = 0;

    mdark = sph_master_frame_new(nx,ny);
    sph_test_image_tools_add_gradient_x(mdark->image,-100.0,100.0);
    cpl_image_set(mdark->badpixelmap,100,100,1);
    cpl_image_set(mdark->image,100,100,100000000);
    cpl_propertylist_update_double(mdark->properties,SPH_COMMON_KEYWORD_DIT,1.7);

    rawframe = sph_master_frame_new(nx,ny);
    sph_test_image_tools_add_gradient_x(rawframe->image,-100.0,100.0);
    cpl_image_set(rawframe->image,100,100,100000000);
    cpl_propertylist_update_double(rawframe->properties,SPH_COMMON_KEYWORD_DIT,1.7);

    CU_ASSERT_EQUAL(sph_ifs_proc_apply_master_dark(rawframe,mdark), CPL_ERROR_NONE);
    CU_ASSERT_EQUAL( cpl_image_get(rawframe->badpixelmap,100,100,&bpix), 1);
    CU_ASSERT_EQUAL( sph_master_frame_get_nbads(rawframe), 1);
    CU_ASSERT_DOUBLE_EQUAL( cpl_image_get_absflux(rawframe->image), 0.0, 0.0001);

    sph_master_frame_delete(rawframe); rawframe = NULL;
    sph_master_frame_delete(mdark); mdark = NULL;

    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_ifs_proc_apply_master_dark_dit_short_no_matchA(void) {
    sph_master_frame*       mdark  = NULL;
    sph_master_frame*       rawframe = NULL;
    int                     nx      = 256;
    int                     ny      = 256;

    mdark = sph_master_frame_new(nx,ny);
    sph_test_image_tools_add_gradient_x(mdark->image,-100.0,100.0);
    cpl_image_set(mdark->badpixelmap,100,100,1);
    cpl_image_set(mdark->image,100,100,100000000);
    cpl_propertylist_update_double(mdark->properties,SPH_COMMON_KEYWORD_DIT,1.7);

    rawframe = sph_master_frame_new(nx,ny);
    sph_test_image_tools_add_gradient_x(rawframe->image,-100.0,100.0);
    cpl_image_set(rawframe->image,100,100,100000000);
    cpl_propertylist_update_double(rawframe->properties,SPH_COMMON_KEYWORD_DIT,2.7);

    CU_ASSERT_EQUAL(sph_ifs_proc_apply_master_dark(rawframe,mdark), CPL_ERROR_ILLEGAL_INPUT);
    cpl_error_reset();
    sph_master_frame_delete(rawframe); rawframe = NULL;
    sph_master_frame_delete(mdark); mdark = NULL;

    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_ifs_proc_apply_master_dark_dit_short_no_matchB(void) {
    sph_master_frame*       mdark  = NULL;
    sph_master_frame*       rawframe = NULL;
    int                     nx      = 256;
    int                     ny      = 256;

    mdark = sph_master_frame_new(nx,ny);
    sph_test_image_tools_add_gradient_x(mdark->image,-100.0,100.0);
    cpl_image_set(mdark->badpixelmap,100,100,1);
    cpl_image_set(mdark->image,100,100,100000000);
    cpl_propertylist_update_double(mdark->properties,SPH_COMMON_KEYWORD_DIT,2.7);

    rawframe = sph_master_frame_new(nx,ny);
    sph_test_image_tools_add_gradient_x(rawframe->image,-100.0,100.0);
    cpl_image_set(rawframe->image,100,100,100000000);
    cpl_propertylist_update_double(rawframe->properties,SPH_COMMON_KEYWORD_DIT,1.7);

    CU_ASSERT_EQUAL(sph_ifs_proc_apply_master_dark(rawframe,mdark), CPL_ERROR_ILLEGAL_INPUT);
    cpl_error_reset();

    sph_master_frame_delete(rawframe); rawframe = NULL;
    sph_master_frame_delete(mdark); mdark = NULL;

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_ifs_proc_apply_master_dark_dit_long_no_match(void) {
    sph_master_frame*       mdark  = NULL;
    sph_master_frame*       rawframe = NULL;
    int                     nx      = 256;
    int                     ny      = 256;
    int                     bpix    = 0;
    cpl_image*              im = NULL;

    mdark = sph_master_frame_new(nx,ny);
    sph_test_image_tools_add_gradient_x(mdark->image,-100.0,100.0);
    cpl_image_set(mdark->badpixelmap,100,100,1);
    cpl_image_set(mdark->image,100,100,100000000);
    cpl_propertylist_update_double(mdark->properties,SPH_COMMON_KEYWORD_DIT,5.7);

    rawframe = sph_master_frame_new(nx,ny);
    sph_test_image_tools_add_gradient_x(rawframe->image,-100.0,100.0);
    cpl_image_multiply_scalar(rawframe->image,2.0);
    cpl_image_set(rawframe->image,100,100,100000000);
    cpl_propertylist_update_double(rawframe->properties,SPH_COMMON_KEYWORD_DIT,11.4);

    CU_ASSERT_EQUAL(sph_ifs_proc_apply_master_dark(rawframe,mdark), CPL_ERROR_NONE);
    CU_ASSERT_EQUAL( cpl_image_get(rawframe->badpixelmap,100,100,&bpix), 1);
    CU_ASSERT_EQUAL( sph_master_frame_get_nbads(rawframe), 1);
    im = sph_master_frame_get_image(rawframe,1);
    CU_ASSERT_DOUBLE_EQUAL( cpl_image_get_absflux(im), 0.0, 0.01);
    cpl_image_delete(im); im = NULL;
    sph_master_frame_delete(rawframe); rawframe = NULL;
    sph_master_frame_delete(mdark); mdark = NULL;

    return;
}


/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test MAIN function
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    int         result      = 0;
    CU_pSuite   pSuite      = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("Testuite IFS apply dark", cutest_init_testsuite,
                          cutest_clean_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "Testing apply dark w/o DIT info",
                             cutest_sph_ifs_proc_apply_master_dark_no_dit_info ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "Testing apply dark short dit (match)",
                             cutest_sph_ifs_proc_apply_master_dark_dit_short_match ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "Testing apply dark long dit (no match)",
                             cutest_sph_ifs_proc_apply_master_dark_dit_long_no_match ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
