
/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "cutest_zpl_basic_collapse_imaging.h"
#include "sph_zpl_basic_collapse_imaging.h"
#include "sph_zpl_utils.h"
#include "sph_utils.h"
#include "sph_zpl_keywords.h"
#include "sph_zpl_tags.h"
#include "sph_common_keywords.h"
#include "sph_keyword_manager.h"
#include "sph_filemanager.h"
#include "sph_keywordnode.h"
#include "sph_test_zpl_simulator.h"
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_image_tools.h"
#include <Basic.h>
#include <CUnit.h>


/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and
 *                                  associated functions.
 *
 */
/*----------------------------------------------------------------------------*/


const char cfgFile[] = "zpl_basic_collapse_imaging.cfg";

static
sph_keywordnode *cutest_util_keywordnode_create_cfg(cpl_frame* frame1, cpl_frame* frame2){
    sph_keywordnode *root = NULL;

    root = sph_keywordnode_new(NULL);
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);
    CU_ASSERT_EQUAL(sph_keywordnode_SetString(root, "FILE1.NAME", cpl_frame_get_filename( frame1 ) ),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE1.REMOVE1.PLANE", 0),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE1.REMOVE2.PLANE", 1),1);
    //CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE1.REMOVE3.PLANE", 2),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetString(root, "FILE2.NAME", cpl_frame_get_filename( frame2 ) ),1);
    //CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE2.REMOVE1.PLANE", 0),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE2.REMOVE2.PLANE", 1),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE2.REMOVE3.PLANE", 2),1);


    CU_ASSERT_EQUAL(sph_keywordnode_SaveConfiguration(root, cfgFile),1); // save keywords configuration on the disk
    return root;
}



static int cutest_init_basic_collapse_imaging_testsuite(void) {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     *--------------------------------------------------------------------*/
    sph_init_debug();
    return 0;
}

static int cutest_clean_basic_collapse_imaging_testsuite(void) {
    sph_error_dump( SPH_ERROR_ERROR );
    cpl_end();
    return 0;
}



void cutest_zpl_basic_collapse_imaging_run_1(void) {
    sph_zpl_basic_collapse_imaging*  	zpl_basic_collapse_imaging = NULL;
    cpl_parameterlist*          	 	inparams            = NULL;
    cpl_frameset*               		inframes            = NULL;
    cpl_frame*                    		zexpcube            = NULL;
    cpl_propertylist*					pl					= NULL;
    sph_error_code              		rerr                = CPL_ERROR_NONE;
    int                            		nframes             = 3;
    //int                         		nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         		ny                  = 100; // -//-

    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_imaging_cube( &zexpcube, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_COLLAPSE_IMAGING_PREPROC_RAW );

    rerr = cpl_frameset_insert( inframes, zexpcube );

    inparams = sph_zpl_basic_collapse_imaging_create_paramlist();
    zpl_basic_collapse_imaging = sph_zpl_basic_collapse_imaging_new( inframes, inparams );
    CU_ASSERT_PTR_NOT_NULL_FATAL( zpl_basic_collapse_imaging );

    //printf("Call basic_collapse_imaging_run\n...");
    rerr = sph_zpl_basic_collapse_imaging_run( zpl_basic_collapse_imaging );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    pl = cpl_propertylist_load( zpl_basic_collapse_imaging->basic_collapse_imaging_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 20.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 21.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;


    rerr = sph_zpl_basic_collapse_imaging_delete( zpl_basic_collapse_imaging );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    return;
}


void cutest_zpl_basic_collapse_imaging_run_2(void) {
    sph_zpl_basic_collapse_imaging*        		zpl_basic_collapse_imaging = NULL;
    cpl_parameterlist*          	 	inparams            = NULL;
    cpl_frameset*               		inframes            = NULL;
    cpl_frame*                    		zexpcube            = NULL;
    cpl_frame*                    		zexpcube1           = NULL;
    cpl_frame*							master_bias_imaging_frame		= NULL;
    cpl_frame*							master_dark_imaging_frame		= NULL;
    cpl_frame*							intensity_flat_imaging_frame	= NULL;
    cpl_propertylist*					pl					= NULL;
    sph_error_code              		rerr                = CPL_ERROR_NONE;
    int                            		nframes             = 3;
    int                         		nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    int                         		ny                  = 100; // -//-

    double								iframe				= 34.0;
    double 								pframe				= 39.0;
    double								iframe1				= 44.0;
    double								pframe1				= 49.0;
    double				                iframe_bias         = 4.0;
    double                				pframe_bias			= 1.0;


    //simulate 2 frames (cubes) of the pre-processed zpl exposure imaging
    inframes = cpl_frameset_new();

    //create first cube
    sph_test_simulator_create_zplpreproc_imaging_cube_set( &zexpcube, iframe, pframe, 1.0, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_COLLAPSE_IMAGING_PREPROC_RAW);
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );
    //create second cube of the pre-processed zpl exposure imaging
    sph_test_simulator_create_zplpreproc_imaging_cube_set( &zexpcube1, iframe1, pframe1, 1.0, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_BASIC_COLLAPSE_IMAGING_PREPROC_RAW);
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );

    //create master bias imaging (double image frame)
    master_bias_imaging_frame = sph_test_zpl_simulator_create_double_image_frame( "test_di_master_bias.fits.tmp", nx, ny, iframe_bias, pframe_bias );
    cpl_frame_set_tag( master_bias_imaging_frame, SPH_ZPL_TAG_BIAS_IMAGING_CALIB );
    cpl_frame_set_group( master_bias_imaging_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, master_bias_imaging_frame );

    //create master dark imaging (double image frame)
    master_dark_imaging_frame = sph_test_zpl_simulator_create_double_image_frame( "test_di_master_dark.fits.tmp", nx, ny, iframe_bias+10., pframe_bias+10. );
    cpl_frame_set_tag( master_dark_imaging_frame, SPH_ZPL_TAG_DARK_IMAGING_CALIB );
    cpl_frame_set_group( master_dark_imaging_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, master_dark_imaging_frame );

    //create intensity flat imaging (double image frame)
    intensity_flat_imaging_frame = sph_test_zpl_simulator_create_double_image_frame( "test_di_intensity_flat.tmp", nx, ny, 2., 3. );
    cpl_frame_set_tag( intensity_flat_imaging_frame, SPH_ZPL_TAG_IFF_IMAGING_CALIB );
    cpl_frame_set_group( intensity_flat_imaging_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, intensity_flat_imaging_frame );

    inparams = sph_zpl_basic_collapse_imaging_create_paramlist();
    zpl_basic_collapse_imaging = sph_zpl_basic_collapse_imaging_new( inframes, inparams );
    rerr = sph_zpl_basic_collapse_imaging_run( zpl_basic_collapse_imaging );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );

    pl = cpl_propertylist_load( zpl_basic_collapse_imaging->basic_collapse_imaging_outfilename , 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 11.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 11.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for collapse imaging with rawframes and rmv planes cfgfile
 */
/*----------------------------------------------------------------------------*/

void cutest_zpl_basic_collapse_imaging_run_3(void) {
    sph_zpl_basic_collapse_imaging*  	zpl_basic_collapse_imaging = NULL;
    cpl_parameterlist*          	 	inparams            = NULL;
    cpl_frameset*               		inframes            = NULL;
    cpl_frame*                    		zexpcube            = NULL;
    cpl_frame*							zexpcube1			= NULL;
    cpl_propertylist*					pl					= NULL;
    sph_error_code              		rerr                = CPL_ERROR_NONE;
    int                            		nframes             = 3;
    //int                         		nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         		ny                  = 100; // -//-
    double								iframe				= 34.0;
    double 								pframe				= 39.0;
    double								iframe1				= 44.0;
    double								pframe1				= 49.0;
    sph_keywordnode*					root				= NULL;


    inframes = cpl_frameset_new();


    //create first cube
    sph_test_simulator_create_zplpreproc_imaging_cube_set( &zexpcube, iframe, pframe, 1.0, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_COLLAPSE_IMAGING_PREPROC_RAW);
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );
    //create second cube of the pre-processed zpl exposure imaging

    sph_test_simulator_create_zplpreproc_imaging_cube_set( &zexpcube1, iframe1, pframe1, 1.0, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_BASIC_COLLAPSE_IMAGING_PREPROC_RAW);
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    root = cutest_util_keywordnode_create_cfg( zexpcube, zexpcube1);
    inparams = sph_zpl_basic_collapse_imaging_create_paramlist();
    zpl_basic_collapse_imaging = sph_zpl_basic_collapse_imaging_new( inframes, inparams );
    CU_ASSERT_PTR_NOT_NULL_FATAL( zpl_basic_collapse_imaging );


    //printf("Call basic_collapse_imaging_run\n...");
    rerr = sph_zpl_basic_collapse_imaging_run( zpl_basic_collapse_imaging );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    pl = cpl_propertylist_load( zpl_basic_collapse_imaging->basic_collapse_imaging_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 40.0, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 45.0, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;


    rerr = sph_zpl_basic_collapse_imaging_delete( zpl_basic_collapse_imaging );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );
    sph_keywordnode_destroy( root );
    remove(cfgFile);

    return;
}


/**
 @brief Unit test for the sph_zpl_basic_collapse_imaging
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_basic_collapse_imaging_run_4(void) {
    sph_zpl_basic_collapse_imaging*     zpl_basic_collapse_imaging  = NULL;
    cpl_parameterlist*          		inparams            = NULL;
    cpl_frameset*               		inframes            = NULL;
    cpl_frame*                  		zexpcube            = NULL;
    cpl_frame*							zexpcube_1			= NULL;
    sph_error_code              		rerr                = CPL_ERROR_NONE;
    cpl_propertylist*	 pl						= NULL;
//    cpl_frame*			 master_bias_imaging_frame		= NULL;
//    cpl_frame*			 master_dark_imaging_frame		= NULL;
//    double				            iframe_bias         = 4.0;
//    double             				pframe_bias			= 1.0;
//    double				            iframe_dark         = 10.0;
//    double             				pframe_dark			= 11.0;
    int                         nz	                = 3;
    int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    int                         ny                  = 100; // -//-
    double                     	iframe    = 34;
    double                     	pframe    = 39;
    double                     	iframe1   = 44;
    double                    	pframe1   = 49;
    double 						ovsc[4]		= {4.0, 1.0, 6.0, 2.0};
    double 						ovsc_1[4]	= {9.0, 1.0, 10.0, 2.0};
//    double 						ovsc[4]		= {0.0, 1.0, 0.0, 2.0};
//    double 						ovsc_1[4]	= {0.0, 1.0, 0.0, 2.0};

    double 						adu1_zero = 0;
    double						adu2_zero = 0;

    char				outfilename[256];


    inparams = sph_zpl_basic_collapse_imaging_create_paramlist();


    inframes = cpl_frameset_new();

    // zpl frame cube with overscan table
    zexpcube = sph_test_simulator_create_zpl_exposure_imaging_overscans_cube( nx, ny, nz,
			  	  	  	  	  	"test_collapse_imaging_zplexp_ovsc_cube",  iframe, pframe, ovsc,	1, 1);
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_COLLAPSE_IMAGING_PREPROC_RAW);
    rerr = cpl_frameset_insert( inframes, zexpcube );

    // zpl frame cube with overscan table
    zexpcube_1 = sph_test_simulator_create_zpl_exposure_imaging_overscans_cube( nx, ny, nz,
			  	  	  	  	  	"test_collapse_imaging_zplexp_ovsc_cube_1",  iframe1,  pframe1, ovsc_1, 1, 0 );
    cpl_frame_set_tag( zexpcube_1, SPH_ZPL_TAG_BASIC_COLLAPSE_IMAGING_PREPROC_RAW);
    rerr = cpl_frameset_insert( inframes, zexpcube_1 );


    //create master bias imaging (double image frame)
    //master_bias_imaging_frame = sph_test_zpl_simulator_create_double_image_frame( "test_di_master_bias.fits.tmp", nx, ny, iframe_bias, pframe_bias );
    //cpl_frame_set_tag( master_bias_imaging_frame, SPH_ZPL_TAG_BIAS_IMAGING_CALIB );
    //cpl_frame_set_group( master_bias_imaging_frame, CPL_FRAME_GROUP_CALIB );
    //rerr = cpl_frameset_insert( inframes, master_bias_imaging_frame );

    //create master dark imaging (double image frame)
    //master_dark_imaging_frame = sph_test_zpl_simulator_create_double_image_frame( "test_di_master_dark.fits.tmp", nx, ny, iframe_dark, pframe_dark );
    //cpl_frame_set_tag( master_dark_imaging_frame, SPH_ZPL_TAG_DARK_IMAGING_CALIB );
    //cpl_frame_set_group( master_dark_imaging_frame, CPL_FRAME_GROUP_CALIB );
    //rerr = cpl_frameset_insert( inframes, master_dark_imaging_frame );


    //printf("raw frame: %s\n", cpl_frame_get_filename(cpl_frameset_get_first( zpl_basic_collapse->rawframes )));
    //printf("rawframes #: %d\n", cpl_frameset_get_size( zpl_basic_collapse->rawframes ));

    zpl_basic_collapse_imaging = sph_zpl_basic_collapse_imaging_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_basic_collapse_imaging);

    rerr = sph_zpl_basic_collapse_imaging_run( zpl_basic_collapse_imaging );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    //test for non-normalized output
    pl = cpl_propertylist_load( zpl_basic_collapse_imaging->basic_collapse_imaging_outfilename, 0);

    adu1_zero = (34.-ovsc[0] +35.-(ovsc[0]+1)+36.-(ovsc[0]+2)+44.-ovsc_1[0]+45.-ovsc_1[0]+46.-ovsc_1[0])/6.;
    adu2_zero = (34.-ovsc[2] +35.-(ovsc[2]+1)+36.-(ovsc[2]+2)+44.-ovsc_1[2]+45.-ovsc_1[2]+46.-ovsc_1[2])/6.;
    //printf("0.5*(adu1_zero+adu2_zero) = %f\n",  0.5*(adu1_zero+adu2_zero));
    //printf("adu1_zero = %f\n",  adu1_zero );
    //printf("adu2_zero = %f\n",  adu2_zero );
    CU_ASSERT_DOUBLE_EQUAL( cpl_propertylist_get_double( pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME ),
    														 0.5*(adu1_zero+adu2_zero), 1e-10);

    adu1_zero = (39.-ovsc[0] +40.-(ovsc[0]+1)+41.-(ovsc[0]+2)+49.-ovsc_1[0]+50.-ovsc_1[0]+51.-ovsc_1[0])/6.;
    adu2_zero = (39.-ovsc[2] +40.-(ovsc[2]+1)+41.-(ovsc[2]+2)+49.-ovsc_1[2]+50.-ovsc_1[2]+51.-ovsc_1[2])/6.;

    CU_ASSERT_DOUBLE_EQUAL( cpl_propertylist_get_double( pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME ),
    														 0.5*(adu1_zero+adu2_zero), 1e-10);
    cpl_propertylist_delete( pl );


    rerr = sph_zpl_basic_collapse_imaging_delete( zpl_basic_collapse_imaging );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    //Destroy here input parameters and inframes explicitly
    cpl_frameset_delete( inframes );
    cpl_parameterlist_delete(inparams);


    return;
}


/**
 @brief Unit test for the sph_zpl_basic_collapse_imaging
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_basic_collapse_imaging_run_5(void) {
    sph_zpl_basic_collapse_imaging*     zpl_basic_collapse_imaging  = NULL;
    cpl_parameterlist*          		inparams            = NULL;
    cpl_frameset*               		inframes            = NULL;
    cpl_frame*                  		zexpcube            = NULL;
    cpl_frame*							zexpcube_1			= NULL;
    sph_error_code              		rerr                = CPL_ERROR_NONE;
//    int                         		nframes             = 6;
//    int                         		nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
//    int                         		ny                  = 100; // -//-
//    double              dV						= 100.0;
//    double 				 image_odd				= 2500.0;
//    double				 image_even				= 2000.0;
    cpl_propertylist*	 pl						= NULL;
    cpl_frame*			 master_bias_imaging_frame		= NULL;
    cpl_frame*			 master_dark_imaging_frame		= NULL;
    double				            iframe_bias         = 4.0;
    double             				pframe_bias			= 1.0;
    double				            iframe_dark         = 10.0;
    double             				pframe_dark			= 11.0;
    int                         nz	                = 3;
    int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    int                         ny                  = 100; // -//-
    double                     	iframe    = 34;
    double                     	pframe    = 39;
    double                     	iframe1   = 44;
    double                    	pframe1   = 49;
    double 						ovsc[4]		= {4.0, 1.0, 6.0, 2.0};
    double 						ovsc_1[4]	= {9.0, 1.0, 10.0, 2.0};
//    double 						ovsc[4]		= {0.0, 1.0, 0.0, 2.0};
//    double 						ovsc_1[4]	= {0.0, 1.0, 0.0, 2.0};

    double 						adu1_zero = 0;
    double						adu2_zero = 0;
    char				outfilename[256];
    sph_keywordnode*					root				= NULL;


    inparams = sph_zpl_basic_collapse_imaging_create_paramlist();


    inframes = cpl_frameset_new();

    // zpl frame cube with overscan table
    zexpcube = sph_test_simulator_create_zpl_exposure_imaging_overscans_cube( nx, ny, nz,
			  	  	  	  	  	"test_collapse_imaging_zplexp_ovsc_cube",  iframe, pframe, ovsc,	1, 1);
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_COLLAPSE_IMAGING_PREPROC_RAW);
    rerr = cpl_frameset_insert( inframes, zexpcube );

    // zpl frame cube with overscan table
    zexpcube_1 = sph_test_simulator_create_zpl_exposure_imaging_overscans_cube( nx, ny, nz,
			  	  	  	  	  	"test_collapse_imaging_zplexp_ovsc_cube_1",  iframe1,  pframe1, ovsc_1, 1, 0 );
    cpl_frame_set_tag( zexpcube_1, SPH_ZPL_TAG_BASIC_COLLAPSE_IMAGING_PREPROC_RAW);
    rerr = cpl_frameset_insert( inframes, zexpcube_1 );


    //create master bias imaging (double image frame)
    //master_bias_imaging_frame = sph_test_zpl_simulator_create_double_image_frame( "test_di_master_bias.fits.tmp", nx, ny, iframe_bias, pframe_bias );
    //cpl_frame_set_tag( master_bias_imaging_frame, SPH_ZPL_TAG_BIAS_IMAGING_CALIB );
    //cpl_frame_set_group( master_bias_imaging_frame, CPL_FRAME_GROUP_CALIB );
    //rerr = cpl_frameset_insert( inframes, master_bias_imaging_frame );

    //create master dark imaging (double image frame)
    //master_dark_imaging_frame = sph_test_zpl_simulator_create_double_image_frame( "test_di_master_dark.fits.tmp", nx, ny, iframe_dark, pframe_dark );
    //cpl_frame_set_tag( master_dark_imaging_frame, SPH_ZPL_TAG_DARK_IMAGING_CALIB );
    //cpl_frame_set_group( master_dark_imaging_frame, CPL_FRAME_GROUP_CALIB );
    //rerr = cpl_frameset_insert( inframes, master_dark_imaging_frame );


    //printf("raw frame: %s\n", cpl_frame_get_filename(cpl_frameset_get_first( zpl_basic_collapse->rawframes )));
    //printf("rawframes #: %d\n", cpl_frameset_get_size( zpl_basic_collapse->rawframes ));

    root = cutest_util_keywordnode_create_cfg( zexpcube, zexpcube_1);

    zpl_basic_collapse_imaging = sph_zpl_basic_collapse_imaging_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_basic_collapse_imaging);

    rerr = sph_zpl_basic_collapse_imaging_run( zpl_basic_collapse_imaging );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    //test for non-normalized output
    pl = cpl_propertylist_load( zpl_basic_collapse_imaging->basic_collapse_imaging_outfilename, 0);

    //adu1_zero = (34.-ovsc[0] +35.-(ovsc[0]+1)+36.-(ovsc[0]+2)+44.-ovsc_1[0]+45.-ovsc_1[0]+46.-ovsc_1[0])/6.;
    //adu2_zero = (34.-ovsc[2] +35.-(ovsc[2]+1)+36.-(ovsc[2]+2)+44.-ovsc_1[2]+45.-ovsc_1[2]+46.-ovsc_1[2])/6.;
    adu1_zero = (36.-(ovsc[0]+2)+44.-ovsc_1[0])/2.;
    adu2_zero = (36.-(ovsc[2]+2)+44.-ovsc_1[2])/2.;

    //printf("0.5*(adu1_zero+adu2_zero) = %f\n",  0.5*(adu1_zero+adu2_zero));
    //printf("adu1_zero = %f\n",  adu1_zero );
    //printf("adu2_zero = %f\n",  adu2_zero );
    CU_ASSERT_DOUBLE_EQUAL( cpl_propertylist_get_double( pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME ),
    														 0.5*(adu1_zero+adu2_zero), 1e-10);

    adu1_zero = (41.-(ovsc[0]+2)+49.-ovsc_1[0])/2.;
    adu2_zero = (41.-(ovsc[2]+2)+49.-ovsc_1[2])/2.;


    CU_ASSERT_DOUBLE_EQUAL( cpl_propertylist_get_double( pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME ),
    														 0.5*(adu1_zero+adu2_zero), 1e-10);
    cpl_propertylist_delete( pl );

    rerr = sph_zpl_basic_collapse_imaging_delete( zpl_basic_collapse_imaging );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    //Destroy here input parameters and inframes explicitly
    cpl_frameset_delete( inframes );
    cpl_parameterlist_delete(inparams);

    sph_keywordnode_destroy( root );
    remove(cfgFile);

    return;
}






/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of basic collapse imaging recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    int result       = 0;

    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("basic_collapse_imaging_test", cutest_init_basic_collapse_imaging_testsuite,
                          cutest_clean_basic_collapse_imaging_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */


    if ( NULL == CU_add_test(pSuite, "cutest_zpl_basic_collapse_imaging_run_1: pure rawframes/pre-processed as inputframes",
                             cutest_zpl_basic_collapse_imaging_run_1 ) )
    {
       CU_cleanup_registry();
      return CU_get_error();
    }

    if ( NULL == CU_add_test(pSuite, "cutest_zpl_basic_collapse_imaging_run_2: rawframes/pre-processed and all detector calibrations",
    						 cutest_zpl_basic_collapse_imaging_run_2 ) )
     {
        CU_cleanup_registry();
        return CU_get_error();
     }

    if ( NULL == CU_add_test(pSuite, "cutest_zpl_basic_collapse_imaging_run_3: rawframes/pre-processed  and rmv cfg file",
    						 cutest_zpl_basic_collapse_imaging_run_3 ) )
     {
        CU_cleanup_registry();
        return CU_get_error();
     }

    if ( NULL == CU_add_test(pSuite, "cutest_zpl_basic_collapse_imaging_run_4: rawframes/pre-processed with overscan tables",
    						 cutest_zpl_basic_collapse_imaging_run_4 ) )
     {
        CU_cleanup_registry();
        return CU_get_error();
     }


    if ( NULL == CU_add_test(pSuite, "cutest_zpl_basic_collapse_imaging_run_5: rawframes/pre-processed with overscan tables and with rmv cfg file",
    						 cutest_zpl_basic_collapse_imaging_run_5 ) )
     {
        CU_cleanup_registry();
        return CU_get_error();
     }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
