/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "cutest_zpl_basic_collapse_polarization.h"
#include "sph_zpl_basic_collapse_polarization.h"
#include "sph_zpl_utils.h"
#include "sph_utils.h"
#include "sph_zpl_keywords.h"
#include "sph_zpl_tags.h"
#include "sph_common_keywords.h"
#include "sph_keyword_manager.h"
#include "sph_filemanager.h"
#include "sph_test_zpl_simulator.h"
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_image_tools.h"
#include "sph_keywordnode.h"
#include <Basic.h>
#include <CUnit.h>


/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and
 *                                  associated functions.
 *
 */
/*----------------------------------------------------------------------------*/


const char cfgFile[] = "zpl_basic_collapse_polarization.cfg";

static
sph_keywordnode *cutest_util_keywordnode_create_cfg(){
    sph_keywordnode *root = NULL;

    root = sph_keywordnode_new(NULL);
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);
    CU_ASSERT_EQUAL(sph_keywordnode_SetString(root, "FILE1.NAME", "test_basic_pol.fits.tmp" ),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE1.REMOVE1.PLANE", 0),1);
    //CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE1.REMOVE2.PLANE", 1),1);
    //CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE1.REMOVE3.PLANE", 2),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetString(root, "FILE2.NAME", "test_basic_pol_1.fits.tmp" ),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE2.REMOVE1.PLANE", 0),1);
    //CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE2.REMOVE2.PLANE", 1),1);
    //CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE2.REMOVE3.PLANE", 2),1);


    CU_ASSERT_EQUAL(sph_keywordnode_SaveConfiguration(root, cfgFile),1); // save keywords configuration on the disk
    return root;
}

static
sph_keywordnode *cutest_util_keywordnode_create_cfg1(){
    sph_keywordnode *root = NULL;

    root = sph_keywordnode_new(NULL);
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);
    CU_ASSERT_EQUAL(sph_keywordnode_SetString(root, "FILE1.NAME", "test_basic_pol.fits.tmp" ),1);
    //CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE1.REMOVE1.PLANE", 0),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE1.REMOVE2.PLANE", 1),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE1.REMOVE3.PLANE", 2),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetString(root, "FILE2.NAME", "test_basic_pol_1.fits.tmp" ),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE2.REMOVE1.PLANE", 0),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE2.REMOVE2.PLANE", 1),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE2.REMOVE3.PLANE", 2),1);


    CU_ASSERT_EQUAL(sph_keywordnode_SaveConfiguration(root, cfgFile),1); // save keywords configuration on the disk
    return root;
}

static
sph_keywordnode *cutest_util_keywordnode_create_cfg2(const char* filename1, const char* filename2){
    sph_keywordnode *root = NULL;

    root = sph_keywordnode_new(NULL);
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);
    CU_ASSERT_EQUAL(sph_keywordnode_SetString(root, "FILE1.NAME", filename1 ),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE1.REMOVE1.PLANE", 0),1);
    //CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE1.REMOVE2.PLANE", 1),1);
    //CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE1.REMOVE3.PLANE", 2),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetString(root, "FILE2.NAME", filename2 ),1);
    //CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE2.REMOVE1.PLANE", 0),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE2.REMOVE2.PLANE", 1),1);
    //CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "FILE2.REMOVE3.PLANE", 2),1);


    CU_ASSERT_EQUAL(sph_keywordnode_SaveConfiguration(root, cfgFile),1); // save keywords configuration on the disk
    return root;
}


static
void cutest_util_create_head_file_hwpz0( const char* infile){
	FILE* 				fp;
	char				outfilename[256];

	sprintf( outfilename, "%s.head", infile );
	fp = fopen( outfilename, "w");
	if ( fp != NULL ){
		fprintf(fp, "ESO DRS ZPL HWPZ SETTING = 0.0\n");
		fclose(fp);
	}

}

static
void cutest_util_create_head_file_hwpz45( const char* infile){
	FILE* 				fp;
	char				outfilename[256];

	sprintf( outfilename, "%s.head", infile );
	fp = fopen( outfilename, "w");
	if ( fp != NULL ){
		fprintf(fp, "ESO DRS ZPL HWPZ SETTING = 45.0\n");
		fclose(fp);
	}

}

static
sph_keyword_manager* cutest_util_create_cfg_file(){
	FILE* 				fp;
    sph_keyword_manager*    km      = NULL;


	fp = fopen( "sphere.cfg", "w");
    fprintf( fp, "SPH.KEYWORDS.ESO_DRS_ZPL_HWPZ_SETTING.NAME = \"ESO DRS ZPL HWPZ SETTING\"\n");
    fprintf( fp, "SPH.KEYWORDS.ESO_DRS_ZPL_HWPZ_SETTING.TYPE = \"DOUBLE\"\n");
    fprintf( fp, "SPH.KEYWORDS.ESO_DRS_ZPL_HWPZ_SETTING.VALUE = \"0.00\"\n");
    fclose(fp);

    km = sph_keyword_manager_new();
    sph_keyword_manager_read_config( "sphere.cfg" );

    return km;
}


static int cutest_init_basic_collapse_polarization_testsuite(void) {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     *--------------------------------------------------------------------*/
    sph_init_debug();
    return 0;
}

static int cutest_clean_basic_collapse_polarization_testsuite(void) {
    sph_error_dump( SPH_ERROR_ERROR );
    cpl_end();
    return 0;
}


/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_basic_collapse_polarization with +Q(hwpz = 0.0) and -Q(hwpz=45.0) rawframes
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_basic_collapse_polarization_run_1(void) {
    sph_zpl_basic_collapse_polarization*     zpl_basic_collapse_polarization  = NULL;
    cpl_parameterlist*          	inparams            = NULL;
    cpl_frameset*               	inframes            = NULL;
    cpl_frame*                  	zexpcube            = NULL;
    cpl_frame*						zexpcube1			= NULL;
    sph_error_code              	rerr                = CPL_ERROR_NONE;
    int                         	nframes             = 3;
    //int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;

    inparams = sph_zpl_basic_collapse_polarization_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_hwpz( &zexpcube,"test_basic_pol.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, 0.0, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_hwpz( &zexpcube1, "test_basic_pol_1.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, 45.0, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    zpl_basic_collapse_polarization = sph_zpl_basic_collapse_polarization_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_basic_collapse_polarization);


    rerr = sph_zpl_basic_collapse_polarization_run( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    //pl = cpl_propertylist_new();
    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_outfilename , 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 400.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 240.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    rerr = sph_zpl_basic_collapse_polarization_delete( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    return;
}


/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_basic_collapse_polarization only with rawframes without hwpz keywords
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_basic_collapse_polarization_run_2(void) {
    sph_zpl_basic_collapse_polarization*     zpl_basic_collapse_polarization  = NULL;
    cpl_parameterlist*          	inparams            = NULL;
    cpl_frameset*               	inframes            = NULL;
    cpl_frame*                  	zexpcube            = NULL;
    cpl_frame*						zexpcube1			= NULL;
    sph_error_code              	rerr                = CPL_ERROR_NONE;
    int                         	nframes             = 3;
    //int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;

    inparams = sph_zpl_basic_collapse_polarization_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube,"test_basic_pol.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube1, "test_basic_pol_1.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    zpl_basic_collapse_polarization = sph_zpl_basic_collapse_polarization_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_basic_collapse_polarization);

    rerr = sph_zpl_basic_collapse_polarization_run( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_outfilename , 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 400.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 40.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    rerr = sph_zpl_basic_collapse_polarization_delete( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    return;
}



/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_basic_collapse_polarization: rawframes (without hwpz keywords) with bias, dark, iff, pff & modem calibrations
 */
/*----------------------------------------------------------------------------*/

void cutest_zpl_basic_collapse_polarization_run_3(void) {
    sph_zpl_basic_collapse_polarization*     zpl_basic_collapse_polarization  = NULL;
    cpl_parameterlist*          inparams            = NULL;
    cpl_frameset*               inframes            = NULL;
    cpl_frame*                  zexpcube            = NULL;
    cpl_frame*					zexpcube1			= NULL;
    sph_error_code              rerr                = CPL_ERROR_NONE;
    int                         nframes             = 3;
    int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;
    cpl_frame*			 master_bias_frame		= NULL;
    cpl_frame*			 master_dark_frame		= NULL;
    cpl_frame*			 intensity_flat_frame	= NULL;
    cpl_frame*			 collapse_polarization_flat_frame	= NULL;
    cpl_frame*			 modem_frame			= NULL;



    inparams = sph_zpl_basic_collapse_polarization_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube,"test_basic_pol.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube1, "test_basic_pol_1.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    //simualate master bias
    zero_odd = 10; zero_even = 11; pi_odd = 12; pi_even = 14;
    master_bias_frame = sph_test_zpl_simulator_create_quad_image_frame("test_master_bias_calib.fits.tmp", nx, ny, zero_odd, zero_even, pi_odd, pi_even);
    cpl_frame_set_tag( master_bias_frame, SPH_ZPL_TAG_BIAS_CALIB );
    cpl_frame_set_group( master_bias_frame, CPL_FRAME_GROUP_CALIB );
    //printf("master bias tag: %s\n", cpl_frame_get_tag(master_bias_frame));
    rerr = cpl_frameset_insert( inframes, master_bias_frame );

    //simualate master dark
    zero_odd = 1; zero_even = 2; pi_odd = 3; pi_even = 5;
    master_dark_frame = sph_test_zpl_simulator_create_quad_image_frame("test_master_dark_calib.fits.tmp", nx, ny, zero_odd, zero_even, pi_odd, pi_even);
    cpl_frame_set_tag( master_dark_frame, SPH_ZPL_TAG_DARK_CALIB );
    cpl_frame_set_group( master_dark_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, master_dark_frame );

    //simualate intensity flat
    zero_odd = 2; zero_even = 2; pi_odd = 2; pi_even = 2;
    intensity_flat_frame = sph_test_zpl_simulator_create_quad_image_frame("test_master_intensity_flat_calib.fits.tmp", nx, ny, zero_odd, zero_even, pi_odd, pi_even);
    cpl_frame_set_tag( intensity_flat_frame, SPH_ZPL_TAG_IFF_CALIB );
    cpl_frame_set_group( intensity_flat_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, intensity_flat_frame );

    //simualate collapse_polarization flat
    collapse_polarization_flat_frame = sph_test_zpl_simulator_create_double_image_frame("test_master_collapse_polarization_flat_calib.fits.tmp", nx, ny, 200.0, 100.0 );
    cpl_frame_set_tag( collapse_polarization_flat_frame, SPH_ZPL_TAG_PFF_CALIB );
    cpl_frame_set_group( collapse_polarization_flat_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, collapse_polarization_flat_frame );

    //simualate modem efficiency
    modem_frame = sph_test_zpl_simulator_create_masterframe_frame("test_modem_calib.fits.tmp", nx, ny, 0.5 );
    cpl_frame_set_tag( modem_frame, SPH_ZPL_TAG_MODEM_EFF_CALIB );
    cpl_frame_set_group( modem_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, modem_frame );


    zpl_basic_collapse_polarization = sph_zpl_basic_collapse_polarization_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_basic_collapse_polarization);

    rerr = sph_zpl_basic_collapse_polarization_run( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );

    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_outfilename , 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 171.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), -133.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;


    rerr = sph_zpl_basic_collapse_polarization_delete( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );


    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_basic_collapse_polarization: rawframes without hwpz keywords,  bias, dark, iff, pff & modem calibrations
 */
/*----------------------------------------------------------------------------*/

void cutest_zpl_basic_collapse_polarization_run_4(void) {
    sph_zpl_basic_collapse_polarization*     zpl_basic_collapse_polarization  = NULL;
    cpl_parameterlist*          inparams            = NULL;
    cpl_frameset*               inframes            = NULL;
    cpl_frame*                  zexpcube            = NULL;
    cpl_frame*					zexpcube1			= NULL;
    sph_error_code              rerr                = CPL_ERROR_NONE;
    int                         nframes             = 3;
    int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;
    cpl_frame*			 master_bias_frame		= NULL;
    cpl_frame*			 master_dark_frame		= NULL;
    cpl_frame*			 intensity_flat_frame	= NULL;
    cpl_frame*			 collapse_polarization_flat_frame	= NULL;
    cpl_frame*			 modem_frame			= NULL;



    inparams = sph_zpl_basic_collapse_polarization_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_hwpz( &zexpcube,"test_basic_pol.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, 0.0, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_hwpz( &zexpcube1, "test_basic_pol_1.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, 45.0, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    //simualate master bias
    zero_odd = 10; zero_even = 11; pi_odd = 12; pi_even = 14;
    master_bias_frame = sph_test_zpl_simulator_create_quad_image_frame("test_master_bias_calib.fits.tmp", nx, ny, zero_odd, zero_even, pi_odd, pi_even);
    cpl_frame_set_tag( master_bias_frame, SPH_ZPL_TAG_BIAS_CALIB );
    cpl_frame_set_group( master_bias_frame, CPL_FRAME_GROUP_CALIB );
    //printf("master bias tag: %s\n", cpl_frame_get_tag(master_bias_frame));
    rerr = cpl_frameset_insert( inframes, master_bias_frame );

    //simualate master dark
    zero_odd = 1; zero_even = 2; pi_odd = 3; pi_even = 5;
    master_dark_frame = sph_test_zpl_simulator_create_quad_image_frame("test_master_dark_calib.fits.tmp", nx, ny, zero_odd, zero_even, pi_odd, pi_even);
    cpl_frame_set_tag( master_dark_frame, SPH_ZPL_TAG_DARK_CALIB );
    cpl_frame_set_group( master_dark_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, master_dark_frame );

    //simualate intensity flat
    zero_odd = 2; zero_even = 2; pi_odd = 2; pi_even = 2;
    intensity_flat_frame = sph_test_zpl_simulator_create_quad_image_frame("test_master_intensity_flat_calib.fits.tmp", nx, ny, zero_odd, zero_even, pi_odd, pi_even);
    cpl_frame_set_tag( intensity_flat_frame, SPH_ZPL_TAG_IFF_CALIB );
    cpl_frame_set_group( intensity_flat_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, intensity_flat_frame );

    //simualate collapse_polarization flat
    collapse_polarization_flat_frame = sph_test_zpl_simulator_create_double_image_frame("test_master_collapse_polarization_flat_calib.fits.tmp", nx, ny, 200.0, 100.0 );
    cpl_frame_set_tag( collapse_polarization_flat_frame, SPH_ZPL_TAG_PFF_CALIB );
    cpl_frame_set_group( collapse_polarization_flat_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, collapse_polarization_flat_frame );

    //simualate modem efficiency
    modem_frame = sph_test_zpl_simulator_create_masterframe_frame("test_modem_calib.fits.tmp", nx, ny, 0.5 );
    cpl_frame_set_tag( modem_frame, SPH_ZPL_TAG_MODEM_EFF_CALIB );
    cpl_frame_set_group( modem_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, modem_frame );


    zpl_basic_collapse_polarization = sph_zpl_basic_collapse_polarization_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_basic_collapse_polarization);

    rerr = sph_zpl_basic_collapse_polarization_run( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_plus_outfilename , 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 171.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 107.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_minus_outfilename , 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 171.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), -373.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;


    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_outfilename , 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 171.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 240.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;


    rerr = sph_zpl_basic_collapse_polarization_delete( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );


    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_basic_collapse_polarization: rawframes with the same hwpz keywords,  bias, dark, iff, pff & modem calibrations
 */
/*----------------------------------------------------------------------------*/

void cutest_zpl_basic_collapse_polarization_run_5(void) {
    sph_zpl_basic_collapse_polarization*     zpl_basic_collapse_polarization  = NULL;
    cpl_parameterlist*          inparams            = NULL;
    cpl_frameset*               inframes            = NULL;
    cpl_frame*                  zexpcube            = NULL;
    cpl_frame*					zexpcube1			= NULL;
    sph_error_code              rerr                = CPL_ERROR_NONE;
    int                         nframes             = 3;
    int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;
    cpl_frame*			 master_bias_frame		= NULL;
    cpl_frame*			 master_dark_frame		= NULL;
    cpl_frame*			 intensity_flat_frame	= NULL;
    cpl_frame*			 collapse_polarization_flat_frame	= NULL;
    cpl_frame*			 modem_frame			= NULL;



    inparams = sph_zpl_basic_collapse_polarization_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_hwpz( &zexpcube,"test_basic_pol.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, 0.0, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_hwpz( &zexpcube1, "test_basic_pol_1.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, 0.0, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    //simualate master bias
    zero_odd = 10; zero_even = 11; pi_odd = 12; pi_even = 14;
    master_bias_frame = sph_test_zpl_simulator_create_quad_image_frame("test_master_bias_calib.fits.tmp", nx, ny, zero_odd, zero_even, pi_odd, pi_even);
    cpl_frame_set_tag( master_bias_frame, SPH_ZPL_TAG_BIAS_CALIB );
    cpl_frame_set_group( master_bias_frame, CPL_FRAME_GROUP_CALIB );
    //printf("master bias tag: %s\n", cpl_frame_get_tag(master_bias_frame));
    rerr = cpl_frameset_insert( inframes, master_bias_frame );

    //simualate master dark
    zero_odd = 1; zero_even = 2; pi_odd = 3; pi_even = 5;
    master_dark_frame = sph_test_zpl_simulator_create_quad_image_frame("test_master_dark_calib.fits.tmp", nx, ny, zero_odd, zero_even, pi_odd, pi_even);
    cpl_frame_set_tag( master_dark_frame, SPH_ZPL_TAG_DARK_CALIB );
    cpl_frame_set_group( master_dark_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, master_dark_frame );

    //simualate intensity flat
    zero_odd = 2; zero_even = 2; pi_odd = 2; pi_even = 2;
    intensity_flat_frame = sph_test_zpl_simulator_create_quad_image_frame("test_master_intensity_flat_calib.fits.tmp", nx, ny, zero_odd, zero_even, pi_odd, pi_even);
    cpl_frame_set_tag( intensity_flat_frame, SPH_ZPL_TAG_IFF_CALIB );
    cpl_frame_set_group( intensity_flat_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, intensity_flat_frame );

    //simualate collapse_polarization flat
    collapse_polarization_flat_frame = sph_test_zpl_simulator_create_double_image_frame("test_master_collapse_polarization_flat_calib.fits.tmp", nx, ny, 200.0, 100.0 );
    cpl_frame_set_tag( collapse_polarization_flat_frame, SPH_ZPL_TAG_PFF_CALIB );
    cpl_frame_set_group( collapse_polarization_flat_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, collapse_polarization_flat_frame );

    //simualate modem efficiency
    modem_frame = sph_test_zpl_simulator_create_masterframe_frame("test_modem_calib.fits.tmp", nx, ny, 0.5 );
    cpl_frame_set_tag( modem_frame, SPH_ZPL_TAG_MODEM_EFF_CALIB );
    cpl_frame_set_group( modem_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, modem_frame );


    zpl_basic_collapse_polarization = sph_zpl_basic_collapse_polarization_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_basic_collapse_polarization);

    rerr = sph_zpl_basic_collapse_polarization_run( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_outfilename , 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 171.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), -133.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;



    rerr = sph_zpl_basic_collapse_polarization_delete( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );


    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_basic_collapse_polarization only with rawframes without hwpz keywords + rmv planes cfgfile
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_basic_collapse_polarization_run_6(void) {
    sph_zpl_basic_collapse_polarization*     zpl_basic_collapse_polarization  = NULL;
    cpl_parameterlist*          	inparams            = NULL;
    cpl_frameset*               	inframes            = NULL;
    cpl_frame*                  	zexpcube            = NULL;
    cpl_frame*						zexpcube1			= NULL;
    sph_error_code              	rerr                = CPL_ERROR_NONE;
    int                         	nframes             = 3;
    //int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;

    sph_keywordnode*	root					= NULL;

    inparams = sph_zpl_basic_collapse_polarization_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube,"test_basic_pol.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube1, "test_basic_pol_1.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    root = cutest_util_keywordnode_create_cfg();
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);


    zpl_basic_collapse_polarization = sph_zpl_basic_collapse_polarization_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_basic_collapse_polarization);

    rerr = sph_zpl_basic_collapse_polarization_run( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_outfilename , 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 400, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 40, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    rerr = sph_zpl_basic_collapse_polarization_delete( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );
    remove(cfgFile);

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_basic_collapse_polarization only with rawframes without hwpz keywords + rmv planes cfgfile1
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_basic_collapse_polarization_run_7(void) {
    sph_zpl_basic_collapse_polarization*     zpl_basic_collapse_polarization  = NULL;
    cpl_parameterlist*          	inparams            = NULL;
    cpl_frameset*               	inframes            = NULL;
    cpl_frame*                  	zexpcube            = NULL;
    cpl_frame*						zexpcube1			= NULL;
    sph_error_code              	rerr                = CPL_ERROR_NONE;
    int                         	nframes             = 3;
    //int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;

    sph_keywordnode*	root					= NULL;

    inparams = sph_zpl_basic_collapse_polarization_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube,"test_basic_pol.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube1, "test_basic_pol_1.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    root = cutest_util_keywordnode_create_cfg1();
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);


    zpl_basic_collapse_polarization = sph_zpl_basic_collapse_polarization_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_basic_collapse_polarization);

    rerr = sph_zpl_basic_collapse_polarization_run( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_outfilename , 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 396, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 280, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    rerr = sph_zpl_basic_collapse_polarization_delete( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );
    //remove(cfgFile);

    return;
}


/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_basic_collapse_polarization with +Q(hwpz = 0.0) and -Q(hwpz=45.0) rawframes
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_basic_collapse_polarization_run_8(void) {
    sph_zpl_basic_collapse_polarization*     zpl_basic_collapse_polarization  = NULL;
    cpl_parameterlist*          	inparams            = NULL;
    cpl_frameset*               	inframes            = NULL;
    cpl_frame*                  	zexpcube            = NULL;
    cpl_frame*						zexpcube1			= NULL;
    sph_error_code              	rerr                = CPL_ERROR_NONE;
    int                         	nframes             = 3;
    //int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;

    sph_keywordnode*	root					= NULL;

    inparams = sph_zpl_basic_collapse_polarization_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_hwpz( &zexpcube,"test_basic_pol.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, 0.0, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_hwpz( &zexpcube1, "test_basic_pol_1.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, 45.0, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    root = cutest_util_keywordnode_create_cfg();

    zpl_basic_collapse_polarization = sph_zpl_basic_collapse_polarization_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_basic_collapse_polarization);


    rerr = sph_zpl_basic_collapse_polarization_run( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    //pl = cpl_propertylist_new();
    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_outfilename , 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 400.0, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 240.0, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    rerr = sph_zpl_basic_collapse_polarization_delete( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );
    remove(cfgFile);

    sph_keywordnode_destroy( root );

    return;
}


/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_basic_collapse_polarization with rawframes without
  hwpz keywords but with cfg and head-files
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_basic_collapse_polarization_run_9(void) {
    sph_zpl_basic_collapse_polarization*     zpl_basic_collapse_polarization  = NULL;
    cpl_parameterlist*          	inparams            = NULL;
    cpl_frameset*               	inframes            = NULL;
    cpl_frame*                  	zexpcube            = NULL;
    cpl_frame*						zexpcube1			= NULL;
    sph_error_code              	rerr                = CPL_ERROR_NONE;
    int                         	nframes             = 3;
    //int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;

    sph_keyword_manager*        keyman      = NULL;

    //create sphere.cfg
    keyman = cutest_util_create_cfg_file();
    CU_ASSERT_PTR_NOT_NULL_FATAL( keyman );

    inparams = sph_zpl_basic_collapse_polarization_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube,"test_basic_pol.fits", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    //create .head file
    cutest_util_create_head_file_hwpz0("test_basic_pol");


    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube1, "test_basic_pol_1.fits", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );

    //create .head file
    cutest_util_create_head_file_hwpz45("test_basic_pol_1");


    zpl_basic_collapse_polarization = sph_zpl_basic_collapse_polarization_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_basic_collapse_polarization);

    rerr = sph_zpl_basic_collapse_polarization_run( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_outfilename , 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 400.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 240.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    rerr = sph_zpl_basic_collapse_polarization_delete( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    sph_keyword_manager_delete();

    return;
}


/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_basic_collapse_polarization
 @note It is fed by raw pre-processed frame with overscan table with hwpz=0(+Q) and hwpz=45(-Q) keywords
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_basic_collapse_polarization_run_10(void){
    sph_zpl_basic_collapse_polarization*        zpl_basic_collapse_polarization     = NULL;
    cpl_parameterlist*          inparams            = NULL;
    cpl_frameset*               inframes            = NULL;
    cpl_frame*                  zexpcube_minus      = NULL;
    cpl_frame*                  zexpcube_plus       = NULL;
    cpl_propertylist*			pl					= NULL;
    sph_error_code              rerr                = CPL_ERROR_NONE;
    int                         nz	                = 3;
    int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    int                         ny                  = 100; // -//-
    int                     	zero_odd    = 169;
    int                     	zero_even   = 29;
    int                     	pi_odd      = 29;
    int                     	pi_even     = 169;
    double 						ovsc[8]		= {5.0, 1.0, 6.0, 2.0, 7.0, 3.0, 8.0, 4.0};
    //double 					ovsc[8]		= {0.0, 1.0, 0.0, 2.0, 0.0, 3.0, 0.0, 4.0};

    int                     	zero_odd_1   = 172;
    int                     	zero_even_1  = 32;
    int                     	pi_odd_1     = 32;
    int                    		pi_even_1    = 172;
    double 						ovsc_1[8]	 = {15.0, 1.0, 16.0, 2.0, 17.0, 3.0, 18.0, 4.0};
    //double 					ovsc_1[8]		= {0.0, 1.0, 0.0, 2.0, 0.0, 3.0, 0.0, 4.0};
    double 						adu1_zero_odd_plus = 0;
    double						adu2_zero_odd_plus = 0;
    double						adu1_zero_even_plus = 0;
    double 						adu2_zero_even_plus = 0;
    double 						adu1_pi_odd_plus = 0;
    double						adu2_pi_odd_plus = 0;
    double						adu1_pi_even_plus = 0;
    double 						adu2_pi_even_plus = 0;
    double						iframe_plus = 0.0;
    double 						pframe_plus = 0.0;

    double 						adu1_zero_odd_minus = 0;
    double						adu2_zero_odd_minus = 0;
    double						adu1_zero_even_minus = 0;
    double 						adu2_zero_even_minus = 0;
    double 						adu1_pi_odd_minus = 0;
    double						adu2_pi_odd_minus = 0;
    double						adu1_pi_even_minus = 0;
    double 						adu2_pi_even_minus = 0;
    double						iframe_minus = 0.0;
    double 						pframe_minus = 0.0;

    double						iframe_adu1_plus = 0.0;
    double 						pframe_adu1_plus = 0.0;
    double						iframe_adu2_plus = 0.0;
    double 						pframe_adu2_plus = 0.0;

    double						iframe_adu1_minus = 0.0;
    double 						pframe_adu1_minus = 0.0;
    double						iframe_adu2_minus = 0.0;
    double 						pframe_adu2_minus = 0.0;


    double						pframe = 0.0;
    double						iframe = 0.0;


    inparams = sph_zpl_basic_collapse_polarization_create_paramlist();
    inframes = cpl_frameset_new();

    // zpl frame cube with overscan table
    zexpcube_plus = sph_test_simulator_create_zpl_exposure_overscans_cube_set_hwpz( nx, ny, nz,
			  	  	  	  	  	"test_basic_collapse_polarization_zplexp_ovsc_cube",  zero_odd,  zero_even,  pi_odd, pi_even, ovsc,
			  	  	  	  	  	1, 1, 0.0);
    cpl_frame_set_tag( zexpcube_plus, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW);
    rerr = cpl_frameset_insert( inframes, zexpcube_plus );

    // zpl frame cube with overscan table
    zexpcube_minus = sph_test_simulator_create_zpl_exposure_overscans_cube_set_hwpz( nx, ny, nz,
			  	  	  	  	  	"test_basic_collapse_polarization_zplexp_ovsc_cube_1",  zero_odd_1,  zero_even_1,  pi_odd_1,
			  	  	  	  	  	pi_even_1, ovsc_1, 1, 0, 45.0 );
    cpl_frame_set_tag( zexpcube_minus, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW);
    rerr = cpl_frameset_insert( inframes, zexpcube_minus );


    zpl_basic_collapse_polarization = sph_zpl_basic_collapse_polarization_new( inframes, inparams );
    CU_ASSERT_PTR_NOT_NULL_FATAL( zpl_basic_collapse_polarization );

    rerr = sph_zpl_basic_collapse_polarization_run( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );

    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_outfilename, 0);

    //calculate expected values
    adu1_zero_odd_plus = (169.-ovsc[0] +170.-(ovsc[0]+1)+171.-(ovsc[0]+2));
    adu2_zero_odd_plus = (169.-ovsc[2] +170.-(ovsc[2]+1)+171.-(ovsc[2]+2));

    adu1_zero_even_plus = (29.-ovsc[0] +30.-(ovsc[0]+1)+31.-(ovsc[0]+2));
    adu2_zero_even_plus = (29.-ovsc[2] +30.-(ovsc[2]+1)+31.-(ovsc[2]+2));

    adu1_pi_odd_plus = (29.-ovsc[4] +30.-(ovsc[4]+1)+31.-(ovsc[4]+2));
    adu2_pi_odd_plus = (29.-ovsc[6] +30.-(ovsc[6]+1)+31.-(ovsc[6]+2));

    adu1_pi_even_plus = (169.-ovsc[4] +170.-(ovsc[4]+1)+171.-(ovsc[4]+2));
    adu2_pi_even_plus = (169.-ovsc[6] +170.-(ovsc[6]+1)+171.-(ovsc[6]+2));

    adu1_zero_odd_minus = (172.-ovsc_1[0]+173.-ovsc_1[0]+174.-ovsc_1[0]);
    adu2_zero_odd_minus = (172.-ovsc_1[2]+173.-ovsc_1[2]+174.-ovsc_1[2]);

    adu1_zero_even_minus = (32.-ovsc_1[0]+33.-ovsc_1[0]+34.-ovsc_1[0]);
    adu2_zero_even_minus = (32.-ovsc_1[2]+33.-ovsc_1[2]+34.-ovsc_1[2]);

    adu1_pi_odd_minus = (32.-ovsc_1[4]+33.-ovsc_1[4]+34.-ovsc_1[4]);
    adu2_pi_odd_minus = (32.-ovsc_1[6]+33.-ovsc_1[6]+34.-ovsc_1[6]);

    adu1_pi_even_minus = (172.-ovsc_1[4]+173.-ovsc_1[4]+174.-ovsc_1[4]);
    adu2_pi_even_minus = (172.-ovsc_1[6]+173.-ovsc_1[6]+174.-ovsc_1[6]);

    iframe_adu1_plus = (adu1_zero_odd_plus)/3. + (adu1_zero_even_plus)/3. +
    		      (adu1_pi_odd_plus)/3. + (adu1_pi_even_plus)/3.;

    iframe_adu2_plus = (adu2_zero_odd_plus)/3. + (adu2_zero_even_plus)/3. +
    		      (adu2_pi_odd_plus)/3. + (adu2_pi_even_plus)/3.;

    pframe_adu1_plus = (adu1_zero_odd_plus)/3. - (adu1_zero_even_plus)/3. -
    		      (adu1_pi_odd_plus)/3. + (adu1_pi_even_plus)/3.;

    pframe_adu2_plus = (adu2_zero_odd_plus)/3. - (adu2_zero_even_plus)/3. -
    		      (adu2_pi_odd_plus)/3. + (adu2_pi_even_plus)/3.;


    iframe_adu1_minus = (adu1_zero_odd_minus)/3. + (adu1_zero_even_minus)/3. +
    		      (adu1_pi_odd_minus)/3. + (adu1_pi_even_minus)/3.;

    iframe_adu2_minus = (adu2_zero_odd_minus)/3. + (adu2_zero_even_minus)/3. +
    		      (adu2_pi_odd_minus)/3. + (adu2_pi_even_minus)/3.;

    pframe_adu1_minus = (adu1_zero_odd_minus)/3. - (adu1_zero_even_minus)/3. -
    		      (adu1_pi_odd_minus)/3. + (adu1_pi_even_minus)/3.;

    pframe_adu2_minus = (adu2_zero_odd_minus)/3. - (adu2_zero_even_minus)/3. -
    		      (adu2_pi_odd_minus)/3. + (adu2_pi_even_minus)/3.;


    iframe_plus = (iframe_adu1_plus + iframe_adu2_plus)/2.;
    pframe_plus = (pframe_adu1_plus + pframe_adu2_plus)/2.;
    iframe_minus = (iframe_adu1_minus + iframe_adu2_minus)/2.;
    pframe_minus = (pframe_adu1_minus + pframe_adu2_minus)/2.;

    iframe = (iframe_plus + iframe_minus)/2.;
    pframe = (pframe_plus - pframe_minus)/2.;


    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), iframe, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), pframe, 1e-10);

    cpl_propertylist_delete( pl ); pl = NULL;

    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_plus_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), iframe_plus, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), pframe_plus, 1e-10);

    cpl_propertylist_delete( pl ); pl = NULL;

    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_minus_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), iframe_minus, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), pframe_minus, 1e-10);

    cpl_propertylist_delete( pl ); pl = NULL;

    rerr = sph_zpl_basic_collapse_polarization_delete( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    //Destroy here input parameters and inframes explicitly
    cpl_frameset_delete( inframes );
    cpl_parameterlist_delete(inparams);

    return;

}

/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_basic_collapse_polarization
 @note It is fed by raw pre-processed frame with overscan table (hwpz=0(+Q) and hwpz=45(-Q)) and cfg file to remove planes
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_basic_collapse_polarization_run_11(void){
    sph_zpl_basic_collapse_polarization*        zpl_basic_collapse_polarization     = NULL;
    cpl_parameterlist*          inparams            = NULL;
    cpl_frameset*               inframes            = NULL;
    cpl_frame*                  zexpcube_minus      = NULL;
    cpl_frame*                  zexpcube_plus       = NULL;
    cpl_propertylist*			pl					= NULL;
    sph_error_code              rerr                = CPL_ERROR_NONE;
    int                         nz	                = 3;
    int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    int                         ny                  = 100; // -//-
    int                     	zero_odd    = 169;
    int                     	zero_even   = 29;
    int                     	pi_odd      = 29;
    int                     	pi_even     = 169;
    double 						ovsc[8]		= {5.0, 1.0, 6.0, 2.0, 7.0, 3.0, 8.0, 4.0};
    //double 					ovsc[8]		= {0.0, 1.0, 0.0, 2.0, 0.0, 3.0, 0.0, 4.0};

    int                     	zero_odd_1   = 172;
    int                     	zero_even_1  = 32;
    int                     	pi_odd_1     = 32;
    int                    		pi_even_1    = 172;
    double 						ovsc_1[8]	 = {15.0, 1.0, 16.0, 2.0, 17.0, 3.0, 18.0, 4.0};
    //double 					ovsc_1[8]		= {0.0, 1.0, 0.0, 2.0, 0.0, 3.0, 0.0, 4.0};
    double 						adu1_zero_odd_plus = 0;
    double						adu2_zero_odd_plus = 0;
    double						adu1_zero_even_plus = 0;
    double 						adu2_zero_even_plus = 0;
    double 						adu1_pi_odd_plus = 0;
    double						adu2_pi_odd_plus = 0;
    double						adu1_pi_even_plus = 0;
    double 						adu2_pi_even_plus = 0;
    double						iframe_plus = 0.0;
    double 						pframe_plus = 0.0;

    double 						adu1_zero_odd_minus = 0;
    double						adu2_zero_odd_minus = 0;
    double						adu1_zero_even_minus = 0;
    double 						adu2_zero_even_minus = 0;
    double 						adu1_pi_odd_minus = 0;
    double						adu2_pi_odd_minus = 0;
    double						adu1_pi_even_minus = 0;
    double 						adu2_pi_even_minus = 0;
    double						iframe_minus = 0.0;
    double 						pframe_minus = 0.0;

    double						iframe_adu1_plus = 0.0;
    double 						pframe_adu1_plus = 0.0;
    double						iframe_adu2_plus = 0.0;
    double 						pframe_adu2_plus = 0.0;

    double						iframe_adu1_minus = 0.0;
    double 						pframe_adu1_minus = 0.0;
    double						iframe_adu2_minus = 0.0;
    double 						pframe_adu2_minus = 0.0;


    double						pframe = 0.0;
    double						iframe = 0.0;

    sph_keywordnode*	root					= NULL;

    inparams = sph_zpl_basic_collapse_polarization_create_paramlist();
    inframes = cpl_frameset_new();

    // zpl frame cube with overscan table
    zexpcube_plus = sph_test_simulator_create_zpl_exposure_overscans_cube_set_hwpz( nx, ny, nz,
    		"test_basic_collapse_polarization_zplexp_ovsc_cube",  zero_odd,  zero_even,  pi_odd, pi_even, ovsc,
			  	  	  	  	  	1, 1, 0.0);
    cpl_frame_set_tag( zexpcube_plus, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW);
    rerr = cpl_frameset_insert( inframes, zexpcube_plus );

    // zpl frame cube with overscan table
    zexpcube_minus = sph_test_simulator_create_zpl_exposure_overscans_cube_set_hwpz( nx, ny, nz,
    		"test_basic_collapse_polarization_zplexp_ovsc_cube_1",  zero_odd_1,  zero_even_1,  pi_odd_1,
			  	  	  	  	  	pi_even_1, ovsc_1, 1, 0, 45.0 );
    cpl_frame_set_tag( zexpcube_minus, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW);
    rerr = cpl_frameset_insert( inframes, zexpcube_minus );

    root = cutest_util_keywordnode_create_cfg2(cpl_frame_get_filename(zexpcube_plus),
    		cpl_frame_get_filename(zexpcube_minus));


    zpl_basic_collapse_polarization = sph_zpl_basic_collapse_polarization_new( inframes, inparams );
    CU_ASSERT_PTR_NOT_NULL_FATAL( zpl_basic_collapse_polarization );

    rerr = sph_zpl_basic_collapse_polarization_run( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );

    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_outfilename, 0);

    //calculate expected values
    adu1_zero_odd_plus = ( 0. + 170.-(ovsc[0]+1)+171.-(ovsc[0]+2));
    adu2_zero_odd_plus = ( 0. +170.-(ovsc[2]+1)+171.-(ovsc[2]+2));

    adu1_zero_even_plus = (0. +30.-(ovsc[0]+1)+31.-(ovsc[0]+2));
    adu2_zero_even_plus = (0. +30.-(ovsc[2]+1)+31.-(ovsc[2]+2));

    adu1_pi_odd_plus = (0. +30.-(ovsc[4]+1)+31.-(ovsc[4]+2));
    adu2_pi_odd_plus = (0. +30.-(ovsc[6]+1)+31.-(ovsc[6]+2));

    adu1_pi_even_plus = (0. +170.-(ovsc[4]+1)+171.-(ovsc[4]+2));
    adu2_pi_even_plus = (0. +170.-(ovsc[6]+1)+171.-(ovsc[6]+2));

    adu1_zero_odd_minus = (172.-ovsc_1[0]+ 0. +174.-ovsc_1[0]);
    adu2_zero_odd_minus = (172.-ovsc_1[2]+ 0. +174.-ovsc_1[2]);

    adu1_zero_even_minus = (32.-ovsc_1[0]+ 0. +34.-ovsc_1[0]);
    adu2_zero_even_minus = (32.-ovsc_1[2]+ 0. +34.-ovsc_1[2]);

    adu1_pi_odd_minus = (32.-ovsc_1[4]+ 0. +34.-ovsc_1[4]);
    adu2_pi_odd_minus = (32.-ovsc_1[6]+ 0. +34.-ovsc_1[6]);

    adu1_pi_even_minus = (172.-ovsc_1[4]+ 0. +174.-ovsc_1[4]);
    adu2_pi_even_minus = (172.-ovsc_1[6]+ 0. +174.-ovsc_1[6]);

    iframe_adu1_plus = (adu1_zero_odd_plus)/2. + (adu1_zero_even_plus)/2. +
    		      (adu1_pi_odd_plus)/2. + (adu1_pi_even_plus)/2.;

    iframe_adu2_plus = (adu2_zero_odd_plus)/2. + (adu2_zero_even_plus)/2. +
    		      (adu2_pi_odd_plus)/2. + (adu2_pi_even_plus)/2.;

    pframe_adu1_plus = (adu1_zero_odd_plus)/2. - (adu1_zero_even_plus)/2. -
    		      (adu1_pi_odd_plus)/2. + (adu1_pi_even_plus)/2.;

    pframe_adu2_plus = (adu2_zero_odd_plus)/2. - (adu2_zero_even_plus)/2. -
    		      (adu2_pi_odd_plus)/2. + (adu2_pi_even_plus)/2.;


    iframe_adu1_minus = (adu1_zero_odd_minus)/2. + (adu1_zero_even_minus)/2. +
    		      (adu1_pi_odd_minus)/2. + (adu1_pi_even_minus)/2.;

    iframe_adu2_minus = (adu2_zero_odd_minus)/2. + (adu2_zero_even_minus)/2. +
    		      (adu2_pi_odd_minus)/2. + (adu2_pi_even_minus)/2.;

    pframe_adu1_minus = (adu1_zero_odd_minus)/2. - (adu1_zero_even_minus)/2. -
    		      (adu1_pi_odd_minus)/2. + (adu1_pi_even_minus)/2.;

    pframe_adu2_minus = (adu2_zero_odd_minus)/2. - (adu2_zero_even_minus)/2. -
    		      (adu2_pi_odd_minus)/2. + (adu2_pi_even_minus)/2.;


    iframe_plus = (iframe_adu1_plus + iframe_adu2_plus)/2.;
    pframe_plus = (pframe_adu1_plus + pframe_adu2_plus)/2.;
    iframe_minus = (iframe_adu1_minus + iframe_adu2_minus)/2.;
    pframe_minus = (pframe_adu1_minus + pframe_adu2_minus)/2.;

    iframe = (iframe_plus + iframe_minus)/2.;
    pframe = (pframe_plus - pframe_minus)/2.;


    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), iframe, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), pframe, 1e-10);

    cpl_propertylist_delete( pl ); pl = NULL;

    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_plus_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), iframe_plus, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), pframe_plus, 1e-10);

    cpl_propertylist_delete( pl ); pl = NULL;

    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_minus_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), iframe_minus, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), pframe_minus, 1e-10);

    cpl_propertylist_delete( pl ); pl = NULL;

    rerr = sph_zpl_basic_collapse_polarization_delete( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    //Destroy here input parameters and inframes explicitly
    cpl_frameset_delete( inframes );
    cpl_parameterlist_delete(inparams);
    sph_keywordnode_destroy( root );

    remove(cfgFile);

    return;

}



/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_basic_collapse_polarization with rawframes without
  hwpz keywords but with cfg and head-files. Modem efficiency calibration is used
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_basic_collapse_polarization_run_12(void) {
    sph_zpl_basic_collapse_polarization*     zpl_basic_collapse_polarization  = NULL;
    cpl_parameterlist*          	inparams            = NULL;
    cpl_frameset*               	inframes            = NULL;
    cpl_frame*                  	zexpcube            = NULL;
    cpl_frame*						zexpcube1			= NULL;
    cpl_frame*						modem_frame			= NULL;
    sph_error_code              	rerr                = CPL_ERROR_NONE;
    int                         	nframes             = 3;
    //int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;

    sph_keyword_manager*        keyman      = NULL;

    //create sphere.cfg
    keyman = cutest_util_create_cfg_file();
    CU_ASSERT_PTR_NOT_NULL_FATAL( keyman );

    inparams = sph_zpl_basic_collapse_polarization_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube,"test_basic_pol.fits", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    //create .head file
    cutest_util_create_head_file_hwpz0("test_basic_pol");


    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube1, "test_basic_pol_1.fits", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_BASIC_COLL_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );

    //create .head file
    cutest_util_create_head_file_hwpz45("test_basic_pol_1");

    //simualate modem efficiency
    modem_frame = sph_test_zpl_simulator_create_masterframe_frame("test_modem_calib.fits.tmp", 100, 100, -0.5 );
    cpl_frame_set_tag( modem_frame, SPH_ZPL_TAG_MODEM_EFF_CALIB );
    cpl_frame_set_group( modem_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, modem_frame );

    zpl_basic_collapse_polarization = sph_zpl_basic_collapse_polarization_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_basic_collapse_polarization);

    rerr = sph_zpl_basic_collapse_polarization_run( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    pl = cpl_propertylist_load( zpl_basic_collapse_polarization->basic_collapse_polarization_outfilename , 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 400.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), -240.0/2., 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    rerr = sph_zpl_basic_collapse_polarization_delete( zpl_basic_collapse_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    sph_keyword_manager_delete();

    return;
}



/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_basic_collapse_polarization recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    int result       = 0;

    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("basic_collapse_polarization_test", cutest_init_basic_collapse_polarization_testsuite,
                          cutest_clean_basic_collapse_polarization_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */

    if ( NULL == CU_add_test(pSuite, "sph_basic_collapse_polarization_run_1: 2 cube-rawframes with +Q[hwpz=0.0] and -Q[hwpz=45.0] ",
                             cutest_zpl_basic_collapse_polarization_run_1 ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    if ( NULL == CU_add_test(pSuite, "sph_basic_collapse_polarization_run_2: 2 cube-rawframes  without hwpz keywords",
                              cutest_zpl_basic_collapse_polarization_run_2 ) )
    {
        CU_cleanup_registry();
        return CU_get_error();
     }

    if ( NULL == CU_add_test(pSuite, "sph_basic_collapse_polarization_run_3: 2 cube-rawframes  without hwpz keywords and all calibrations",
                              cutest_zpl_basic_collapse_polarization_run_3 ) )
    {
        CU_cleanup_registry();
        return CU_get_error();
     }

    if ( NULL == CU_add_test(pSuite, "sph_basic_collapse_polarization_run_4: 2 cube-rawframes  with +Q[hwpz=0.0] and -Q[hwpz=45.0] and all calibrations",
                              cutest_zpl_basic_collapse_polarization_run_4 ) )
    {
        CU_cleanup_registry();
        return CU_get_error();
     }

    if ( NULL == CU_add_test(pSuite, "sph_basic_collapse_polarization_run_5: 2 cube-rawframes with the same hwpz keywords[hwpz=0.0] and all calibrations ",
                              cutest_zpl_basic_collapse_polarization_run_5 ) )
    {
        CU_cleanup_registry();
        return CU_get_error();
     }

    if ( NULL == CU_add_test(pSuite, "sph_basic_collapse_polarization_run_6: 2 cube-rawframes  without hwpz keywords + rmv planes cfg file",
                              cutest_zpl_basic_collapse_polarization_run_6 ) )
    {
        CU_cleanup_registry();
        return CU_get_error();
     }
//
//    if ( NULL == CU_add_test(pSuite, "sph_basic_collapse_polarization_run_7: 2 cube-rawframes  without hwpz keywords + rmv planes cfg1 file",
//                              cutest_zpl_basic_collapse_polarization_run_7 ) )
//    {
//        CU_cleanup_registry();
//        return CU_get_error();
//    }

    if ( NULL == CU_add_test(pSuite, "sph_basic_collapse_polarization_run_8: 2 cube-rawframes with  with +Q[hwpz=0.0] and -Q[hwpz=45.0] + rmvplanes cfg",
                             cutest_zpl_basic_collapse_polarization_run_8 ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    if ( NULL == CU_add_test(pSuite, "sph_basic_collapse_polarization_run_9: 2 cube-rawframes without hwpz keyword and sphere.cfg and head files",
                             cutest_zpl_basic_collapse_polarization_run_9 ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_basic_collapse_polarization_run_10: "
    		"2 cube-rawframes (+Q[hwpz=0] and -Q[hwpz=45]) with overscan tables",
                             cutest_zpl_basic_collapse_polarization_run_10 ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    if ( NULL == CU_add_test(pSuite, "sph_basic_collapse_polarization_run_11: "
    		"2 cube-rawframes (+Q[hwpz=0] and -Q[hwpz=45]) with overscan tables + rmv planes cfg2 file",
                             cutest_zpl_basic_collapse_polarization_run_11 ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    if ( NULL == CU_add_test(pSuite, "sph_basic_collapse_polarization_run_12: "
    		"2 cube-rawframes (+Q[hwpz=0] and -Q[hwpz=45]) with head-files and modem efficiency",
                             cutest_zpl_basic_collapse_polarization_run_11 ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
