/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "cutest_zpl_basic_polarization.h"
#include "sph_zpl_basic_polarization.h"
#include "sph_zpl_utils.h"
#include "sph_utils.h"
#include "sph_zpl_keywords.h"
#include "sph_zpl_tags.h"
#include "sph_common_keywords.h"
#include "sph_keyword_manager.h"
#include "sph_filemanager.h"
#include "sph_test_zpl_simulator.h"
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_image_tools.h"
#include <Basic.h>
#include <CUnit.h>


/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and
 *                                  associated functions.
 *
 */
/*----------------------------------------------------------------------------*/


static int cutest_init_basic_polarization_testsuite(void) {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     *--------------------------------------------------------------------*/
    sph_init_debug();
    return 0;
}

static int cutest_clean_basic_polarization_testsuite(void) {
    sph_error_dump( SPH_ERROR_ERROR );
    cpl_end();
    return 0;
}



/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_basic_polarization only with rawframes
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_basic_polarization_run_1(void) {
    sph_zpl_basic_polarization*     zpl_basic_polarization  = NULL;
    cpl_parameterlist*          	inparams            = NULL;
    cpl_frameset*               	inframes            = NULL;
    cpl_frame*                  	zexpcube            = NULL;
    cpl_frame*						zexpcube1			= NULL;
    cpl_frame*						outframe			= NULL;
    sph_error_code              	rerr                = CPL_ERROR_NONE;
    int                         	nframes             = 3;
    //int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;

    inparams = sph_zpl_basic_polarization_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube,"test_basic_pol.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube1, "test_basic_pol_1.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_BASIC_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    zpl_basic_polarization = sph_zpl_basic_polarization_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_basic_polarization);

    rerr = sph_zpl_basic_polarization_run( zpl_basic_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    outframe = cpl_frameset_get_first( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 396.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 280.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    outframe = cpl_frameset_get_next( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 400.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 280.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    outframe = cpl_frameset_get_next( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 404.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 280.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    outframe = cpl_frameset_get_next( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 396.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), -200.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    outframe = cpl_frameset_get_next( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 400.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), -200.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    outframe = cpl_frameset_get_next( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 404.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), -200.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    //cpl_frame_delete( outframe );
    rerr = sph_zpl_basic_polarization_delete( zpl_basic_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    return;
}


/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_basic_polarization: rawframes,  bias, dark, iff, pff & modem calibrations
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_basic_polarization_run_2(void) {
    sph_zpl_basic_polarization*     zpl_basic_polarization  = NULL;
    cpl_parameterlist*          inparams            = NULL;
    cpl_frameset*               inframes            = NULL;
    cpl_frame*                  zexpcube            = NULL;
    cpl_frame*					zexpcube1			= NULL;
    sph_error_code              rerr                = CPL_ERROR_NONE;
    int                         nframes             = 3;
    int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;
    cpl_frame*			 master_bias_frame		= NULL;
    cpl_frame*			 master_dark_frame		= NULL;
    cpl_frame*			 intensity_flat_frame	= NULL;
    cpl_frame*			 polarization_flat_frame	= NULL;
    cpl_frame*			 modem_frame			= NULL;
    cpl_frame*			 outframe				= NULL;



    inparams = sph_zpl_basic_polarization_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube,"test_basic_pol.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube1, "test_basic_pol_1.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_BASIC_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    //simualate master bias
    zero_odd = 10; zero_even = 11; pi_odd = 12; pi_even = 14;
    master_bias_frame = sph_test_zpl_simulator_create_quad_image_frame("test_master_bias_calib.fits.tmp", nx, ny, zero_odd, zero_even, pi_odd, pi_even);
    cpl_frame_set_tag( master_bias_frame, SPH_ZPL_TAG_BIAS_CALIB );
    cpl_frame_set_group( master_bias_frame, CPL_FRAME_GROUP_CALIB );
    //printf("master bias tag: %s\n", cpl_frame_get_tag(master_bias_frame));
    rerr = cpl_frameset_insert( inframes, master_bias_frame );

    //simualate master dark
    zero_odd = 1; zero_even = 2; pi_odd = 3; pi_even = 5;
    master_dark_frame = sph_test_zpl_simulator_create_quad_image_frame("test_master_dark_calib.fits.tmp", nx, ny, zero_odd, zero_even, pi_odd, pi_even);
    cpl_frame_set_tag( master_dark_frame, SPH_ZPL_TAG_DARK_CALIB );
    cpl_frame_set_group( master_dark_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, master_dark_frame );

    //simualate intensity flat
    zero_odd = 2; zero_even = 2; pi_odd = 2; pi_even = 2;
    intensity_flat_frame = sph_test_zpl_simulator_create_quad_image_frame("test_master_intensity_flat_calib.fits.tmp", nx, ny, zero_odd, zero_even, pi_odd, pi_even);
    cpl_frame_set_tag( intensity_flat_frame, SPH_ZPL_TAG_IFF_CALIB );
    cpl_frame_set_group( intensity_flat_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, intensity_flat_frame );

    //simualate polarization flat
    polarization_flat_frame = sph_test_zpl_simulator_create_double_image_frame("test_master_polarization_flat_calib.fits.tmp", nx, ny, 200.0, 100.0 );
    cpl_frame_set_tag( polarization_flat_frame, SPH_ZPL_TAG_PFF_CALIB );
    cpl_frame_set_group( polarization_flat_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, polarization_flat_frame );

    //simualate modem efficiency
    modem_frame = sph_test_zpl_simulator_create_masterframe_frame("test_modem_calib.fits.tmp", nx, ny, 0.5 );
    cpl_frame_set_tag( modem_frame, SPH_ZPL_TAG_MODEM_EFF_CALIB );
    cpl_frame_set_group( modem_frame, CPL_FRAME_GROUP_CALIB );
    rerr = cpl_frameset_insert( inframes, modem_frame );


    zpl_basic_polarization = sph_zpl_basic_polarization_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_basic_polarization);

    rerr = sph_zpl_basic_polarization_run( zpl_basic_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );

    outframe = cpl_frameset_get_first( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 169.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 109.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    outframe = cpl_frameset_get_next( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 171.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 107.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    outframe = cpl_frameset_get_next( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 173.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 105.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    outframe = cpl_frameset_get_next( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 169.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), -371.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    outframe = cpl_frameset_get_next( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 171.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), -373.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    outframe = cpl_frameset_get_next( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 173.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), -375.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    rerr = sph_zpl_basic_polarization_delete( zpl_basic_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    return;
}


void cutest_zpl_basic_polarization_run_3(void){
    sph_zpl_basic_polarization*        zpl_basic_polarization     = NULL;
    cpl_parameterlist*          inparams            = NULL;
    cpl_frameset*               inframes            = NULL;
    cpl_frame*                  zexpcube_minus      = NULL;
    cpl_frame*                  zexpcube_plus       = NULL;
    cpl_frame*					outframe			= NULL;
    cpl_propertylist*			pl					= NULL;
    sph_error_code              rerr                = CPL_ERROR_NONE;
    int                         nz	                = 3;
    int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    int                         ny                  = 100; // -//-
    int                     	zero_odd    = 169;
    int                     	zero_even   = 29;
    int                     	pi_odd      = 29;
    int                     	pi_even     = 169;
    double 						ovsc[8]		= {5.0, 1.0, 6.0, 2.0, 7.0, 3.0, 8.0, 4.0};
   // double 					ovsc[8]		= {0.0, 1.0, 0.0, 2.0, 0.0, 3.0, 0.0, 4.0};

    int                     	zero_odd_1   = 172;
    int                     	zero_even_1  = 32;
    int                     	pi_odd_1     = 32;
    int                    		pi_even_1    = 172;
    double 						ovsc_1[8]	 = {15.0, 1.0, 16.0, 2.0, 17.0, 3.0, 18.0, 4.0};
    //double 					ovsc_1[8]		= {0.0, 1.0, 0.0, 2.0, 0.0, 3.0, 0.0, 4.0};
    double 						adu1_zero_odd_plus = 0;
    double						adu2_zero_odd_plus = 0;
    double						adu1_zero_even_plus = 0;
    double 						adu2_zero_even_plus = 0;
    double 						adu1_pi_odd_plus = 0;
    double						adu2_pi_odd_plus = 0;
    double						adu1_pi_even_plus = 0;
    double 						adu2_pi_even_plus = 0;

    double 						adu1_zero_odd_minus = 0;
    double						adu2_zero_odd_minus = 0;
    double						adu1_zero_even_minus = 0;
    double 						adu2_zero_even_minus = 0;
    double 						adu1_pi_odd_minus = 0;
    double						adu2_pi_odd_minus = 0;
    double						adu1_pi_even_minus = 0;
    double 						adu2_pi_even_minus = 0;
    double						iframe = 0.0;
    double 						pframe = 0.0;
    double						iframe_adu1 = 0.0;
    double 						pframe_adu1 = 0.0;
    double						iframe_adu2 = 0.0;
    double 						pframe_adu2 = 0.0;

    //double						pframe_over_iframe = 0.0;

    inparams = sph_zpl_basic_polarization_create_paramlist();
    inframes = cpl_frameset_new();

    // zpl frame cube with overscan table
    zexpcube_plus = sph_test_simulator_create_zpl_exposure_overscans_cube_set_hwpz( nx, ny, nz,
			  	  	  	  	  	"test_basic_polarization_zplexp_ovsc_cube",  zero_odd,  zero_even,  pi_odd, pi_even, ovsc,
			  	  	  	  	  	1, 1, 0.0);
    cpl_frame_set_tag( zexpcube_plus, SPH_ZPL_TAG_BASIC_POL_PREPROC_RAW);
    rerr = cpl_frameset_insert( inframes, zexpcube_plus );

    // zpl frame cube with overscan table
    zexpcube_minus = sph_test_simulator_create_zpl_exposure_overscans_cube_set_hwpz( nx, ny, nz,
			  	  	  	  	  	"test_basic_polarization_zplexp_ovsc_cube_1",  zero_odd_1,  zero_even_1,  pi_odd_1,
			  	  	  	  	  	pi_even_1, ovsc_1, 1, 0, 0.0 );
    cpl_frame_set_tag( zexpcube_minus, SPH_ZPL_TAG_BASIC_POL_PREPROC_RAW);
    rerr = cpl_frameset_insert( inframes, zexpcube_minus );


    zpl_basic_polarization = sph_zpl_basic_polarization_new( inframes, inparams );
    CU_ASSERT_PTR_NOT_NULL_FATAL( zpl_basic_polarization );

    rerr = sph_zpl_basic_polarization_run( zpl_basic_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );

    //calculate expected values for the first plane from the first cube
    adu1_zero_odd_plus = (169.-ovsc[0]);
    adu2_zero_odd_plus = (169.-ovsc[2]);
    adu1_zero_even_plus = (29.-ovsc[0]);
    adu2_zero_even_plus = (29.-ovsc[2]);
    adu1_pi_odd_plus = (29.-ovsc[4]);
    adu2_pi_odd_plus = (29.-ovsc[6]);
    adu1_pi_even_plus = (169.-ovsc[4]);
    adu2_pi_even_plus = (169.-ovsc[6]);

    iframe_adu1 = adu1_zero_odd_plus + adu1_zero_even_plus +
    		 	  adu1_pi_odd_plus + adu1_pi_even_plus;

    iframe_adu2 = adu2_zero_odd_plus + adu2_zero_even_plus +
    		 	  adu2_pi_odd_plus + adu2_pi_even_plus;

    pframe_adu1 = adu1_zero_odd_plus - adu1_zero_even_plus -
    		 	  adu1_pi_odd_plus + adu1_pi_even_plus;

    pframe_adu2 = adu2_zero_odd_plus - adu2_zero_even_plus -
    		 	  adu2_pi_odd_plus + adu2_pi_even_plus;

    iframe = 0.5*(iframe_adu1 + iframe_adu2);
    pframe = 0.5*(pframe_adu1 + pframe_adu2);

    outframe = cpl_frameset_get_first( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), iframe, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), pframe, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    //calculate expected values for the second plane from the first cube
    adu1_zero_odd_plus = (170.-(ovsc[0]+1));
    adu2_zero_odd_plus = (170.-(ovsc[2]+1));
    adu1_zero_even_plus = (30.-(ovsc[0]+1));
    adu2_zero_even_plus = (30.-(ovsc[2]+1));
    adu1_pi_odd_plus = (30.-(ovsc[4]+1));
    adu2_pi_odd_plus = (30.-(ovsc[6]+1));
    adu1_pi_even_plus = (170.-(ovsc[4]+1));
    adu2_pi_even_plus = (170.-(ovsc[6]+1));

    iframe_adu1 = adu1_zero_odd_plus + adu1_zero_even_plus +
    		 	  adu1_pi_odd_plus + adu1_pi_even_plus;

    iframe_adu2 = adu2_zero_odd_plus + adu2_zero_even_plus +
    		 	  adu2_pi_odd_plus + adu2_pi_even_plus;

    pframe_adu1 = adu1_zero_odd_plus - adu1_zero_even_plus -
    		 	  adu1_pi_odd_plus + adu1_pi_even_plus;

    pframe_adu2 = adu2_zero_odd_plus - adu2_zero_even_plus -
    		 	  adu2_pi_odd_plus + adu2_pi_even_plus;

    iframe = 0.5*(iframe_adu1 + iframe_adu2);
    pframe = 0.5*(pframe_adu1 + pframe_adu2);

    outframe = cpl_frameset_get_next( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), iframe, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), pframe, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    //calculate expected values for the third plane from the first cube
    adu1_zero_odd_plus = (171.-(ovsc[0]+2));
    adu2_zero_odd_plus = (171.-(ovsc[2]+2));

    adu1_zero_even_plus = (31.-(ovsc[0]+2));
    adu2_zero_even_plus = (31.-(ovsc[2]+2));

    adu1_pi_odd_plus = (31.-(ovsc[4]+2));
    adu2_pi_odd_plus = (31.-(ovsc[6]+2));

    adu1_pi_even_plus = (171.-(ovsc[4]+2));
    adu2_pi_even_plus = (171.-(ovsc[6]+2));

    iframe_adu1 = adu1_zero_odd_plus + adu1_zero_even_plus +
    		 	  adu1_pi_odd_plus + adu1_pi_even_plus;

    iframe_adu2 = adu2_zero_odd_plus + adu2_zero_even_plus +
    		 	  adu2_pi_odd_plus + adu2_pi_even_plus;

    pframe_adu1 = adu1_zero_odd_plus - adu1_zero_even_plus -
    		 	  adu1_pi_odd_plus + adu1_pi_even_plus;

    pframe_adu2 = adu2_zero_odd_plus - adu2_zero_even_plus -
    		 	  adu2_pi_odd_plus + adu2_pi_even_plus;

    iframe = 0.5*(iframe_adu1 + iframe_adu2);
    pframe = 0.5*(pframe_adu1 + pframe_adu2);

    outframe = cpl_frameset_get_next( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), iframe, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), pframe, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    //calculate expected values for the first plane from the second cube
    adu1_zero_odd_minus = (172.-ovsc_1[0]);
    adu2_zero_odd_minus = (172.-ovsc_1[2]);

    adu1_zero_even_minus = (32.-ovsc_1[0]);
    adu2_zero_even_minus = (32.-ovsc_1[2]);

    adu1_pi_odd_minus = (32.-ovsc_1[4]);
    adu2_pi_odd_minus = (32.-ovsc_1[6]);

    adu1_pi_even_minus = (172.-ovsc_1[4]);
    adu2_pi_even_minus = (172.-ovsc_1[6]);


    iframe_adu1 = adu1_zero_odd_minus + adu1_zero_even_minus +
    		 	  adu1_pi_odd_minus + adu1_pi_even_minus;

    iframe_adu2 = adu2_zero_odd_minus + adu2_zero_even_minus +
    		 	  adu2_pi_odd_minus + adu2_pi_even_minus;

    pframe_adu1 = adu1_zero_odd_minus - adu1_zero_even_minus -
    		 	  adu1_pi_odd_minus + adu1_pi_even_minus;

    pframe_adu2 = adu2_zero_odd_minus - adu2_zero_even_minus -
    		 	  adu2_pi_odd_minus + adu2_pi_even_minus;

    iframe = 0.5*(iframe_adu1 + iframe_adu2);
    pframe = 0.5*(pframe_adu1 + pframe_adu2);

    outframe = cpl_frameset_get_next( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), iframe, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), pframe, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    //calculate expected values for the second plane from the second cube
    adu1_zero_odd_minus = (173.-ovsc_1[0]);
    adu2_zero_odd_minus = (173.-ovsc_1[2]);

    adu1_zero_even_minus = (33.-ovsc_1[0]);
    adu2_zero_even_minus = (33.-ovsc_1[2]);

    adu1_pi_odd_minus = (33.-ovsc_1[4]);
    adu2_pi_odd_minus = (33.-ovsc_1[6]);

    adu1_pi_even_minus = (173.-ovsc_1[4]);
    adu2_pi_even_minus = (173.-ovsc_1[6]);

    iframe_adu1 = adu1_zero_odd_minus + adu1_zero_even_minus +
    		 	  adu1_pi_odd_minus + adu1_pi_even_minus;

    iframe_adu2 = adu2_zero_odd_minus + adu2_zero_even_minus +
    		 	  adu2_pi_odd_minus + adu2_pi_even_minus;

    pframe_adu1 = adu1_zero_odd_minus - adu1_zero_even_minus -
    		 	  adu1_pi_odd_minus + adu1_pi_even_minus;

    pframe_adu2 = adu2_zero_odd_minus - adu2_zero_even_minus -
    		 	  adu2_pi_odd_minus + adu2_pi_even_minus;

    iframe = 0.5*(iframe_adu1 + iframe_adu2);
    pframe = 0.5*(pframe_adu1 + pframe_adu2);

    outframe = cpl_frameset_get_next( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), iframe, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), pframe, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    //calculate expected values for the third plane from the second cube
    adu1_zero_odd_minus = (174.-ovsc_1[0]);
    adu2_zero_odd_minus = (174.-ovsc_1[2]);

    adu1_zero_even_minus = (34.-ovsc_1[0]);
    adu2_zero_even_minus = (34.-ovsc_1[2]);

    adu1_pi_odd_minus = (34.-ovsc_1[4]);
    adu2_pi_odd_minus = (34.-ovsc_1[6]);

    adu1_pi_even_minus = (174.-ovsc_1[4]);
    adu2_pi_even_minus = (174.-ovsc_1[6]);

    iframe_adu1 = adu1_zero_odd_minus + adu1_zero_even_minus +
    		 	  adu1_pi_odd_minus + adu1_pi_even_minus;

    iframe_adu2 = adu2_zero_odd_minus + adu2_zero_even_minus +
    		 	  adu2_pi_odd_minus + adu2_pi_even_minus;

    pframe_adu1 = adu1_zero_odd_minus - adu1_zero_even_minus -
    		 	  adu1_pi_odd_minus + adu1_pi_even_minus;

    pframe_adu2 = adu2_zero_odd_minus - adu2_zero_even_minus -
    		 	  adu2_pi_odd_minus + adu2_pi_even_minus;

    iframe = 0.5*(iframe_adu1 + iframe_adu2);
    pframe = 0.5*(pframe_adu1 + pframe_adu2);

    outframe = cpl_frameset_get_next( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), iframe, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), pframe, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;



    //Destroy recipe
    rerr = sph_zpl_basic_polarization_delete( zpl_basic_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    //Destroy here input parameters and inframes explicitly
    cpl_frameset_delete( inframes );
    cpl_parameterlist_delete(inparams);

    return;

}

/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_basic_polarization only with rawframes (cube with a single plane)
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_basic_polarization_run_4(void) {
    sph_zpl_basic_polarization*     zpl_basic_polarization  = NULL;
    cpl_parameterlist*          	inparams            = NULL;
    cpl_frameset*               	inframes            = NULL;
    cpl_frame*                  	zexpcube            = NULL;
    cpl_frame*						zexpcube1			= NULL;
    cpl_frame*						outframe			= NULL;
    sph_error_code              	rerr                = CPL_ERROR_NONE;
    int                         	nframes             = 1;
    //int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;

    inparams = sph_zpl_basic_polarization_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube,"test_basic_pol_single.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_BASIC_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube1, "test_basic_pol_1_single.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_BASIC_POL_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    zpl_basic_polarization = sph_zpl_basic_polarization_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_basic_polarization);

    rerr = sph_zpl_basic_polarization_run( zpl_basic_polarization );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    outframe = cpl_frameset_get_first( zpl_basic_polarization->basic_pol_frameset);
    pl = cpl_propertylist_load( cpl_frame_get_filename( outframe ), 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 396.0, 1e-20);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 280.0, 1e-20);
    cpl_propertylist_delete( pl ); pl = NULL;

    //cpl_frame_delete( outframe );
    rerr = sph_zpl_basic_polarization_delete( zpl_basic_polarization );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    return;
}





/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_basic_polarization recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    int result       = 0;

    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("basic_polarization_test", cutest_init_basic_polarization_testsuite,
                          cutest_clean_basic_polarization_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */


    if ( NULL == CU_add_test(pSuite, "sph_basic_polarization_run_1: 2 cube-rawframes ",
                             cutest_zpl_basic_polarization_run_1 ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    if ( NULL == CU_add_test(pSuite, "sph_basic_polarization_run_2: 2 cube-rawframes  with bias-, dark-, iff-, pff-, modem- calibrations",
                              cutest_zpl_basic_polarization_run_2 ) )
    {
        CU_cleanup_registry();
        return CU_get_error();
     }

    if ( NULL == CU_add_test(pSuite, "sph_basic_polarization_run_3: 2 cube-rawframes with overscan tables ",
                             cutest_zpl_basic_polarization_run_3 ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    if ( NULL == CU_add_test(pSuite, "sph_basic_polarization_run_4: 2 cube-rawframes with a single plane ",
                             cutest_zpl_basic_polarization_run_4 ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
