/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <Basic.h>
#include <CUnit.h>
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_ngc_ir_simulator.h"
#include "sph_test_pupilimage_creator.h"
#include "sph_create_flat.h"
#include "sph_test_image_tools.h"
#include "sph_master_frame.h"
#include "sph_utils.h"
#include "sph_common_keywords.h"
#include "sph_smart_imagelist.h"
/*----------------------------------------------------------------------------*/
/**
 * @defgroup A CUnit Test Suite -- representing a collection of testcases
 * @par Synopsis:
 * @code
 * @endcode
 * @par Desciption:
 *
 * This module provides a collection of tests for one specific, distinct
 * module or set-up. The testing code is implemented using the CUnit
 * framework.
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*----------------------------------------------------------------------------*/
/*-                            INTERNAL HELPER FUNCTIONS                          */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to initiailise the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_init_testsuite() {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__,"moeller@mpia-hd.mpg.de",CPL_MSG_DEBUG);
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to clean the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_clean_testsuite() {
    sph_error_dump( SPH_ERROR_ERROR );
    sph_end_test();
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_test_create_flat()
{
    cpl_parameterlist*        comb_params = NULL;
    sph_error_code             rerr        = CPL_ERROR_NONE;
    cpl_frameset*            rawframes    = NULL;
    sph_master_frame*        result        = NULL;
    cpl_image*                image        = NULL;
    int                        ii            = 0;
    gsl_rng*                pRNG        = NULL;
    int                        xx            = 0;
    int                        yy            = 0;
    int                        bpix        = 0;

    pRNG = gsl_rng_alloc(gsl_rng_taus);
    rawframes = cpl_frameset_new();
    comb_params = cpl_parameterlist_new();
    sph_filemanager_new();
    for (ii = 0; ii < 3; ++ii) {
        image = sph_test_image_tools_create_flat_image_double(256, 256, ( ii + 1 ) * 1000.0 );
        sph_test_image_tools_add_noise(image,2.0,pRNG);
        cpl_frameset_insert( rawframes,
                sph_test_frame_image(image,
                        CPL_BPP_32_SIGNED,"NONE",CPL_FRAME_GROUP_RAW));
        cpl_image_delete(image); image = NULL;
    }
    result = sph_create_flat( rawframes, 0, NULL, NULL, NULL, -100.0, 100.0, 1000.0,NULL, 0, NULL);
    CU_ASSERT_PTR_NOT_NULL_FATAL(result);
    for (ii = 0; ii < 100; ++ii) {
        xx = gsl_ran_flat(pRNG,1.0,cpl_image_get_size_x(result->image));
        yy = gsl_ran_flat(pRNG,1.0,cpl_image_get_size_y(result->image));
        CU_ASSERT_DOUBLE_EQUAL(cpl_image_get(result->image,xx,yy,&bpix),1.0,0.01);
        CU_ASSERT_EQUAL(bpix,0);
    }
    CU_ASSERT_DOUBLE_EQUAL(sph_master_frame_get_mean(result,NULL),1.0,0.0001);
    sph_master_frame_delete(result); result = NULL;
    gsl_rng_free(pRNG);pRNG = NULL;
    cpl_parameterlist_delete(comb_params); comb_params = NULL;
    cpl_frameset_delete(rawframes); rawframes = NULL;
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_test_create_flat_with_dark()
{
    cpl_parameterlist*        comb_params = NULL;
    sph_error_code             rerr        = CPL_ERROR_NONE;
    cpl_frameset*            rawframes    = NULL;
    sph_master_frame*        result        = NULL;
    cpl_image*                image        = NULL;
    cpl_image*                dark        = NULL;
    sph_master_frame*        dark_mframe    = NULL;
    int                        ii            = 0;
    gsl_rng*                pRNG        = NULL;
    int                        xx            = 0;
    int                        yy            = 0;
    int                        bpix        = 0;

    pRNG = gsl_rng_alloc(gsl_rng_taus);
    rawframes = cpl_frameset_new();
    comb_params = cpl_parameterlist_new();
    dark = sph_test_image_tools_create_striped_image(256, 256, 6, CPL_TYPE_DOUBLE, 1000.0, 10000.0 );
    for (ii = 0; ii < 3; ++ii) {
        image = sph_test_image_tools_create_flat_image_double(256, 256, ( ii + 1 ) * 1000.0 );
        cpl_image_add(image,dark);
        sph_test_image_tools_add_noise(image,2.0,pRNG);
        cpl_frameset_insert( rawframes,
                sph_test_frame_image(image,
                        CPL_BPP_32_SIGNED,"NONE",CPL_FRAME_GROUP_RAW) );
        cpl_image_delete(image); image = NULL;
    }
    dark_mframe = sph_master_frame_new_from_cpl_image(dark);
    cpl_image_set(dark_mframe->badpixelmap,10,10,1.0);
    result = sph_create_flat( rawframes, 0, NULL, dark_mframe, NULL, -100.0, 100.0, 1000.0, NULL, 0, NULL);
    CU_ASSERT_PTR_NOT_NULL_FATAL(result);
    for (ii = 0; ii < 100; ++ii) {
        xx = gsl_ran_flat(pRNG,1.0,cpl_image_get_size_x(result->image));
        yy = gsl_ran_flat(pRNG,1.0,cpl_image_get_size_y(result->image));
        CU_ASSERT_DOUBLE_EQUAL(cpl_image_get(result->image,xx,yy,&bpix),1.0,0.01);
        CU_ASSERT_EQUAL(bpix,0);
    }
    CU_ASSERT_DOUBLE_EQUAL(sph_master_frame_get_mean(result,NULL),1.0,0.0001);
    CU_ASSERT_NOT_EQUAL(cpl_image_get(result->badpixelmap,10,10,&bpix),0);
    CU_ASSERT_EQUAL(cpl_image_get(result->badpixelmap,11,11,&bpix),0);
    sph_master_frame_delete(result); result = NULL;
    sph_master_frame_delete(dark_mframe); dark_mframe = NULL;
    gsl_rng_free(pRNG);pRNG = NULL;
    cpl_parameterlist_delete(comb_params); comb_params = NULL;
    cpl_frameset_delete(rawframes); rawframes = NULL;
    cpl_image_delete(dark);dark = NULL;
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_test_create_flat_with_dark_robust()
{
    cpl_parameterlist*        comb_params = NULL;
    sph_error_code             rerr        = CPL_ERROR_NONE;
    cpl_frameset*            rawframes    = NULL;
    sph_master_frame*        result        = NULL;
    cpl_image*                image        = NULL;
    cpl_image*                dark        = NULL;
    sph_master_frame*        dark_mframe    = NULL;
    int                        ii            = 0;
    gsl_rng*                pRNG        = NULL;
    int                        xx            = 0;
    int                        yy            = 0;
    int                        bpix        = 0;

    pRNG = gsl_rng_alloc(gsl_rng_taus);
    rawframes = cpl_frameset_new();
    comb_params = cpl_parameterlist_new();
    dark = sph_test_image_tools_create_striped_image(256, 256, 6, CPL_TYPE_DOUBLE, 1000.0, 10000.0 );
    for (ii = 0; ii < 5; ++ii) {
        image = sph_test_image_tools_create_flat_image_double(256, 256, ( ii + 1 ) * 1000.0 );
        cpl_image_add(image,dark);
        sph_test_image_tools_add_noise(image,2.0,pRNG);
        cpl_frameset_insert( rawframes,
                sph_test_frame_image(image,
                        CPL_BPP_32_SIGNED,"NONE",CPL_FRAME_GROUP_RAW) );
        cpl_image_delete(image); image = NULL;
    }
    dark_mframe = sph_master_frame_new_from_cpl_image(dark);
    cpl_image_set(dark_mframe->badpixelmap,10,10,1.0);
    result = sph_create_flat( rawframes, 1, NULL, dark_mframe, NULL, -100.0, 100.0, 1000.0, NULL, 0, NULL);
    CU_ASSERT_PTR_NOT_NULL_FATAL(result);
    sph_master_frame_save(result,"flat.fits",NULL);
    for (ii = 0; ii < 100; ++ii) {
        xx = gsl_ran_flat(pRNG,1.0,cpl_image_get_size_x(result->image));
        yy = gsl_ran_flat(pRNG,1.0,cpl_image_get_size_y(result->image));
        CU_ASSERT_DOUBLE_EQUAL(cpl_image_get(result->image,xx,yy,&bpix),1.0,0.01);
        CU_ASSERT_EQUAL(bpix,0);
    }
    CU_ASSERT_DOUBLE_EQUAL(sph_master_frame_get_mean(result,NULL),1.0,0.0001);
    CU_ASSERT_NOT_EQUAL(cpl_image_get(result->badpixelmap,10,10,&bpix),0);
    CU_ASSERT_EQUAL(cpl_image_get(result->badpixelmap,11,11,&bpix),0);
    sph_master_frame_delete(result); result = NULL;
    sph_master_frame_delete(dark_mframe); dark_mframe = NULL;
    gsl_rng_free(pRNG);pRNG = NULL;
    cpl_parameterlist_delete(comb_params); comb_params = NULL;
    cpl_frameset_delete(rawframes); rawframes = NULL;
    cpl_image_delete(dark);dark = NULL;
    cpl_error_reset();
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_test_create_flat_with_dark_lampflux_check()
{
    cpl_parameterlist*        comb_params = NULL;
    sph_error_code             rerr        = CPL_ERROR_NONE;
    cpl_frameset*            rawframes    = NULL;
    sph_master_frame*        result        = NULL;
    cpl_image*                image        = NULL;
    cpl_image*                dark        = NULL;
    sph_master_frame*        dark_mframe    = NULL;
    int                        ii            = 0;
    int                        xx            = 0;
    int                        yy            = 0;
    int                        bpix        = 0;
    cpl_vector*                 lampflux       = NULL;
    cpl_vector*                 lampfluxstdev       = NULL;
    gsl_rng*                pRNG        = NULL;

    pRNG = gsl_rng_alloc(gsl_rng_taus);
    rawframes = cpl_frameset_new();
    comb_params = cpl_parameterlist_new();
    dark = sph_test_image_tools_create_striped_image(256, 256, 6, CPL_TYPE_DOUBLE, 1000.0, 10000.0 );
    for (ii = 0; ii < 3; ++ii) {
        image = sph_test_image_tools_create_flat_image_double(256, 256, ( ii + 1 ) * 1000.0 );
        cpl_image_add(image,dark);
        sph_test_image_tools_add_noise(image,2.0,pRNG);
        cpl_frameset_insert( rawframes,
                sph_test_frame_image(image,
                        CPL_BPP_32_SIGNED,"NONE",CPL_FRAME_GROUP_RAW) );
        cpl_image_delete(image); image = NULL;
    }
    dark_mframe = sph_master_frame_new_from_cpl_image(dark);
    cpl_image_set(dark_mframe->badpixelmap,10,10,1.0);
    result = sph_create_flat( rawframes, 0, NULL, dark_mframe, NULL, -100.0, 100.0, 1000.0, &lampflux, 0, &lampfluxstdev);
    CU_ASSERT_PTR_NOT_NULL_FATAL(lampflux);
    CU_ASSERT_PTR_NOT_NULL_FATAL(lampfluxstdev);
    CU_ASSERT_PTR_NOT_NULL_FATAL(result);
    CU_ASSERT_EQUAL( cpl_vector_get_size(lampflux), 3 );
    for (ii = 0; ii < cpl_vector_get_size(lampflux); ++ii) {
        CU_ASSERT_DOUBLE_EQUAL( cpl_vector_get(lampflux,ii), (ii + 1) * 1000.0,1.1);
        CU_ASSERT_DOUBLE_EQUAL( cpl_vector_get(lampfluxstdev,ii), 2.0 ,1.1);
    }
    sph_master_frame_delete(result); result = NULL;
    sph_master_frame_delete(dark_mframe); dark_mframe = NULL;
    cpl_parameterlist_delete(comb_params); comb_params = NULL;
    cpl_frameset_delete(rawframes); rawframes = NULL;
    cpl_image_delete(dark);dark = NULL;
    gsl_rng_free(pRNG);pRNG = NULL;
    cpl_vector_delete(lampflux); lampflux = NULL;
    cpl_vector_delete(lampfluxstdev); lampfluxstdev = NULL;
    return;
}


static
void cutest_test_create_flat_pure_frames_5()
{
    cpl_parameterlist*        comb_params = NULL;
    sph_error_code            rerr        = CPL_ERROR_NONE;
    cpl_frameset*             rawframes   = NULL;
    sph_master_frame*         result      = NULL;
    cpl_image*                image       = NULL;
    int                       ii          = 0;
    double					  value		  = 2500.0;
    int						  nframes	  = 5;

    rawframes = cpl_frameset_new();
    comb_params = cpl_parameterlist_new();
    sph_filemanager_new();
    for (ii = 0; ii < nframes; ++ii) {
        image = sph_test_image_tools_create_flat_image_double(256, 256, (2500.0+ii*10.0) );
        cpl_frameset_insert( rawframes,
                sph_test_frame_image(image,
                        CPL_BPP_32_SIGNED,"NONE",CPL_FRAME_GROUP_RAW));
        cpl_image_delete(image); image = NULL;
    }
    result = sph_create_flat( rawframes,  0, NULL, NULL, NULL, -100.0, 100.0, 1000.0,NULL, 0,NULL);
    CU_ASSERT_PTR_NOT_NULL_FATAL(result);
    CU_ASSERT_DOUBLE_EQUAL(sph_master_frame_get_mean(result,NULL),1.0,0.0001);
    CU_ASSERT_DOUBLE_EQUAL( cpl_image_get_mean(result->rmsmap), 0.0, 0.000001); //small rms value
    CU_ASSERT_DOUBLE_EQUAL( cpl_image_get_mean(result->badpixelmap), 0.0, 0.000001); //no badpixel, mean badpixel must be 1.0

    sph_master_frame_save(result, "test_creat_flat_5.fits.tmp", NULL);

    sph_master_frame_delete(result); result = NULL;
    cpl_parameterlist_delete(comb_params); comb_params = NULL;
    cpl_frameset_delete(rawframes); rawframes = NULL;
    return;
}

static
void cutest_test_create_flat_pure_frames_3_error()
{
    cpl_parameterlist*        comb_params = NULL;
    sph_error_code            rerr        = CPL_ERROR_NONE;
    cpl_frameset*             rawframes   = NULL;
    sph_master_frame*         result      = NULL;
    cpl_image*                image       = NULL;
    int                       ii          = 0;
    double					  value		  = 2500.0;
    int						  nframes	  = 3;

    rawframes = cpl_frameset_new();
    comb_params = cpl_parameterlist_new();
    sph_filemanager_new();
    for (ii = 0; ii < nframes; ++ii) {
        image = sph_test_image_tools_create_flat_image_double(256, 256, (2500.0+ii*10.0) );
        cpl_frameset_insert( rawframes,
                sph_test_frame_image(image,
                        CPL_BPP_32_SIGNED,"NONE",CPL_FRAME_GROUP_RAW));
        cpl_image_delete(image); image = NULL;
    }
    result = sph_create_flat( rawframes, 0, NULL, NULL, NULL, -100.0, 100.0, 1000.0,NULL, 0, NULL);
    CU_ASSERT_PTR_NOT_NULL_FATAL(result);
    //ASSERT with wrong huge rms = 1.0E10
    //printf("rmsmap: %f10 \n", cpl_image_get_mean(result->rmsmap));
    //CU_ASSERT_DOUBLE_EQUAL( cpl_image_get_mean(result->rmsmap), 1E10, 0.0001); //huge rms!!!
    CU_ASSERT_DOUBLE_EQUAL( cpl_image_get_mean(result->rmsmap), 0.0, 0.0001); //rms is not 0.0 within tolerance
    CU_ASSERT_DOUBLE_EQUAL( cpl_image_get_mean(result->badpixelmap), 0.0, 0.000001); //mean badpixel must be 1.0

    sph_master_frame_save(result, "test_creat_flat_3.fits.tmp", NULL);

    sph_master_frame_delete(result); result = NULL;
    cpl_parameterlist_delete(comb_params); comb_params = NULL;
    cpl_frameset_delete(rawframes); rawframes = NULL;
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test MAIN function
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    int         result      = 0;
    CU_pSuite   pSuite      = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("Creation of flat", cutest_init_testsuite,
                          cutest_clean_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "simple test case",
                             cutest_test_create_flat ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "test case with dark",
                             cutest_test_create_flat_with_dark ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "test case with dark robust",
                             cutest_test_create_flat_with_dark_robust ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "test case with dark, lamp flux output",
                             cutest_test_create_flat_with_dark_lampflux_check ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "simple test case with pure flat 5 images: produces correct result",
                             cutest_test_create_flat_pure_frames_5 ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "simple test case with pure flat 3 images: produces error (inverse verification to make it run) ",
                             cutest_test_create_flat_pure_frames_3_error ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
