/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <Basic.h>
#include <CUnit.h>
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_ngc_ir_simulator.h"
#include "sph_test_pupilimage_creator.h"
#include "sph_create_super_flat.h"
#include "sph_master_frame.h"
#include "sph_common_keywords.h"
#include "sph_test_image_tools.h"
/*----------------------------------------------------------------------------*/
/**
 * @defgroup A CUnit Test Suite -- representing a collection of testcases
 * @par Synopsis:
 * @code
 * @endcode
 * @par Desciption:
 *
 * This module provides a collection of tests for one specific, distinct
 * module or set-up. The testing code is implemented using the CUnit
 * framework.
 */
/*----------------------------------------------------------------------------*/
/**@{*/


/*----------------------------------------------------------------------------*/
/**
 @brief    Function to initiailise the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_init_testsuite() {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__,"moeller@mpia-hd.mpg.de",CPL_MSG_DEBUG);
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to clean the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_clean_testsuite() {
    sph_error_dump( SPH_ERROR_ERROR );
    return sph_end_test();
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_test_create_flat() {
    sph_master_frame*            ldff1 = NULL;
    sph_master_frame*            ldff2 = NULL;
    sph_master_frame*            ldff3 = NULL;
    sph_master_frame*            ldff4 = NULL;
    sph_master_frame*            ldffbb = NULL;
    sph_master_frame*            sdffbb = NULL;
    sph_master_frame*            result = NULL;
    sph_master_frame*            ideal  = NULL;
    sph_pixel_description_table*    pdt    = NULL;
    sph_ifs_lenslet_model*        model = NULL;
    cpl_image*                    wim = NULL;
    cpl_propertylist*             pl = NULL;

    model = sph_test_create_small_lenslet_model();
    CU_ASSERT_PTR_NOT_NULL_FATAL( model );
    pdt = sph_pixel_description_table_new_from_model( model,0.0,0.0 );
    CU_ASSERT_PTR_NOT_NULL_FATAL( pdt );
    ldff1 = sph_master_frame_new(model->detsize_pixels,model->detsize_pixels);
    cpl_image_delete( ldff1->image ); ldff1->image = NULL;
    ldff2 = sph_master_frame_new(model->detsize_pixels,model->detsize_pixels);
    cpl_image_delete( ldff2->image ); ldff2->image = NULL;
    ldff3 = sph_master_frame_new(model->detsize_pixels,model->detsize_pixels);
    cpl_image_delete( ldff3->image ); ldff3->image = NULL;
    ldff4 = sph_master_frame_new(model->detsize_pixels,model->detsize_pixels);
    cpl_image_delete( ldff4->image ); ldff4->image = NULL;
    ldffbb = sph_master_frame_new(model->detsize_pixels,model->detsize_pixels);
    cpl_image_delete( ldffbb->image ); ldffbb->image = NULL;
    sdffbb = sph_master_frame_new(model->detsize_pixels,model->detsize_pixels);
    cpl_image_delete( sdffbb->image ); sdffbb->image = NULL;
    ldff1->image = sph_test_image_tools_create_flat_image_double(
            model->detsize_pixels,model->detsize_pixels,model->minlambda );
    ldff2->image = sph_test_image_tools_create_flat_image_double(
            model->detsize_pixels,model->detsize_pixels,
            model->minlambda + (model->maxlambda - model->minlambda) * 0.3333 );
    ldff3->image = sph_test_image_tools_create_flat_image_double(
            model->detsize_pixels,model->detsize_pixels,
            model->minlambda + (model->maxlambda - model->minlambda) * 0.6666 );
    ldff4->image = sph_test_image_tools_create_flat_image_double(
            model->detsize_pixels,model->detsize_pixels,model->maxlambda );
    ldffbb->image = sph_test_image_tools_create_flat_image_double(
            model->detsize_pixels,model->detsize_pixels,1.0 );
    sdffbb->image = sph_test_image_tools_create_flat_image_double(
            model->detsize_pixels,model->detsize_pixels,1.0 );

    cpl_propertylist_update_double( ldff1->properties,
            SPH_COMMON_KEYWORD_CALIB_LAMBDA, model->minlambda );
    cpl_propertylist_update_double( ldff2->properties,
            SPH_COMMON_KEYWORD_CALIB_LAMBDA,
            model->minlambda + (model->maxlambda - model->minlambda) * 0.3333 );
    cpl_propertylist_update_double( ldff3->properties,
            SPH_COMMON_KEYWORD_CALIB_LAMBDA,
            model->minlambda + (model->maxlambda - model->minlambda) * 0.6666 );
    cpl_propertylist_update_double( ldff4->properties,
            SPH_COMMON_KEYWORD_CALIB_LAMBDA, model->maxlambda );
    result = sph_create_super_flat( pdt, ldff1, ldff2, ldff3, ldff4);
    sph_master_frame_mask_tolerance(result, 0.5, 100000 );
    sph_master_frame_quality_check( result );
    sph_master_frame_save( result,
            "super_flat.fits",
            pl = sph_ifs_lenslet_model_get_as_propertylist(model));
    cpl_propertylist_delete(pl); pl = NULL;

    ideal = sph_master_frame_new_from_cpl_image(
            wim = sph_pixel_description_table_get_wavimage(pdt) );
    sph_master_frame_subtract_master_frame( result, ideal  );
    sph_master_frame_save( result,
            "super_flat_difference.fits",
            pl = sph_ifs_lenslet_model_get_as_propertylist(model));
    cpl_propertylist_delete(pl); pl = NULL;

    CU_ASSERT_DOUBLE_EQUAL( sph_master_frame_get_absflux(result,NULL),0.0,0.1);
    CU_ASSERT_PTR_NOT_NULL_FATAL( result );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );
    sph_master_frame_delete( ldff1 ); ldff1 = NULL;
    sph_master_frame_delete( ldff2 ); ldff2 = NULL;
    sph_master_frame_delete( ldff3 ); ldff3 = NULL;
    sph_master_frame_delete( ldff4 ); ldff4 = NULL;
    sph_master_frame_delete( ldffbb ); ldffbb = NULL;
    sph_master_frame_delete( sdffbb ); sdffbb = NULL;
    sph_master_frame_delete( result ); result = NULL;
    sph_master_frame_delete( ideal ); ideal = NULL;
    sph_pixel_description_table_delete(pdt); pdt = NULL;
    sph_ifs_lenslet_model_delete(model); model = NULL;
    cpl_image_delete(wim); wim = NULL;
    cpl_error_reset();
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_test_create_flat_only_three() {
    sph_master_frame*            ldff1 = NULL;
    sph_master_frame*            ldff2 = NULL;
    cpl_image*                    wim = NULL;
    sph_master_frame*            ldff3 = NULL;
    sph_master_frame*            ldffbb = NULL;
    sph_master_frame*            sdffbb = NULL;
    sph_master_frame*            result = NULL;
    sph_master_frame*            ideal  = NULL;
    sph_pixel_description_table*    pdt    = NULL;
    sph_ifs_lenslet_model*        model = NULL;
    cpl_propertylist*             pl = NULL;

    model = sph_ifs_lenslet_model_new();//sph_test_create_small_lenslet_model();
    CU_ASSERT_PTR_NOT_NULL_FATAL( model );
    pdt = sph_pixel_description_table_new_from_model( model,0.0,0.0 );
    CU_ASSERT_PTR_NOT_NULL_FATAL( pdt );
    sph_pixel_description_table_save( pdt, "pdt.fits", model );
    ldff1 = sph_master_frame_new(model->detsize_pixels,model->detsize_pixels);
    cpl_image_delete( ldff1->image ); ldff1->image = NULL;
    ldff2 = sph_master_frame_new(model->detsize_pixels,model->detsize_pixels);
    cpl_image_delete( ldff2->image ); ldff2->image = NULL;
    ldff3 = sph_master_frame_new(model->detsize_pixels,model->detsize_pixels);
    cpl_image_delete( ldff3->image ); ldff3->image = NULL;
    ldffbb = sph_master_frame_new(model->detsize_pixels,model->detsize_pixels);
    cpl_image_delete( ldffbb->image ); ldffbb->image = NULL;
    sdffbb = sph_master_frame_new(model->detsize_pixels,model->detsize_pixels);
    cpl_image_delete( sdffbb->image ); sdffbb->image = NULL;
    ldff1->image = sph_test_image_tools_create_flat_image_double(
            model->detsize_pixels,model->detsize_pixels,model->minlambda );
    ldff2->image = sph_test_image_tools_create_flat_image_double(
            model->detsize_pixels,model->detsize_pixels,
            model->minlambda + (model->maxlambda - model->minlambda) * 0.3333 );
    ldff3->image = sph_test_image_tools_create_flat_image_double(
            model->detsize_pixels,
            model->detsize_pixels,
            model->minlambda + (model->maxlambda - model->minlambda) * 0.6666 );
    ldffbb->image = sph_test_image_tools_create_flat_image_double(
            model->detsize_pixels,model->detsize_pixels,1.0 );
    sdffbb->image = sph_test_image_tools_create_flat_image_double(
            model->detsize_pixels,model->detsize_pixels,1.0 );

    cpl_propertylist_update_double(
            ldff1->properties,
            SPH_COMMON_KEYWORD_CALIB_LAMBDA, model->minlambda );
    cpl_propertylist_update_double(
            ldff2->properties,
            SPH_COMMON_KEYWORD_CALIB_LAMBDA,
            model->minlambda + (model->maxlambda - model->minlambda) * 0.3333 );
    cpl_propertylist_update_double(
            ldff3->properties,
            SPH_COMMON_KEYWORD_CALIB_LAMBDA,
            model->minlambda + (model->maxlambda - model->minlambda) * 0.6666 );
    result = sph_create_super_flat(
            pdt, ldff1, ldff2, ldff3, NULL );
    sph_master_frame_mask_tolerance(result, 0.5, 100000 );
    sph_master_frame_quality_check( result );
    sph_master_frame_save( result, "super_flat.fits",
            pl = sph_ifs_lenslet_model_get_as_propertylist(model));
    cpl_propertylist_delete(pl); pl = NULL;

    ideal = sph_master_frame_new_from_cpl_image(
            wim = sph_pixel_description_table_get_wavimage(pdt) );
    sph_master_frame_subtract_master_frame( result, ideal  );
    sph_master_frame_save( result,
            "super_flat_difference.fits",
            pl = sph_ifs_lenslet_model_get_as_propertylist(model));
    cpl_propertylist_delete(pl); pl = NULL;

    CU_ASSERT_DOUBLE_EQUAL( sph_master_frame_get_absflux(result,NULL),0.0,0.1);
    CU_ASSERT_PTR_NOT_NULL_FATAL( result );
    sph_master_frame_delete( ldff1 ); ldff1 = NULL;
    sph_master_frame_delete( ldff2 ); ldff2 = NULL;
    sph_master_frame_delete( ldff3 ); ldff3 = NULL;
    sph_master_frame_delete( ldffbb ); ldffbb = NULL;
    sph_master_frame_delete( sdffbb ); sdffbb = NULL;
    sph_master_frame_delete( result ); result = NULL;
    sph_master_frame_delete( ideal ); ideal = NULL;
    sph_pixel_description_table_delete(pdt); pdt = NULL;
    sph_ifs_lenslet_model_delete(model); model = NULL;
    cpl_image_delete(wim); wim = NULL;
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_test_create_flat_no_sdffb() {
    sph_master_frame*            ldff1 = NULL;
    sph_master_frame*            ldff2 = NULL;
    sph_master_frame*            ldff3 = NULL;
    sph_master_frame*            ldffbb = NULL;
    sph_master_frame*            result = NULL;
    sph_master_frame*            ideal  = NULL;
    cpl_image*                    wim = NULL;
    sph_pixel_description_table*    pdt    = NULL;
    sph_ifs_lenslet_model*        model = NULL;
    cpl_propertylist*             pl = NULL;

    model = sph_ifs_lenslet_model_new();//sph_test_create_small_lenslet_model();
    CU_ASSERT_PTR_NOT_NULL_FATAL( model );
    pdt = sph_pixel_description_table_new_from_model( model,0.0,0.0 );
    CU_ASSERT_PTR_NOT_NULL_FATAL( pdt );
    sph_pixel_description_table_save( pdt, "pdt.fits", model );
    ldff1 = sph_master_frame_new(model->detsize_pixels,model->detsize_pixels);
    cpl_image_delete( ldff1->image ); ldff1->image = NULL;
    ldff2 = sph_master_frame_new(model->detsize_pixels,model->detsize_pixels);
    cpl_image_delete( ldff2->image ); ldff2->image = NULL;
    ldff3 = sph_master_frame_new(model->detsize_pixels,model->detsize_pixels);
    cpl_image_delete( ldff3->image ); ldff3->image = NULL;
    ldffbb = sph_master_frame_new(model->detsize_pixels,model->detsize_pixels);
    cpl_image_delete( ldffbb->image ); ldffbb->image = NULL;
    ldff1->image = sph_test_image_tools_create_flat_image_double(
            model->detsize_pixels,model->detsize_pixels,model->minlambda );
    ldff2->image = sph_test_image_tools_create_flat_image_double(
            model->detsize_pixels,model->detsize_pixels,model->minlambda +
            (model->maxlambda - model->minlambda) * 0.3333 );
    ldff3->image = sph_test_image_tools_create_flat_image_double(
            model->detsize_pixels,
            model->detsize_pixels,model->minlambda +
            (model->maxlambda - model->minlambda) * 0.6666 );
    ldffbb->image = sph_test_image_tools_create_flat_image_double(
            model->detsize_pixels,model->detsize_pixels,1.0 );

    cpl_propertylist_update_double( ldff1->properties,
            SPH_COMMON_KEYWORD_CALIB_LAMBDA, model->minlambda );
    cpl_propertylist_update_double( ldff2->properties,
            SPH_COMMON_KEYWORD_CALIB_LAMBDA,
            model->minlambda + (model->maxlambda - model->minlambda) * 0.3333 );
    cpl_propertylist_update_double( ldff3->properties,
            SPH_COMMON_KEYWORD_CALIB_LAMBDA,
            model->minlambda + (model->maxlambda - model->minlambda) * 0.6666 );
    result = sph_create_super_flat( pdt,
            ldff1, ldff2, ldff3, NULL );
    sph_master_frame_mask_tolerance(result, 0.5, 100000 );
    sph_master_frame_quality_check( result );
    sph_master_frame_save( result, "super_flat.fits",
            pl = sph_ifs_lenslet_model_get_as_propertylist(model));
    cpl_propertylist_delete(pl); pl = NULL;
    ideal = sph_master_frame_new_from_cpl_image(
            wim = sph_pixel_description_table_get_wavimage(pdt) );
    sph_master_frame_subtract_master_frame( result, ideal  );
    sph_master_frame_save( result, "super_flat_difference.fits",
            pl = sph_ifs_lenslet_model_get_as_propertylist(model));
    cpl_propertylist_delete(pl); pl = NULL;
    CU_ASSERT_DOUBLE_EQUAL( sph_master_frame_get_absflux(result,NULL),0.0,0.1);
    CU_ASSERT_PTR_NOT_NULL_FATAL( result );
    sph_master_frame_delete( ldff1 ); ldff1 = NULL;
    sph_master_frame_delete( ldff2 ); ldff2 = NULL;
    sph_master_frame_delete( ldff3 ); ldff3 = NULL;
    sph_master_frame_delete( ldffbb ); ldffbb = NULL;
    sph_master_frame_delete( result ); result = NULL;
    sph_master_frame_delete( ideal ); ideal = NULL;
    sph_pixel_description_table_delete(pdt); pdt = NULL;
    sph_ifs_lenslet_model_delete(model); model = NULL;
    cpl_image_delete(wim); wim = NULL;
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test MAIN function
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    int         result      = 0;
    CU_pSuite   pSuite      = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("create_super_flat", cutest_init_testsuite,
                          cutest_clean_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "Testing with wavelength flats",
                             cutest_test_create_flat ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "Testing with only three wav. flats",
                             cutest_test_create_flat_only_three ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "Testing without short FF",
                             cutest_test_create_flat_no_sdffb ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
