/* $Id: $
 *
 * This file is part of the SPHERE Pipesph_filemanager
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "sph_filemanager.h"
#include "sph_common_keywords.h"
#include <math.h>
#include "sph_error.h"
#include <Basic.h>
#include <CUnit.h>
#include "sph_test.h"
#include "sph_utils.h"
/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and
 *                                  associated functions.
 *
 */
/*----------------------------------------------------------------------------*/

/**@{*/
int cutest_init_sph_filemanager_testsuite() {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__,"moeller@mpia.de", CPL_MSG_DEBUG);
    return 0;
}

int cutest_clean_sph_filemanager_testsuite() {
    sph_error_dump( SPH_ERROR_ERROR );
    sph_end_test();
    return 0;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_filemanager_new function.
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_filemanager_new() {
    sph_filemanager*    sphfilemanager            = NULL;

    sphfilemanager = sph_filemanager_new( );

    CU_ASSERT_PTR_NOT_NULL_FATAL( sphfilemanager );
    CU_ASSERT_PTR_NOT_NULL_FATAL(sphfilemanager->workdir);
    CU_ASSERT_PTR_NOT_NULL_FATAL(sphfilemanager->tmpfiles);
    CU_ASSERT_EQUAL(sphfilemanager->ntmpfiles,0);
    CU_ASSERT_STRING_EQUAL(sphfilemanager->workdir,".");
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_filemanager_new function.
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_filemanager_get_numbered_filename() {
    sph_filemanager*    sphfilemanager            = NULL;
    char*         filename                   = NULL;

    sphfilemanager = sph_filemanager_new( );

    CU_ASSERT_PTR_NOT_NULL_FATAL( sphfilemanager );
    CU_ASSERT_PTR_NOT_NULL_FATAL(sphfilemanager->workdir);
    CU_ASSERT_PTR_NOT_NULL_FATAL(sphfilemanager->tmpfiles);

    filename = sph_filemanager_get_numbered_filename("atest.fits",1);
    CU_ASSERT_STRING_EQUAL(filename,"atest_00001.fits");
    cpl_free(filename); filename = NULL;
    filename = sph_filemanager_get_numbered_filename_new_file("atest.fits");
    CU_ASSERT_STRING_EQUAL(filename,"atest_00000.fits");
    CU_ASSERT_EQUAL(sphfilemanager->ntmpfiles,0);
    CU_ASSERT_STRING_EQUAL(sphfilemanager->workdir,".");
    cpl_free(filename); filename = NULL;
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_filemanager_new function.
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_filemanager_get_unique_duplicate_frame() {
    cpl_frame*          inframe                     = NULL;
    cpl_frame*          outframe                    = NULL;

    inframe = cpl_frame_new();
    CU_ASSERT_PTR_NOT_NULL_FATAL( inframe );
    cpl_frame_set_filename(inframe,"atest.fits");
    cpl_frame_set_tag(inframe,"ATAG");
    cpl_frame_set_group(inframe,CPL_FRAME_GROUP_PRODUCT);



    outframe = sph_filemanager_get_unique_duplicate_frame(inframe);
    CU_ASSERT_PTR_NOT_NULL_FATAL( outframe );
    CU_ASSERT_STRING_EQUAL(cpl_frame_get_filename(outframe),"atest_00000.fits");
    CU_ASSERT_STRING_EQUAL(cpl_frame_get_tag(outframe),"ATAG");
    CU_ASSERT_EQUAL(cpl_frame_get_group(outframe),CPL_FRAME_GROUP_PRODUCT);
    cpl_frame_delete(outframe); outframe = NULL;
    cpl_frame_delete(inframe); inframe = NULL;
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_filemanager_new function.
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_filemanager_remove_dir() {
    const char*        fname = "this/path/leads/to/nothing";
    const char*        fname2 = "nothing";
    char*               nname = NULL;
    CU_ASSERT_PTR_NOT_NULL_FATAL( nname = sph_filemanager_remove_dir(fname) );
    CU_ASSERT_STRING_EQUAL(nname,"nothing");
    cpl_free(nname); nname = NULL;
    CU_ASSERT_PTR_NOT_NULL_FATAL( nname = sph_filemanager_remove_dir(fname2) );
    CU_ASSERT_STRING_EQUAL(nname,"nothing");
    cpl_free(nname); nname = NULL;

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_filemanager_new function.
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_filemanager_split() {
    char        base[256];
    char        ext[256];

    CU_ASSERT_EQUAL( sph_filemanager_split("atest.fits",base,ext), CPL_ERROR_NONE );
    CU_ASSERT_STRING_EQUAL(base,"atest");
    CU_ASSERT_STRING_EQUAL(ext,"fits");
    CU_ASSERT_EQUAL( sph_filemanager_split("btest",base,ext), CPL_ERROR_NONE );
    CU_ASSERT_STRING_EQUAL(base,"btest");
    CU_ASSERT_EQUAL( sph_filemanager_split(".txt",base,ext), CPL_ERROR_NONE );
    CU_ASSERT_STRING_EQUAL(ext,"txt");

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_filemanager_new function.
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_filemanager_new_filename_from_base() {
    char        base[256];
    char        ext[256];
    char*       result = NULL;

    result = sph_filemanager_new_filename_from_base("atest.fits","hello");
    CU_ASSERT_PTR_NOT_NULL_FATAL(result);
    CU_ASSERT_STRING_EQUAL(result,"atest_hello.fits");
    cpl_free(result); result = NULL;
    result = sph_filemanager_new_filename_from_base("atest","hello");
    CU_ASSERT_PTR_NOT_NULL_FATAL(result);
    CU_ASSERT_STRING_EQUAL(result,"atest_hello");
    cpl_free(result); result = NULL;
    result = sph_filemanager_new_filename_from_base("atest.fits.txt","hello");
    CU_ASSERT_PTR_NOT_NULL_FATAL(result);
    CU_ASSERT_STRING_EQUAL(result,"atest.fits_hello.txt");
    cpl_free(result); result = NULL;
    result = sph_filemanager_new_filename_from_base(".txt","hello");
    CU_ASSERT_PTR_NOT_NULL_FATAL(result);
    CU_ASSERT_STRING_EQUAL(result,"_hello.txt");
    cpl_free(result); result = NULL;
    return;
}


/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_filemanager_delete function.
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_filemanager_delete() {
    sph_filemanager*    sphfft            = NULL;
    FILE*                fp = NULL;
    sphfft = sph_filemanager_new( );
    CU_ASSERT_PTR_NOT_NULL_FATAL( sphfft );
    fp = fopen("testdummy.txt","w");
    fprintf(fp,"Hello!\n");
    fclose(fp);fp = NULL;
    fp = fopen("testdummy.txt","r");
    CU_ASSERT_PTR_NOT_NULL_FATAL(fp);
    fclose(fp);fp = NULL;

    sph_filemanager_add_tmp_file("testdummy.txt");

    sph_filemanager_delete( 0 );
    fp = fopen("testdummy.txt","r");
    CU_ASSERT_PTR_NULL(fp);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_master_dark recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {

    int         result      = 0;
    CU_pSuite   pSuite      = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("sph_filemanager_test", cutest_init_sph_filemanager_testsuite,
                          cutest_clean_sph_filemanager_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "sph_filemanager_new",
                             cutest_sph_filemanager_new ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_filemanager_delete",
                             cutest_sph_filemanager_delete ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_filemanager_get_numbered_filename",
                             cutest_sph_filemanager_get_numbered_filename ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_filemanager_get_unique_duplicate_frame",
                             cutest_sph_filemanager_get_unique_duplicate_frame ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_filemanager_split",
                             cutest_sph_filemanager_split ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_filemanager_remove_dir",
                             cutest_sph_filemanager_remove_dir ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_filemanager_new_filename_from_base",
                             cutest_sph_filemanager_new_filename_from_base ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
