/* $Id: $
 *
 * This file is part of the SPHERE Pipesph_framecombination
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "sph_framecombination.h"
#include "sph_common_keywords.h"
#include <math.h>
#include "sph_error.h"
#include <Basic.h>
#include <CUnit.h>
#include "sph_test.h"
#include "sph_utils.h"
/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and
 *                                  associated functions.
 *
 */
/*----------------------------------------------------------------------------*/

/**@{*/
int cutest_init_sph_framecombination_testsuite() {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__,"moeller@mpia-hd.mpg.de",CPL_MSG_DEBUG);
    return 0;
}

int cutest_clean_sph_framecombination_testsuite() {
    sph_error_dump( SPH_ERROR_ERROR );
    sph_end_test();
    return 0;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_framecombination_new function.
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_framecombination_master_frame_from_frameset() {
    cpl_frameset*           inframes        = NULL;
    cpl_parameterlist*      params          = NULL;
    int                     nframes         = 3;
    int                     ii              = 0;
    cpl_image*               image           = NULL;
    sph_master_frame*       result          = NULL;

    params  = cpl_parameterlist_new();
    inframes = cpl_frameset_new();

    cpl_parameterlist_append( params,
                              cpl_parameter_new_range( "clean_mean.reject_high",
                                                       CPL_TYPE_INT,
                                                       NULL,
                                                       NULL,
                                                       0 ,
                                                       0,20
                                                      )
                            );
    cpl_parameterlist_append( params,
                              cpl_parameter_new_range( "clean_mean.reject_low",
                                                       CPL_TYPE_INT,
                                                       NULL,
                                                       NULL,
                                                       0 ,
                                                       0,20
                                                      )
                            );


    for (ii = 0; ii < nframes; ++ii) {
        image = sph_test_image_tools_create_flat_image_double(256,256,10.0);
        cpl_frameset_insert( inframes,
                sph_test_frame_image(
                        image,
                        CPL_BPP_32_SIGNED,
                        "ATAG",
                        CPL_FRAME_GROUP_RAW));
        cpl_image_delete(image);
    }
    result = sph_framecombination_master_frame_from_frameset( inframes, SPH_COLL_ALG_CLEAN_MEAN, params, 0 );
    //CU_ASSERT_PTR_NOT_NULL_FATAL(result);
    //CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_mean(result->image),10.0,0.00001);
    //printf("Mean=%f\n",cpl_image_get_mean(result->image));
    sph_master_frame_delete(result);
    cpl_parameterlist_delete(params);
    cpl_frameset_delete(inframes);
    sph_filemanager_delete(0);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_master_dark recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {

    int         result      = 0;
    CU_pSuite   pSuite      = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("sph_framecombination_test", cutest_init_sph_framecombination_testsuite,
                          cutest_clean_sph_framecombination_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "sph_framecombination_master_frame_from_frameset",
                             cutest_sph_framecombination_master_frame_from_frameset ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
