/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/*
 * cutest_sph_keyword_engine.c
 *
 *  Created on: Apr 2, 2009
 *      Author: pavlov
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "sph_keywordnode.h"

#include "sph_common_keywords.h"
#include <math.h>
#include "sph_error.h"
#include "sph_test.h"
#include <Basic.h>
#include <CUnit.h>
#include "sph_test.h"


/*
 * Create the tree with the following entries:
 * @verbatim
PIPELINE.VERSION = "0.9" \# current version of the PIPELINE
PIPELINE.ZIMPOL.CAM1.CALNAME = "DARK"
PIPELINE.ZIMPOL.CAM1.DIT = "30.5"
PIPELINE.ZIMPOL.CAM1.NDIT = "20"
PIPELINE.ZIMPOL.CAM1.CALNAME = "DARK"
PIPELINE.ZIMPOL.CAM1.DIT = "40.5"
PIPELINE.ZIMPOL.CAM1.NDIT = "50"
@endverbatim
 *
 * the following tree is created:
 *
@verbatim
ROOT
 +-- PIPELINE
      +-- VERSION = 0.9 (double)
      +-- CAM
           +-- 1
           |   +-- CALNAME = DARK (string, i.e. char*)
           |   +-- DIT = 30.5 (double)
           |   +-- NDIT = 20 (int)
           +-- 2
              +-- CALNAME = DARK (string, i.e. char*)
           |   +-- DIT = 15.25 (double)
           |   +-- NDIT = 40 (int)

@endverbatim
 *
 */

/* static variables */
//keywords
const double version = 0.9;
const char test = 'T'; // or 'F'
const char* cam1_calname = "DARK";
const double cam1_dit = 30.5;
const int cam1_ndit = 20;
const char* cam2_calname = "DARK";
const double cam2_dit = 15.25;
const int cam2_ndit = 40;

const double granularity = 1e-15;

const char cfgFile[] = "sph_fits_keywords_test.cfg";
const char cfgFileDel[] = "sph_fits_keywords_test_del.cfg";
const char cfgFileNum[] = "sph_fits_keywords_test_num.cfg";


sph_keywordnode *cutest_util_keywordnode_create_cfg(){
    sph_keywordnode *root = NULL;

    root = sph_keywordnode_new(NULL);
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);
    CU_ASSERT_EQUAL(sph_keywordnode_SetDouble(root, "PIPELINE.VERSION", version),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetChar(root, "PIPELINE.TEST", test),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetString(root, "PIPELINE.CAM1.CALNAME", cam1_calname),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetDouble(root, "PIPELINE.CAM1.DIT", cam1_dit),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "PIPELINE.CAM1.NDIT", cam1_ndit),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetString(root, "PIPELINE.CAM2.CALNAME", cam2_calname),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetDouble(root, "PIPELINE.CAM2.DIT", cam2_dit),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "PIPELINE.CAM2.NDIT", cam2_ndit),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "CAMERA1.DIT1", cam1_dit),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "PIPELINE.CAMERA1.CAMERA2.DIT2.CAMERA2", cam2_dit),1);


    CU_ASSERT_EQUAL(sph_keywordnode_SaveConfiguration(root, cfgFile),1); // save keywords configuration on the disk
    return root;
}

sph_keywordnode *cutest_util_keywordnode_create_cfg_start_by_number(){
    sph_keywordnode *root = NULL;

    root = sph_keywordnode_new(NULL);
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);
    CU_ASSERT_EQUAL(sph_keywordnode_SetDouble(root, "PIPELINE.VERSION", version),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetChar(root, "PIPELINE.TEST", test),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetString(root, "1PIPELINE.CAM1.CALNAME", cam1_calname),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetDouble(root, "1PIPELINE.CAM1.DIT", cam1_dit),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "1PIPELINE.CAM1.NDIT", cam1_ndit),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetString(root, "2PIPELINE.CAM2.CALNAME", cam2_calname),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetDouble(root, "2PIPELINE.CAM2.DIT", cam2_dit),1);
    CU_ASSERT_EQUAL(sph_keywordnode_SetInt(root, "2PIPELINE.CAM2.NDIT", cam2_ndit),1);

    CU_ASSERT_EQUAL(sph_keywordnode_SaveConfiguration(root, cfgFileNum),1); // save keywords configuration on the disk
    return root;
}


int cutest_init_keywordnode_testsuite() {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
  //sph_init();
  sph_init_test(__FILE__,"pavlov@mpia.de",CPL_MSG_DEBUG);
  return 0;
}

int cutest_clean_keywordnode_testsuite() {
    sph_end_test();
    return 0;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_keywordnode_ function.
 */
/*----------------------------------------------------------------------------*/
void cutest_keywordnode_LoadConfiguration() {
    sph_keywordnode *root = NULL;
    root = cutest_util_keywordnode_create_cfg();
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);
    sph_keywordnode_destroy(root); root = NULL;
    root = sph_keywordnode_load_configuration(cfgFile);
    CU_ASSERT_PTR_NOT_NULL_FATAL( root );
    sph_error_raise(SPH_ERROR_GENERAL,
            __FILE__, __func__, __LINE__, SPH_ERROR_INFO, " - cutest_keywordnode_LoadConfiguration -");
    //printf(" - cutest_keywordnode_LoadConfiguration:");
    sph_keywordnode_destroy( root );
}

/*
 * this function is just to test whether keyword with is saved properly
 */
void cutest_keywordnode_keywordnode_save_number(){
    sph_keywordnode *root = NULL;
     root = cutest_util_keywordnode_create_cfg_start_by_number();
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);
    sph_keywordnode_destroy( root );
}


void cutest_keywordnode_keywordnode_destroy(){
    sph_keywordnode *root = NULL;
     root = cutest_util_keywordnode_create_cfg();
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);

    sph_keywordnode *node;
    double value;

    sph_error_raise(SPH_ERROR_GENERAL,
            __FILE__, __func__, __LINE__, SPH_ERROR_INFO, " - cutest_keywordnode_keywordnode_destroy -");
    //printf(" - cutest_keywordnode_keywordnode_destroy: \n");

    if ((node = sph_keywordnode_get_keywordnode_by_name(root,"PIPELINE.VERSION")) != NULL){
        //printf(" -- Before node-destroy of PIPELINE.VERSION keywordnode:\n");

        CU_ASSERT_EQUAL(sph_keywordnode_GetDouble(root,"PIPELINE.VERSION", &value), 1);
        CU_ASSERT_DOUBLE_EQUAL(value, version, granularity);
        //printf(" -- PIPELINE.VERSION = %f \n", value);
        sph_keywordnode_destroy(node);
     } else {
         //printf("-- PIPELINE.VERSION keyword has not been created, please verify \n.");
         sph_error_raise(SPH_ERROR_GENERAL,
                 __FILE__, __func__, __LINE__, SPH_ERROR_ERROR, " -- PIPELINE.VERSION keyword has not been created, please verify --");
     }
    CU_ASSERT_EQUAL(sph_keywordnode_GetDouble(root,"PIPELINE.VERSION", &value), 0);
    CU_ASSERT_EQUAL(sph_keywordnode_SaveConfiguration(root, cfgFileDel),1); // look at this file (PIPELINE.VERSION must be erased!)
    sph_keywordnode_destroy( root );
}


void cutest_keywordnode_GetInt() {
    int value;
    sph_keywordnode *root = NULL;
    root = cutest_util_keywordnode_create_cfg();
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);
    CU_ASSERT_EQUAL(sph_keywordnode_GetInt(root,"PIPELINE.CAM1.NDIT", &value), 1);
    CU_ASSERT_EQUAL(value, cam1_ndit);
    //printf(" - cutest_keywordnode_GetInt:");
    //printf(" -- PIPELINE.CAM1.NDIT = %i \n", value);
    sph_keywordnode_destroy( root );
}

void cutest_keywordnode_GetDouble() {
    double value;
    sph_keywordnode *root = NULL;
    root = cutest_util_keywordnode_create_cfg();
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);
    CU_ASSERT_EQUAL(sph_keywordnode_GetDouble(root,"PIPELINE.VERSION", &value), 1);
    CU_ASSERT_DOUBLE_EQUAL(value, version, granularity);
    //printf(" - cutest_keywordnode_GetDouble:");
    //printf(" -- PIPELINE.VERSION = %f \n", value);
    sph_keywordnode_destroy( root );
}

void cutest_keywordnode_GetString() {
    char string[100];
    char qualname[100], qualname_list[100];
    sph_keywordnode *root = NULL;
    sph_keywordnode *node;
    cpl_propertylist* proplist;
    int k;

    root = cutest_util_keywordnode_create_cfg();
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);
    CU_ASSERT_EQUAL(sph_keywordnode_GetString(root,"PIPELINE.CAM1.CALNAME", string, sizeof(string)), 1);
    CU_ASSERT_STRING_EQUAL(string, cam1_calname);
    //printf(" -- PIPELINE.CAM1.CALNAME = %s \n", string);
    //printf(" -- Set Double instead of String \n");
    sph_keywordnode_SetInt(root,"PIPELINE.CAM1.CALNAME", 10);
    sph_keywordnode_GetInt(root, "PIPELINE.CAM1.CALNAME", &k);
    //printf(" -- Get New Integer : PIPELINE.CAM1.CALNAME = %d", k);


    //node = GetKeywordNodeByName(root, "PIPELINE.CAMERA1.CAMERA2.DIT2.CAMERA2");
    node = sph_keywordnode_get_keywordnode_by_name(root, "PIPELINE.CAM1.CALNAME");
    sph_keywordnode_BuildQualifiedName(node, qualname, 100);
    //printf("-- ### BuildQualifiedName: %s ###\n\n", qualname);
    proplist = cpl_propertylist_new();
    
    //node = GetKeywordNodeByName(root, "PIPELINE.CAM1.CALNAME");
    //BuildQualifiedName(node, qualname_list, 100);
    sph_keywordnode_BuildQualifiedNameList(node, qualname_list, 100, proplist);
    //printf("-- ### BuildQualifiedNameList: %s ###\n", qualname_list);
    //printf("-- ### starting loop for the list...\n");
    //for (i=0; i < cpl_propertylist_get_size(proplist);i++){
        //cpl_property* p = cpl_propertylist_get(proplist,i);
        //printf("Prop %d: %s = %d \n", i, cpl_property_get_name(p), cpl_property_get_int(p));
    //}
    sph_keywordnode_destroy( root );
    cpl_propertylist_delete( proplist ); proplist = NULL;

}

void cutest_keywordnode_GetChar() {
    char s_char;
    sph_keywordnode *root = NULL;
    root = cutest_util_keywordnode_create_cfg();
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);
    CU_ASSERT_EQUAL(sph_keywordnode_GetChar(root,"PIPELINE.TEST", &s_char), 1);
    CU_ASSERT(s_char == test);
    //printf(" - cutest_keywordnode_GetChar:");
    //printf(" -- PIPELINE.TEST = %c \n", s_char);
    sph_keywordnode_destroy( root );
}

void cutest_keywordnode_GetType() {
     sph_keywordnode *root = NULL;
     keyword_TYPE type, type1;
    root = cutest_util_keywordnode_create_cfg();
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);
    type = sph_keywordnode_GetType(sph_keywordnode_LocateByName(root,"PIPELINE.VERSION", 0));
    CU_ASSERT(type == keyword_DOUBLE);
    type1 = sph_keywordnode_GetType(sph_keywordnode_get_keywordnode_by_name(root, "PIPELINE.CAM1.CALNAME"));
    CU_ASSERT(type1 == keyword_STRING);
    //printf(" - cutest_keywordnode_GetType:");
    sph_keywordnode_destroy( root );
}

void cutest_keywordnode_QueryVerify(){
    sph_keywordnode *root = NULL;
     root = cutest_util_keywordnode_create_cfg();
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);

    // Query template with verifications starts here
    sph_keywordnode *node;
    keyword_TYPE type;
    double value;
    //printf(" - cutest_keywordnode_QueryVerify: \n");
    sph_error_raise(SPH_ERROR_GENERAL,
                __FILE__, __func__, __LINE__, SPH_ERROR_INFO, " -  cutest_keywordnode_QueryVerify -");

    if ((node = sph_keywordnode_get_keywordnode_by_name(root,"PIPELINE.VERSION")) != NULL){
        if ( (type = sph_keywordnode_GetType(node)) == keyword_DOUBLE) {
           if (sph_keywordnode_GetDouble(root, "PIPELINE.VERSION", &value)) {
              // printf(" -- PIPELINE.VERSION = %f \n", value);
               CU_ASSERT_DOUBLE_EQUAL(value, version, granularity);
           }
        } else {
           //printf(" -- PIPELINE.VERSION is not a type of keyword_DOUBLE \n");
           sph_error_raise(SPH_ERROR_GENERAL,
                    __FILE__, __func__, __LINE__, SPH_ERROR_ERROR, " -- PIPELINE.VERSION is not a type of keyword_DOUBLE --");
        }
    } else {
         //printf(" -- PIPELINE.VERSION keyword doesn't exist! \n");
         sph_error_raise(SPH_ERROR_GENERAL,
                   __FILE__, __func__, __LINE__, SPH_ERROR_ERROR, " -- PIPELINE.VERSION  keyword doesn't exist! --");

    }
    //Query template with verifications ends here

    // CU unit verifications
    CU_ASSERT_PTR_NOT_NULL_FATAL(node);
    CU_ASSERT_DOUBLE_EQUAL(value, version, granularity);
    CU_ASSERT(type == keyword_DOUBLE);
    sph_keywordnode_destroy( root );
}


void cutest_keywordnode_ChildQueries(){
    sph_keywordnode *root = NULL;
     root = cutest_util_keywordnode_create_cfg();
    CU_ASSERT_PTR_NOT_NULL_FATAL(root);

    //Query template for the multiple childs starts here
    sph_keywordnode *node;
    //printf(" - cutest_keywordnode_ChildQueries: \n");
    sph_error_raise(SPH_ERROR_GENERAL,
            __FILE__, __func__, __LINE__, SPH_ERROR_INFO, " - cutest_keywordnode_ChildQueries -");

    if ((node = sph_keywordnode_get_keywordnode_by_name(root,"PIPELINE.CAM")) != NULL){
        int i = 0; // this line is only for cunit purpose, don't use in the template
        for (node = sph_keywordnode_GetFirstChild(node); node != NULL; node = sph_keywordnode_GetNext(node)) {
            double dit;
            i++; // this line is only for the cunit purpose,  don't use in the template
            if (sph_keywordnode_GetDouble(node, "DIT", &dit)) {
                //printf("DIT for camera %d = %f\n", GetNr(node), dit);
                CU_ASSERT_EQUAL(i,sph_keywordnode_GetNr(node)); // this line is only for the cunit purpose, don't use in the template
                if (sph_keywordnode_GetNr(node) == 1) {
                    CU_ASSERT_EQUAL(dit, cam1_dit);
                }
                if (sph_keywordnode_GetNr(node) == 2) {
                    CU_ASSERT_EQUAL(dit, cam2_dit);
                }
            } else {
                //printf("DIT for camera %d unknown\n", sph_keywordnode_GetNr(node));
                sph_error_raise(SPH_ERROR_GENERAL,
                        __FILE__, __func__, __LINE__, SPH_ERROR_ERROR, " -- DIT for camera %d unknown: ", sph_keywordnode_GetNr(node));
            }

        }
    }
    //Query template for the multiple childs ends here
    sph_keywordnode_destroy( root );
}


/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of sph_keywordnode and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    int         result  = 0;
    CU_pSuite   pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("keywordnode_test", cutest_init_keywordnode_testsuite,
                          cutest_clean_keywordnode_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */

    if ( NULL == CU_add_test(pSuite, "cutest_keywordnode_LoadConfiguration",  cutest_keywordnode_LoadConfiguration))
    {
       CU_cleanup_registry();
       return CU_get_error();
    }


    if ( NULL == CU_add_test(pSuite, "cutest_keywordnode_GetDouble",  cutest_keywordnode_GetDouble ))
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    if ( NULL == CU_add_test(pSuite, "cutest_keywordnode_GetInt",  cutest_keywordnode_GetInt ))
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    if ( NULL == CU_add_test(pSuite, "cutest_keywordnode_GetString",  cutest_keywordnode_GetString ))
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    if ( NULL == CU_add_test(pSuite, "cutest_keywordnode_GetChar",  cutest_keywordnode_GetChar ))
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    if ( NULL == CU_add_test(pSuite, "cutest_keywordnode_GetType",  cutest_keywordnode_GetType ))
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    if ( NULL == CU_add_test(pSuite, "cutest_keywordnode_QueryVerify",  cutest_keywordnode_QueryVerify ))
      {
         CU_cleanup_registry();
         return CU_get_error();
      }

    if ( NULL == CU_add_test(pSuite, "cutest_keywordnode_ChildQueries",  cutest_keywordnode_ChildQueries))
      {
         CU_cleanup_registry();
         return CU_get_error();
      }


    if ( NULL == CU_add_test(pSuite, "cutest_keywordnode_keywordnode_destroy",  cutest_keywordnode_keywordnode_destroy))
       {
          CU_cleanup_registry();
          return CU_get_error();
       }

    if ( NULL == CU_add_test(pSuite, "cutest_keywordnode_keywordnode_save_number",  cutest_keywordnode_keywordnode_save_number))
       {
          CU_cleanup_registry();
          return CU_get_error();
       }



    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}


