/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>
#include <math.h>
#include <Basic.h>
#include <CUnit.h>
#include <gsl/gsl_rng.h>

#include "sph_master_frame.h"
#include "sph_common_keywords.h"
#include "sph_error.h"
#include "sph_test.h"
#include "sph_utils.h"
#include "sph_test_image_tools.h"
/*----------------------------------------------------------------------------*/
/**
 * @defgroup cutest_sph_master_frame Unit test for the sph_master_frame module            
 *
 */
/*----------------------------------------------------------------------------*/

/**@{*/

int cutest_init_sph_master_frame_testsuite() {
    sph_init_test(__FILE__,"moeller@mpia-hd.mpg.de",CPL_MSG_DEBUG);
    return 0;
}

int cutest_clean_sph_master_frame_testsuite() {
    /*--------------------------------------------------------------------
     * -    Close and dump errors.
     * -------------------------------------------------------------------*/
    sph_error_dump( SPH_ERROR_INFO );
    sph_end_test();
    return 0;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_new function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_new() {
    CU_FAIL( "Not yet implemented!");
    return;   
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_new_collapse function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_new_collapse() {
    CU_FAIL( "Not yet implemented!");
    return;   
}


/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_new_collapse function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_new_empty() {
    CU_FAIL( "Not yet implemented!");
    return;   
}



/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_collapse function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_collapse() {
    CU_FAIL( "Not yet implemented!");
    return;   
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_collapse_cleanmean function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_collapse_cleanmean() {
    CU_FAIL( "Not yet implemented!");
    return;   
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_collapse function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_collapse_get_params_clean_mean() {
    CU_FAIL( "Not yet implemented!");
    return;   
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_mask_sigma function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_mask_sigma() {
    CU_FAIL( "Not yet implemented!");
    return;   
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_quality_check function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_quality_check() {
    CU_FAIL( "Not yet implemented!");
    return;   
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_rebin function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_rebin() {
    CU_FAIL( "Not yet implemented!");
    return;   
}


/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_resample function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_resample() {
    CU_FAIL( "Not yet implemented!");
    return;   
}


/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_save_dfs function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_save_dfs() {
    CU_FAIL( "Not yet implemented!");
    return;   
}


/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_set_coll_alg function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_set_coll_alg() {
    CU_FAIL( "Not yet implemented!");
    return;   
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_sn_map function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_sn_map() {
    CU_FAIL( "Not yet implemented!");
    return;   
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_delete function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_delete() {
    return;   
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_delete function.
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_inverse() {
    cpl_image*          im = NULL;
    sph_master_frame*   mframe  = NULL;
    sph_master_frame*   result  = NULL;
    cpl_image*          resim = NULL;

    im = sph_test_image_tools_create_flat_image_double(256,256,2.0);
    mframe = sph_master_frame_new_from_cpl_image(im);
    result = sph_master_frame_inverse(mframe);
    resim = sph_master_frame_get_image(result,1);
    CU_ASSERT_EQUAL(cpl_error_get_code(), CPL_ERROR_NONE);
    CU_ASSERT_PTR_NOT_NULL_FATAL(resim);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_max(resim),0.5,0.0001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_min(resim),0.5,0.0001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_mean(resim),0.5,0.0001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_median(resim),0.5,0.0001);
    cpl_image_delete(resim); resim = NULL;
    sph_master_frame_delete(result); result = NULL;
    cpl_image_set(mframe->image,100,100,0.0);
    result = sph_master_frame_inverse(mframe);
    resim = sph_master_frame_get_image(result,1);
    CU_ASSERT_PTR_NOT_NULL_FATAL(resim);
    CU_ASSERT_EQUAL(cpl_error_get_code(), CPL_ERROR_NONE);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_max(resim),0.5,0.0001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_min(resim),0.5,0.0001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_mean(resim),0.5,0.0001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_median(resim),0.5,0.0001);
    CU_ASSERT_EQUAL(cpl_mask_count(cpl_image_get_bpm_const(resim)),1);
    cpl_image_delete(resim); resim = NULL;
    sph_master_frame_delete(result); result = NULL;

    cpl_image_delete(im);
    sph_master_frame_delete(mframe);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_delete function.
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_interpolate_bpix() {
    cpl_image*          im = NULL;
    sph_master_frame*   mframe  = NULL;
    sph_master_frame*   result  = NULL;
    cpl_image*          resim = NULL;
    gsl_rng*            pRNG = NULL;

    CU_ASSERT_EQUAL(cpl_error_get_code(),CPL_ERROR_NONE);
    pRNG = gsl_rng_alloc(gsl_rng_taus);
    im = sph_test_image_tools_create_flat_image_double(256,256,50.0);
    sph_test_image_tools_apply_poisson_noise(im,pRNG);
    sph_test_image_tools_add_hotpixels(im,100,100000.0,pRNG);
    mframe = sph_master_frame_new_from_cpl_image(im);
    sph_master_frame_quality_check(mframe);
    sph_master_frame_mask_sigma(mframe,5.0);
    CU_ASSERT_EQUAL(sph_master_frame_get_nbads(mframe),100);
    sph_master_frame_interpolate_bpix(mframe);
    CU_ASSERT_EQUAL(sph_master_frame_get_nbads(mframe),0);
    CU_ASSERT_EQUAL(cpl_error_get_code(), CPL_ERROR_NONE);
    sph_master_frame_delete(mframe);
    cpl_image_delete(im);
    gsl_rng_free(pRNG);
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_delete function.
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_smooth() {
    cpl_image*          im = NULL;
    cpl_image*          im2 = NULL;
    sph_master_frame*   mframe  = NULL;
    sph_master_frame*   result  = NULL;
    cpl_image*          resim = NULL;
    double              flux0 = 0.0;
    double              flux = 0.0;
    CU_ASSERT_EQUAL(cpl_error_get_code(),CPL_ERROR_NONE);
    im = sph_test_image_tools_create_flat_image_double(256,256,0.0);
    sph_test_image_tools_add_gauss(im,128.0,128.0,1.0,1.0);
    sph_test_image_tools_add_gauss(im,198.0,198.0,1.0,1.0);
    flux0 = cpl_image_get_absflux(im);
    im2 = sph_test_image_tools_create_flat_image_double(256,256,0.0);
    sph_test_image_tools_add_gauss(im2,128.0,128.0,15.0/2.3548,1.0);
    sph_test_image_tools_add_gauss(im2,198.0,198.0,15.0/2.3548,1.0);
    cpl_image_multiply_scalar(im2,flux0/cpl_image_get_absflux(im2));
    mframe = sph_master_frame_new_from_cpl_image(im);
    CU_ASSERT_EQUAL(sph_master_frame_get_nbads(mframe),0);
    sph_master_frame_smooth(mframe,15.0);
    CU_ASSERT_EQUAL(sph_master_frame_get_nbads(mframe),0);
    flux = cpl_image_get_absflux(mframe->image);
    CU_ASSERT_EQUAL(cpl_error_get_code(), CPL_ERROR_NONE);
    CU_ASSERT_DOUBLE_EQUAL(flux0,flux,0.000001);
    CU_ASSERT_DOUBLE_EQUAL(flux0,cpl_image_get_flux(im2),0.000001);
    sph_master_frame_save(mframe,"smoothed.fits",NULL);
    cpl_image_save(im2,"smoothed.fits",CPL_TYPE_DOUBLE,NULL,CPL_IO_EXTEND);
    cpl_image_subtract(im2,mframe->image);
    cpl_image_save(im2,"smoothed.fits",CPL_TYPE_DOUBLE,NULL,CPL_IO_EXTEND);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_absflux(im2),0.0,0.1);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_max(im2),0.0,0.0001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_min(im2),0.0,0.0001);
    cpl_image_delete(im);
    sph_master_frame_delete(mframe);
    cpl_image_delete(im2);
    return;
}


/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_master_frame_delete function.
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_master_frame_fit_gauss_hist() {
    cpl_image*          im = NULL;
    gsl_rng*            pRNG    = NULL;
    sph_master_frame*   mframe  = NULL;
    cpl_vector*         binpos = NULL;
    cpl_vector*         binvals = NULL;
    double              mean = 0.0;
    double              rms  = 0.0;
    double              red  = 0.0;

    pRNG = gsl_rng_alloc(gsl_rng_taus);
    im = sph_test_image_tools_create_flat_image_double(256,256,0.0);
    sph_test_image_tools_add_noise(im,100.0,pRNG);

    binvals = cpl_vector_new(100);
    cpl_vector_fill(binvals,0.0);
    binpos = cpl_vector_new(100);

    mframe = sph_master_frame_new_from_cpl_image(im);
    sph_master_frame_fit_gauss_hist(mframe, binpos, binvals, -200.0,200.0,&mean,&rms,&red);

    gsl_rng_free(pRNG);

    CU_ASSERT_EQUAL(cpl_error_get_code(), CPL_ERROR_NONE);
    SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL,"Mean: %f +/- %f with certainty: %f", mean, rms, red);
    CU_ASSERT_DOUBLE_EQUAL( mean, 0.0, 1.0 );
    CU_ASSERT_DOUBLE_EQUAL( rms, 100.0, 10.0 );
    CU_ASSERT_DOUBLE_EQUAL( red, 1.0, 1.0 );
    cpl_vector_delete(binpos);
    cpl_vector_delete(binvals);
    cpl_image_delete(im);
    sph_master_frame_delete(mframe);
    return;
}




/*----------------------------------------------------------------------------*/
/**
 @brief   Main to run the tests.
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    int         result  = 0;
    CU_pSuite   pSuite  = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("cutest_sph_master_frame", cutest_init_sph_master_frame_testsuite, 
                          cutest_clean_sph_master_frame_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "sph_master_frame_fit_gauss_hist",
                             cutest_sph_master_frame_fit_gauss_hist ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_master_frame_inverse",
                             cutest_sph_master_frame_inverse ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_master_frame_interpolate_bpix",
                             cutest_sph_master_frame_interpolate_bpix ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_master_frame_smooth",
                             cutest_sph_master_frame_smooth ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_master_frame_delete", 
                             cutest_sph_master_frame_delete ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
